@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2022 - pancake, nibble */
 
 #include <r_anal.h>
 #include <r_util.h>
@@ -77,6 +77,7 @@ static void r_meta_item_free(void *_item) {
 	}
 }
 
+// Take nullable RArchConfig as argument?
 R_API RAnal *r_anal_new(void) {
 	int i;
 	RAnal *anal = R_NEW0 (RAnal);
@@ -90,13 +91,13 @@ R_API RAnal *r_anal_new(void) {
 	anal->bb_tree = NULL;
 	anal->ht_addr_fun = ht_up_new0 ();
 	anal->ht_name_fun = ht_pp_new0 ();
-	anal->os = strdup (R_SYS_OS);
+	anal->config = r_arch_config_new ();
 	anal->esil_goto_limit = R_ANAL_ESIL_GOTO_LIMIT;
 	anal->opt.nopskip = true; // skip nops in code analysis
 	anal->opt.hpskip = false; // skip `mov reg,reg` and `lea reg,[reg]`
 	anal->gp = 0LL;
 	anal->sdb = sdb_new0 ();
-	anal->cpp_abi = R_ANAL_CPP_ABI_ITANIUM;
+	anal->cxxabi = R_ANAL_CPP_ABI_ITANIUM;
 	anal->opt.depth = 32;
 	anal->opt.noncode = false; // do not analyze data by default
 	r_spaces_init (&anal->meta_spaces, "CS");
@@ -117,6 +118,7 @@ R_API RAnal *r_anal_new(void) {
 	anal->sdb_classes_attrs = sdb_ns (anal->sdb_classes, "attrs", 1);
 	anal->zign_path = strdup ("");
 	anal->cb_printf = (PrintfCallback) printf;
+	anal->esil = NULL; // nul on purpose, otherwise many analysisi fail O_O
 	(void)r_anal_pin_init (anal);
 	(void)r_anal_xrefs_init (anal);
 	anal->diff_thbb = R_ANAL_THRESHOLDBB;
@@ -131,17 +133,18 @@ R_API RAnal *r_anal_new(void) {
 	anal->fcns = r_list_newf ((RListFree)r_anal_function_free);
 	anal->leaddrs = NULL;
 	anal->imports = r_list_newf (free);
-	r_anal_set_bits (anal, 32);
+/// 	r_anal_set_bits (anal, 32);
 	anal->plugins = r_list_newf ((RListFree) r_anal_plugin_free);
 	if (anal->plugins) {
 		for (i = 0; anal_static_plugins[i]; i++) {
 			r_anal_add (anal, anal_static_plugins[i]);
 		}
 	}
+	R_DIRTY (anal);
 	return anal;
 }
 
-R_API void r_anal_plugin_free (RAnalPlugin *p) {
+R_API void r_anal_plugin_free(RAnalPlugin *p) {
 	if (p && p->fini) {
 		p->fini (NULL);
 	}
@@ -149,19 +152,19 @@ R_API void r_anal_plugin_free (RAnalPlugin *p) {
 
 void __block_free_rb(RBNode *node, void *user);
 
-R_API RAnal *r_anal_free(RAnal *a) {
+R_API void r_anal_free(RAnal *a) {
 	if (!a) {
-		return NULL;
+		return;
 	}
 	/* TODO: Free anals here */
+	free (a->pincmd);
 	r_list_free (a->fcns);
 	ht_up_free (a->ht_addr_fun);
 	ht_pp_free (a->ht_name_fun);
 	set_u_free (a->visited);
 	r_anal_hint_storage_fini (a);
 	r_interval_tree_fini (&a->meta);
-	free (a->cpu);
-	free (a->os);
+	r_unref (a->config);
 	free (a->zign_path);
 	r_list_free (a->plugins);
 	r_rbtree_free (a->bb_tree, __block_free_rb, NULL);
@@ -182,7 +185,6 @@ R_API RAnal *r_anal_free(RAnal *a) {
 	r_list_free (a->imports);
 	r_str_constpool_fini (&a->constpool);
 	free (a);
-	return NULL;
 }
 
 R_API void r_anal_set_user_ptr(RAnal *anal, void *user) {
@@ -213,10 +215,16 @@ R_API int r_anal_add(RAnal *anal, RAnalPlugin *foo) {
 	return true;
 }
 
+R_API char *r_anal_mnemonics(RAnal *anal, int id, bool json) {
+	if (anal->cur && anal->cur->mnemonics) {
+		return anal->cur->mnemonics (anal, id, json);
+	}
+	return NULL;
+}
+
 R_API bool r_anal_use(RAnal *anal, const char *name) {
 	RListIter *it;
 	RAnalPlugin *h;
-
 	if (anal) {
 		r_list_foreach (anal->plugins, it, h) {
 			if (!h->name || strcmp (h->name, name)) {
@@ -229,7 +237,8 @@ R_API bool r_anal_use(RAnal *anal, const char *name) {
 			}
 #endif
 			anal->cur = h;
-			r_anal_set_reg_profile (anal);
+			r_arch_use (anal->config, h->arch);
+			r_anal_set_reg_profile (anal, NULL);
 			return true;
 		}
 	}
@@ -242,7 +251,10 @@ R_API char *r_anal_get_reg_profile(RAnal *anal) {
 }
 
 // deprecate.. or at least reuse get_reg_profile...
-R_API bool r_anal_set_reg_profile(RAnal *anal) {
+R_API bool r_anal_set_reg_profile(RAnal *anal, const char *p) {
+	if (p) {
+		return r_reg_set_profile_string (anal->reg, p);
+	}
 	bool ret = false;
 	if (anal && anal->cur && anal->cur->set_reg_profile) {
 		ret = anal->cur->set_reg_profile (anal);
@@ -266,10 +278,10 @@ R_API bool r_anal_set_triplet(RAnal *anal, const char *os, const char *arch, int
 		arch = anal->cur? anal->cur->arch: R_SYS_ARCH;
 	}
 	if (bits < 1) {
-		bits = anal->bits;
+		bits = anal->config->bits;
 	}
-	free (anal->os);
-	anal->os = strdup (os);
+	free (anal->config->os);
+	anal->config->os = strdup (os);
 	r_anal_set_bits (anal, bits);
 	return r_anal_use (anal, arch);
 }
@@ -285,45 +297,34 @@ static void sdb_concat_by_path(Sdb *s, const char *path) {
 R_API bool r_anal_set_os(RAnal *anal, const char *os) {
 	Sdb *types = anal->sdb_types;
 	const char *dir_prefix = r_sys_prefix (NULL);
-	const char *dbpath = sdb_fmt (R_JOIN_3_PATHS ("%s", R2_SDB_FCNSIGN, "types-%s.sdb"),
-		dir_prefix, os);
+	SdbGperf *gp = r_anal_get_gperf_types (os);
+	if (gp) {
+		Sdb *gd = sdb_new0 ();
+		sdb_open_gperf (gd, gp);
+		sdb_reset (anal->sdb_types);
+		sdb_merge (anal->sdb_types, gd);
+		sdb_close (gd);
+		sdb_free (gd);
+		return r_anal_set_triplet (anal, os, NULL, -1);
+	}
+
+	// char *ff = r_str_newf ("types-%s.sdb", os);
+	// char *dbpath = r_file_new (dir_prefix, r2_sdb_fcnsign, ff);
+	char *dbpath = r_str_newf ("%s/%s/types-%s.sdb", dir_prefix, R2_SDB_FCNSIGN, os);
 	if (r_file_exists (dbpath)) {
 		sdb_concat_by_path (types, dbpath);
 	}
+	free (dbpath);
 	return r_anal_set_triplet (anal, os, NULL, -1);
 }
 
 R_API bool r_anal_set_bits(RAnal *anal, int bits) {
-	switch (bits) {
-	case 8:
-	case 16:
-	case 27:
-	case 32:
-	case 64:
-		if (anal->bits != bits) {
-			anal->bits = bits;
-			r_anal_set_reg_profile (anal);
-		}
-		return true;
-	}
-	return false;
-}
-
-R_API void r_anal_set_cpu(RAnal *anal, const char *cpu) {
-	free (anal->cpu);
-	anal->cpu = cpu ? strdup (cpu) : NULL;
-	int v = r_anal_archinfo (anal, R_ANAL_ARCHINFO_ALIGN);
-	if (v != -1) {
-		anal->pcalign = v;
-	}
-}
-
-R_API void r_anal_set_big_endian(RAnal *anal, int bigend) {
-	r_return_if_fail (anal);
-	anal->big_endian = bigend;
-	if (anal->reg) {
-		anal->reg->big_endian = bigend;
+	int obits = anal->config->bits;
+	r_arch_set_bits (anal->config, bits);
+	if (bits != obits) {
+		r_anal_set_reg_profile (anal, NULL);
 	}
+	return true;
 }
 
 R_API ut8 *r_anal_mask(RAnal *anal, int size, const ut8 *data, ut64 at) {
@@ -359,10 +360,10 @@ R_API ut8 *r_anal_mask(RAnal *anal, int size, const ut8 *data, ut64 at) {
 		}
 		idx += oplen;
 		at += oplen;
+		R_FREE (op->mnemonic);
 	}
 
 	r_anal_op_free (op);
-
 	return ret;
 }
 
@@ -379,9 +380,10 @@ R_API void r_anal_trace_bb(RAnal *anal, ut64 addr) {
 			}
 		}
 	}
+	R_DIRTY (anal);
 }
 
-R_API RList* r_anal_get_fcns (RAnal *anal) {
+R_API RList* r_anal_get_fcns(RAnal *anal) {
 	// avoid received to free this thing
 	anal->fcns->free = NULL;
 	return anal->fcns;
@@ -443,6 +445,7 @@ R_API int r_anal_archinfo(RAnal *anal, int query) {
 	switch (query) {
 	case R_ANAL_ARCHINFO_MIN_OP_SIZE:
 	case R_ANAL_ARCHINFO_MAX_OP_SIZE:
+	case R_ANAL_ARCHINFO_INV_OP_SIZE:
 	case R_ANAL_ARCHINFO_ALIGN:
 		if (anal->cur && anal->cur->archinfo) {
 			return anal->cur->archinfo (anal, query);
@@ -452,13 +455,19 @@ R_API int r_anal_archinfo(RAnal *anal, int query) {
 	return -1;
 }
 
+R_API bool r_anal_is_aligned(RAnal *anal, const ut64 addr) {
+	const int align = r_anal_archinfo (anal, R_ANAL_ARCHINFO_ALIGN);
+	return align <= 1 || !(addr % align);
+}
+
 static bool __nonreturn_print_commands(void *p, const char *k, const char *v) {
 	RAnal *anal = (RAnal *)p;
 	if (!strncmp (v, "func", strlen ("func") + 1)) {
-		char *query = sdb_fmt ("func.%s.noreturn", k);
+		char *query = r_str_newf ("func.%s.noreturn", k);
 		if (sdb_bool_get (anal->sdb_types, query, NULL)) {
 			anal->cb_printf ("tnn %s\n", k);
 		}
+		free (query);
 	}
 	if (!strncmp (k, "addr.", 5)) {
 		anal->cb_printf ("tna 0x%s %s\n", k + 5, v);
@@ -505,10 +514,11 @@ R_API void r_anal_noreturn_list(RAnal *anal, int mode) {
 	}
 }
 
-#define K_NORET_ADDR(x) sdb_fmt ("addr.%"PFMT64x".noreturn", x)
-#define K_NORET_FUNC(x) sdb_fmt ("func.%s.noreturn", x)
+#define K_NORET_ADDR(x) r_strf ("addr.%"PFMT64x".noreturn", x)
+#define K_NORET_FUNC(x) r_strf ("func.%s.noreturn", x)
 
 R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
+	r_strf_buffer (128);
 	const char *tmp_name = NULL;
 	Sdb *TDB = anal->sdb_types;
 	char *fnl_name = NULL;
@@ -532,7 +542,10 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 		}
 		tmp_name = fcn ? fcn->name: fi->name;
 		if (fcn) {
-			fcn->is_noreturn = true;
+			if (!fcn->is_noreturn) {
+  				fcn->is_noreturn = true;
+				R_DIRTY (anal);
+			}
 		}
 	}
 	if (r_type_func_exist (TDB, tmp_name)) {
@@ -557,6 +570,7 @@ R_API bool r_anal_noreturn_add(RAnal *anal, const char *name, ut64 addr) {
 }
 
 R_API bool r_anal_noreturn_drop(RAnal *anal, const char *expr) {
+	r_strf_buffer (64);
 	Sdb *TDB = anal->sdb_types;
 	expr = r_str_trim_head_ro (expr);
 	const char *fcnname = NULL;
@@ -584,13 +598,14 @@ R_API bool r_anal_noreturn_drop(RAnal *anal, const char *expr) {
 		free (tmp);
 		return true;
 	}
-	eprintf ("Can't find prototype for %s in types database", fcnname);
+	eprintf ("Can't find prototype for %s in types database\n", fcnname);
 #endif
 	return false;
 }
 
 static bool r_anal_noreturn_at_name(RAnal *anal, const char *name) {
-	if (sdb_bool_get (anal->sdb_types, K_NORET_FUNC(name), NULL)) {
+	r_strf_buffer (128);
+	if (sdb_bool_get (anal->sdb_types, K_NORET_FUNC (name), NULL)) {
 		return true;
 	}
 	char *tmp = r_type_func_guess (anal->sdb_types, (char *)name);
@@ -608,6 +623,7 @@ static bool r_anal_noreturn_at_name(RAnal *anal, const char *name) {
 }
 
 R_API bool r_anal_noreturn_at_addr(RAnal *anal, ut64 addr) {
+	r_strf_buffer (64);
 	return sdb_bool_get (anal->sdb_types, K_NORET_ADDR (addr), NULL);
 }
 
@@ -616,7 +632,7 @@ static bool noreturn_recurse(RAnal *anal, ut64 addr) {
 	ut8 bbuf[0x10] = {0};
 	ut64 recurse_addr = UT64_MAX;
 	if (!anal->iob.read_at (anal->iob.io, addr, bbuf, sizeof (bbuf))) {
-		eprintf ("Couldn't read buffer\n");
+		R_LOG_ERROR ("Couldn't read buffer");
 		return false;
 	}
 	if (r_anal_op (anal, &op, addr, bbuf, sizeof (bbuf), R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_VAL) < 1) {
@@ -678,6 +694,12 @@ R_API void r_anal_bind(RAnal *anal, RAnalBind *b) {
 		b->anal = anal;
 		b->get_fcn_in = r_anal_get_fcn_in;
 		b->get_hint = r_anal_hint_get;
+		b->encode = (RAnalEncode)r_anal_opasm;
+		b->decode = (RAnalDecode)r_anal_op;
+		b->opinit = r_anal_op_init;
+		b->mnemonics = r_anal_mnemonics;
+		b->opfini = r_anal_op_fini;
+		b->use = r_anal_use;
 	}
 }
 
@@ -717,6 +739,7 @@ R_API void r_anal_add_import(RAnal *anal, const char *imp) {
 	if (!cimp) {
 		return;
 	}
+	R_DIRTY (anal);
 	r_list_push (anal->imports, cimp);
 }
 
@@ -725,6 +748,7 @@ R_API void r_anal_remove_import(RAnal *anal, const char *imp) {
 	const char *eimp;
 	r_list_foreach (anal->imports, it, eimp) {
 		if (!strcmp (eimp, imp)) {
+			R_DIRTY (anal);
 			r_list_delete (anal->imports, it);
 			return;
 		}
@@ -732,5 +756,6 @@ R_API void r_anal_remove_import(RAnal *anal, const char *imp) {
 }
 
 R_API void r_anal_purge_imports(RAnal *anal) {
+	R_DIRTY(anal);
 	r_list_purge (anal->imports);
 }
