@@ -1,12 +1,12 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #include <r_lib.h>
 #include <r_asm.h>
 
 static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 	char *ipath, *opath;
-	if (a->syntax != R_ASM_SYNTAX_INTEL) {
-		eprintf ("asm.x86.nasm does not support non-intel syntax\n");
+	if (a->config->syntax != R_ASM_SYNTAX_INTEL) {
+		R_LOG_ERROR ("asm.x86.nasm does not support non-intel syntax");
 		return -1;
 	}
 
@@ -21,7 +21,7 @@ static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 		return -1;
 	}
 
-	char *asm_buf = r_str_newf ("[BITS %i]\nORG 0x%"PFMT64x"\n%s\n", a->bits, a->pc, buf);
+	char *asm_buf = r_str_newf ("[BITS %i]\nORG 0x%"PFMT64x"\n%s\n", a->config->bits, a->pc, buf);
 	if (asm_buf) {
 		int slen = strlen (asm_buf);
 		int wlen = write (ifd, asm_buf, slen);
@@ -38,7 +38,7 @@ static int assemble(RAsm *a, RAsmOp *op, const char *buf) {
 		op->size = read (ofd, buf, sizeof (buf));
 		r_asm_op_set_buf (op, buf, op->size);
 	} else {
-		eprintf ("Error running 'nasm'\n");
+		R_LOG_ERROR ("running 'nasm'");
 	}
 
 	close (ofd);
