@@ -1,4 +1,6 @@
-/* radare2 - LGPL - Copyright 2009-2020 - pancake, nibble, dso */
+/* radare2 - LGPL - Copyright 2009-2022 - pancake, nibble, dso */
+
+#define R_LOG_ORIGIN "bin"
 
 #include <r_bin.h>
 #include <r_types.h>
@@ -63,7 +65,7 @@ R_API RBinXtrData *r_bin_xtrdata_new(RBuffer *buf, ut64 offset, ut64 size, ut32
 		data->size = size;
 		data->file_count = file_count;
 		data->metadata = metadata;
-		data->loaded = 0;
+		data->loaded = false;
 // don't slice twice TODO. review this
 		data->buf = r_buf_ref (buf); // r_buf_new_slice (buf, offset, size);
 	}
@@ -105,7 +107,7 @@ R_API RList *r_bin_dump_strings(RBinFile *bf, int min, int raw) {
 	return r_bin_file_get_strings (bf, min, 1, raw);
 }
 
-R_API void r_bin_options_init(RBinOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, int rawstr) {
+R_API void r_bin_file_options_init(RBinFileOptions *opt, int fd, ut64 baseaddr, ut64 loadaddr, int rawstr) {
 	memset (opt, 0, sizeof (*opt));
 	opt->baseaddr = baseaddr;
 	opt->loadaddr = loadaddr;
@@ -134,6 +136,7 @@ R_API void r_bin_info_free(RBinInfo *rb) {
 	r_list_free (rb->file_hashes);
 	free (rb->intrp);
 	free (rb->file);
+	free (rb->charset);
 	free (rb->type);
 	free (rb->bclass);
 	free (rb->rclass);
@@ -175,13 +178,6 @@ R_API void r_bin_import_free(RBinImport *imp) {
 	}
 }
 
-R_API const char *r_bin_symbol_name(RBinSymbol *s) {
-	if (s->dup_count) {
-		return sdb_fmt ("%s_%d", s->name, s->dup_count);
-	}
-	return s->name;
-}
-
 R_API RBinSymbol *r_bin_symbol_new(const char *name, ut64 paddr, ut64 vaddr) {
 	RBinSymbol *sym = R_NEW0 (RBinSymbol);
 	if (sym) {
@@ -215,7 +211,7 @@ R_API void r_bin_string_free(void *_str) {
 // kinda a clunky functions
 // XXX - this is a rather hacky way to do things, there may need to be a better
 // way.
-R_API bool r_bin_open(RBin *bin, const char *file, RBinOptions *opt) {
+R_API bool r_bin_open(RBin *bin, const char *file, RBinFileOptions *opt) {
 	r_return_val_if_fail (bin && bin->iob.io && opt, false);
 
 	RIOBind *iob = &(bin->iob);
@@ -223,7 +219,7 @@ R_API bool r_bin_open(RBin *bin, const char *file, RBinOptions *opt) {
 		opt->fd = iob->fd_open (iob->io, file, R_PERM_R, 0644);
 	}
 	if (opt->fd < 0) {
-		eprintf ("Couldn't open bin for file '%s'\n", file);
+		R_LOG_ERROR ("Couldn't open bin for file '%s'", file);
 		return false;
 	}
 	opt->sz = 0;
@@ -236,19 +232,22 @@ R_API bool r_bin_reload(RBin *bin, ut32 bf_id, ut64 baseaddr) {
 
 	RBinFile *bf = r_bin_file_find_by_id (bin, bf_id);
 	if (!bf) {
-		eprintf ("r_bin_reload: No file to reopen\n");
+		R_LOG_ERROR ("r_bin_reload: No file to reopen");
 		return false;
 	}
-	RBinOptions opt;
-	r_bin_options_init (&opt, bf->fd, baseaddr, bf->loadaddr, bin->rawstr);
+	RBinFileOptions opt;
+	r_bin_file_options_init (&opt, bf->fd, baseaddr, bf->loadaddr, bin->rawstr);
 	opt.filename = bf->file;
-
+	if (!bf->buf) {
+		r_bin_file_delete (bin, bf->id);
+		return false;
+	}
 	bool res = r_bin_open_buf (bin, bf->buf, &opt);
 	r_bin_file_delete (bin, bf->id);
 	return res;
 }
 
-R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinOptions *opt) {
+R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinFileOptions *opt) {
 	r_return_val_if_fail (bin && opt, false);
 
 	RListIter *it;
@@ -270,7 +269,7 @@ R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinOptions *opt) {
 				eprintf ("Missing check_buffer callback for '%s'\n", xtr->name);
 				continue;
 			}
-			if (xtr->check_buffer (buf)) {
+			if (xtr->check_buffer (bf, buf)) {
 				if (xtr->extract_from_buffer || xtr->extractall_from_buffer ||
 				    xtr->extract_from_bytes || xtr->extractall_from_bytes) {
 					bf = r_bin_file_xtr_load_buffer (bin, xtr,
@@ -283,7 +282,7 @@ R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinOptions *opt) {
 	if (!bf) {
 		// Uncomment for this speedup: 20s vs 22s
 		// RBuffer *buf = r_buf_new_slurp (bin->file);
-		bf = r_bin_file_new_from_buffer (bin, bin->file, buf, bin->rawstr,
+		bf = r_bin_file_new_from_buffer (bin, bin->file? bin->file: "?", buf, bin->rawstr,
 			opt->baseaddr, opt->loadaddr, opt->fd, opt->pluginname);
 		if (!bf) {
 			return false;
@@ -296,7 +295,7 @@ R_API bool r_bin_open_buf(RBin *bin, RBuffer *buf, RBinOptions *opt) {
 	return true;
 }
 
-R_API bool r_bin_open_io(RBin *bin, RBinOptions *opt) {
+R_API bool r_bin_open_io(RBin *bin, RBinFileOptions *opt) {
 	r_return_val_if_fail (bin && opt && bin->iob.io, false);
 	r_return_val_if_fail (opt->fd >= 0 && (st64)opt->sz >= 0, false);
 
@@ -334,7 +333,7 @@ R_API bool r_bin_open_io(RBin *bin, RBinOptions *opt) {
 	RBuffer *slice = buf;
 	if (!is_debugger && (opt->loadaddr != 0 || opt->sz != r_buf_size (buf))) {
 		slice = r_buf_new_slice (buf, opt->loadaddr, opt->sz);
-	} else if (is_debugger && opt->baseaddr != UT64_MAX && opt->baseaddr != 0) {
+	} else if (is_debugger && opt->baseaddr != UT64_MAX) {
 		slice = r_buf_new_slice (buf, opt->baseaddr, opt->sz);
 	}
 	if (slice != buf) {
@@ -362,7 +361,7 @@ R_IPI RBinPlugin *r_bin_get_binplugin_by_name(RBin *bin, const char *name) {
 	return NULL;
 }
 
-R_API RBinPlugin *r_bin_get_binplugin_by_buffer(RBin *bin, RBuffer *buf) {
+R_API RBinPlugin *r_bin_get_binplugin_by_buffer(RBin *bin, RBinFile *bf, RBuffer *buf) {
 	RBinPlugin *plugin;
 	RListIter *it;
 
@@ -370,7 +369,7 @@ R_API RBinPlugin *r_bin_get_binplugin_by_buffer(RBin *bin, RBuffer *buf) {
 
 	r_list_foreach (bin->plugins, it, plugin) {
 		if (plugin->check_buffer) {
-			if (plugin->check_buffer (buf)) {
+			if (plugin->check_buffer (bf, buf)) {
 				return plugin;
 			}
 		}
@@ -389,8 +388,6 @@ R_IPI RBinXtrPlugin *r_bin_get_xtrplugin_by_name(RBin *bin, const char *name) {
 		if (!strcmp (xtr->name, name)) {
 			return xtr;
 		}
-		// must be set to null
-		xtr = NULL;
 	}
 	return NULL;
 }
@@ -549,7 +546,7 @@ R_API bool r_bin_list_plugin(RBin *bin, const char* name, PJ *pj, int json) {
 		return true;
 	}
 
-	eprintf ("Cannot find plugin %s\n", name);
+	R_LOG_ERROR ("Cannot find plugin %s", name);
 	return false;
 }
 
@@ -649,7 +646,7 @@ R_API void r_bin_set_baddr(RBin *bin, ut64 baddr) {
 			}
 		}
 	} else {
-		eprintf ("Warning: This should be an assert probably.\n");
+		R_LOG_WARN ("This should be an assert probably");
 	}
 	// XXX - update all the infos?
 	// maybe in RBinFile.rebase() ?
@@ -677,7 +674,7 @@ R_API RList *r_bin_get_fields(RBin *bin) {
 	return o ? o->fields : NULL;
 }
 
-R_API RList *r_bin_get_imports(RBin *bin) {
+R_API const RList *r_bin_get_imports(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
 	RBinObject *o = r_bin_cur_object (bin);
 	return o ? o->imports : NULL;
@@ -695,43 +692,18 @@ R_API RList *r_bin_get_libs(RBin *bin) {
 	return o ? o->libs : NULL;
 }
 
-static RList *relocs_rbtree2list(RBNode *root) {
-	RList *res = r_list_new ();
-	RBinReloc *reloc;
-	RBIter it;
-
-	r_rbtree_foreach (root, it, reloc, RBinReloc, vrb) {
-		r_list_append (res, reloc);
-	}
-	return res;
-}
-
-R_API RBNode *r_bin_patch_relocs(RBin *bin) {
+R_API RRBTree *r_bin_patch_relocs(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
 	RBinObject *o = r_bin_cur_object (bin);
 	return o? r_bin_object_patch_relocs (bin, o): NULL;
 }
 
-// return a list of <const RBinReloc> that needs to be freed by the caller
-R_API RList *r_bin_patch_relocs_list(RBin *bin) {
-	r_return_val_if_fail (bin, NULL);
-	RBNode *root = r_bin_patch_relocs (bin);
-	return root? relocs_rbtree2list (root): NULL;
-}
-
-R_API RBNode *r_bin_get_relocs(RBin *bin) {
+R_API RRBTree *r_bin_get_relocs(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
 	RBinObject *o = r_bin_cur_object (bin);
 	return o ? o->relocs : NULL;
 }
 
-// return a list of <const RBinReloc> that needs to be freed by the caller
-R_API RList *r_bin_get_relocs_list(RBin *bin) {
-	r_return_val_if_fail (bin, NULL);
-	RBNode *root = r_bin_get_relocs (bin);
-	return root? relocs_rbtree2list (root): NULL;
-}
-
 R_API RList *r_bin_get_sections(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
 	RBinObject *o = r_bin_cur_object (bin);
@@ -905,7 +877,7 @@ R_API bool r_bin_use_arch(RBin *bin, const char *arch, int bits, const char *nam
 
 	RBinFile *binfile = r_bin_file_find_by_arch_bits (bin, arch, bits);
 	if (!binfile) {
-		R_LOG_WARN ("Cannot find binfile with arch/bits %s/%d\n", arch, bits);
+		R_LOG_WARN ("Cannot find binfile with arch/bits %s/%d", arch, bits);
 		return false;
 	}
 
@@ -943,7 +915,7 @@ R_API int r_bin_select_object(RBinFile *binfile, const char *arch, int bits, con
 }
 
 // NOTE: this functiona works as expected, but  we need to merge bfid and boid
-R_API bool r_bin_select_bfid (RBin *bin, ut32 bf_id) {
+R_API bool r_bin_select_bfid(RBin *bin, ut32 bf_id) {
 	r_return_val_if_fail (bin, false);
 	RBinFile *bf = r_bin_file_find_by_id (bin, bf_id);
 	return bf? r_bin_file_set_obj (bin, bf, NULL): false;
@@ -1015,11 +987,11 @@ R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
 	}
 	Sdb *binfile_sdb = binfile? binfile->sdb: NULL;
 	if (!binfile_sdb) {
-	//	eprintf ("Cannot find SDB!\n");
+	//	R_LOG_ERROR ("Cannot find SDB!");
 		return;
 	}
 	if (!binfile) {
-	//	eprintf ("Binary format not currently loaded!\n");
+	//	R_LOG_ERROR ("Binary format not currently loaded!");
 		return;
 	}
 	sdb_unset (binfile_sdb, ARCHS_KEY, 0);
@@ -1069,9 +1041,11 @@ R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
 			pj_end (pj);
 			break;
 		default:
-			str_fmt = h_flag && strcmp (h_flag, "unknown_flag")? sdb_fmt ("%s_%i %s", arch, bits, h_flag) \
-				: sdb_fmt ("%s_%i", arch, bits);
+			str_fmt = h_flag && strcmp (h_flag, "unknown_flag")
+				? r_str_newf ("%s_%i %s", arch, bits, h_flag) \
+				: r_str_newf ("%s_%i", arch, bits);
 			r_table_add_rowf (table, fmt, 0, boffset, obj_size, str_fmt, machine);
+			free (str_fmt);
 			bin->cb_printf ("%s", r_table_tostring (table));
 		}
 		snprintf (archline, sizeof (archline) - 1,
@@ -1101,9 +1075,11 @@ R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
 				pj_end (pj);
 				break;
 			default:
-				str_fmt = h_flag && strcmp (h_flag, "unknown_flag")? sdb_fmt ("%s_%i %s", arch, bits, h_flag) \
-					: sdb_fmt ("%s_%i", arch, bits);
+				str_fmt = h_flag && strcmp (h_flag, "unknown_flag")
+					? r_str_newf ("%s_%i %s", arch, bits, h_flag)
+					: r_str_newf ("%s_%i", arch, bits);
 				r_table_add_rowf (table, fmt, 0, boffset, obj_size, str_fmt, "");
+				free (str_fmt);
 				bin->cb_printf ("%s", r_table_tostring (table));
 			}
 			snprintf (archline, sizeof (archline),
@@ -1133,7 +1109,7 @@ R_API void r_bin_list_archs(RBin *bin, PJ *pj, int mode) {
 				"0x%08" PFMT64x ":%" PFMT64u ":%s:%d",
 				boffset, obj_size, "unk", 0);
 		} else {
-			eprintf ("Error: Invalid RBinFile.\n");
+			R_LOG_ERROR ("Invalid RBinFile");
 		}
 		//sdb_array_push (binfile_sdb, ARCHS_KEY, archline, 0);
 	}
@@ -1149,7 +1125,7 @@ R_API void r_bin_set_user_ptr(RBin *bin, void *user) {
 
 static RBinSection* __get_vsection_at(RBin *bin, ut64 vaddr) {
 	r_return_val_if_fail (bin, NULL);
-	if (!bin->cur) {
+	if (!bin->cur || !bin->cur->o) {
 		return NULL;
 	}
 	return r_bin_get_section_at (bin->cur->o, vaddr, true);
@@ -1175,11 +1151,11 @@ R_API RBuffer *r_bin_create(RBin *bin, const char *p,
 
 	RBinPlugin *plugin = r_bin_get_binplugin_by_name (bin, p);
 	if (!plugin) {
-		R_LOG_WARN ("Cannot find RBin plugin named '%s'.\n", p);
+		R_LOG_WARN ("Cannot find RBin plugin named '%s'", p);
 		return NULL;
 	}
 	if (!plugin->create) {
-		R_LOG_WARN ("RBin plugin '%s' does not implement \"create\" method.\n", p);
+		R_LOG_WARN ("RBin plugin '%s' does not implement \"create\" method", p);
 		return NULL;
 	}
 	codelen = R_MAX (codelen, 0);
@@ -1200,7 +1176,7 @@ R_API RBuffer *r_bin_package(RBin *bin, const char *type, const char *file, RLis
 		ut8 *num8 = (ut8*)&num;
 		RBuffer *buf = r_buf_new_file (file, O_RDWR | O_CREAT, 0644);
 		if (!buf) {
-			eprintf ("Cannot open file %s - Permission Denied.\n", file);
+			R_LOG_ERROR ("Cannot open file %s - Permission Denied", file);
 			return NULL;
 		}
 		r_buf_write_at (buf, 0, (const ut8*)"\xca\xfe\xba\xbe", 4);
@@ -1217,7 +1193,7 @@ R_API RBuffer *r_bin_package(RBin *bin, const char *type, const char *file, RLis
 			if (f_buf) {
 				eprintf ("ADD %s %"PFMT64u"\n", f, (ut64)f_len);
 			} else {
-				eprintf ("Cannot open %s\n", f);
+				R_LOG_ERROR ("Cannot open %s", f);
 				free (f_buf);
 				continue;
 			}
@@ -1254,7 +1230,7 @@ R_API RBuffer *r_bin_package(RBin *bin, const char *type, const char *file, RLis
 	return NULL;
 }
 
-R_API RList * /*<RBinClass>*/ r_bin_get_classes(RBin *bin) {
+R_API RList */*<RBinClass>*/ r_bin_get_classes(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
 	RBinObject *o = r_bin_cur_object (bin);
 	return o ? o->classes : NULL;
@@ -1302,8 +1278,8 @@ R_API RBinFile *r_bin_cur(RBin *bin) {
 
 R_API RBinObject *r_bin_cur_object(RBin *bin) {
 	r_return_val_if_fail (bin, NULL);
-	RBinFile *binfile = r_bin_cur (bin);
-	return binfile ? binfile->o : NULL;
+	RBinFile *bf = r_bin_cur (bin);
+	return bf ? bf->o : NULL;
 }
 
 R_API void r_bin_force_plugin(RBin *bin, const char *name) {
