@@ -1,18 +1,17 @@
-/* radare - LGPL - Copyright 2006-2021 - pancake */
+/* radare - LGPL - Copyright 2006-2022 - pancake */
 
 #include "r_config.h"
 
 R_API RConfigNode* r_config_node_new(const char *name, const char *value) {
 	r_return_val_if_fail (name && *name && value, NULL);
 	RConfigNode *node = R_NEW0 (RConfigNode);
-	if (!node) {
-		return NULL;
+	if (R_LIKELY (node)) {
+		node->name = strdup (name);
+		node->value = strdup (r_str_get (value));
+		node->flags = CN_RW | CN_STR;
+		node->i_value = r_num_get (NULL, value);
+		node->options = NULL;
 	}
-	node->name = strdup (name);
-	node->value = strdup (r_str_get (value));
-	node->flags = CN_RW | CN_STR;
-	node->i_value = r_num_get (NULL, value);
-	node->options = NULL;
 	return node;
 }
 
@@ -21,6 +20,7 @@ R_API char *r_config_node_to_string(RConfigNode *node) {
 }
 
 R_API void r_config_node_purge_options(RConfigNode *node) {
+	r_return_if_fail (node);
 	if (node->options) {
 		r_list_purge (node->options);
 	} else {
@@ -29,6 +29,7 @@ R_API void r_config_node_purge_options(RConfigNode *node) {
 }
 
 R_API void r_config_node_add_option(RConfigNode *node, const char *option) {
+	r_return_if_fail (node && option);
 	if (!node->options) {
 		node->options = r_list_newf (free);
 	}
@@ -38,29 +39,27 @@ R_API void r_config_node_add_option(RConfigNode *node, const char *option) {
 R_API RConfigNode* r_config_node_clone(RConfigNode *n) {
 	r_return_val_if_fail (n, NULL);
 	RConfigNode *cn = R_NEW0 (RConfigNode);
-	if (!cn) {
-		return NULL;
+	if (R_LIKELY (cn)) {
+		cn->name = strdup (n->name);
+		cn->desc = n->desc ? strdup (n->desc) : NULL;
+		cn->value = strdup (r_str_get (n->value));
+		cn->i_value = n->i_value;
+		cn->flags = n->flags;
+		cn->setter = n->setter;
+		cn->options = n->options? r_list_clone (n->options): NULL;
 	}
-	cn->name = strdup (n->name);
-	cn->desc = n->desc ? strdup (n->desc) : NULL;
-	cn->value = strdup (r_str_get (n->value));
-	cn->i_value = n->i_value;
-	cn->flags = n->flags;
-	cn->setter = n->setter;
-	cn->options = n->options? r_list_clone (n->options): NULL;
 	return cn;
 }
 
 R_API void r_config_node_free(void *n) {
 	RConfigNode *node = (RConfigNode *)n;
-	if (!node) {
-		return;
+	if (R_LIKELY (node)) {
+		free (node->name);
+		free (node->desc);
+		free (node->value);
+		r_list_free (node->options);
+		free (node);
 	}
-	free (node->name);
-	free (node->desc);
-	free (node->value);
-	r_list_free (node->options);
-	free (node);
 }
 
 static void config_print_value_json(RConfig *cfg, RConfigNode *node) {
@@ -130,7 +129,7 @@ static void config_print_node(RConfig *cfg, RConfigNode *node, const char *pfx,
 	} else {
 		if (verbose) {
 			cfg->cb_printf ("%s%s = %s%s %s; %s", pfx,
-				node->name, node->value, sfx, 
+				node->name, node->value, sfx,
 				r_config_node_is_ro (node) ? "(ro)" : "",
 				node->desc);
 			if (node->options && !r_list_empty (node->options)) {
@@ -196,6 +195,19 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 			}
 		}
 		break;
+	case 'r':
+		r_list_foreach (cfg->nodes, iter, node) {
+			if (!str || (str && (!strncmp (str, node->name, len)))) {
+				if (r_str_startswith (node->name, "bin.laddr")) {
+					continue;
+				}
+				if (r_str_startswith (node->name, "dir.")) {
+					continue;
+				}
+				config_print_node (cfg, node, "\"e ", "\"", verbose, json);
+			}
+		}
+		break;
 	case 2:
 		r_list_foreach (cfg->nodes, iter, node) {
 			if (!str || (str && (!strncmp (str, node->name, len)))) {
@@ -259,7 +271,7 @@ R_API void r_config_list(RConfig *cfg, const char *str, int rad) {
 		break;
 	case 'J':
 		verbose = true;
-	/* fallthrou */
+		/* fallthrou */
 	case 'j':
 		isFirst = true;
 		if (verbose) {
@@ -305,7 +317,7 @@ R_API const char* r_config_get(RConfig *cfg, const char *name) {
 		}
 		return node->value;
 	} else {
-		eprintf ("r_config_get: variable '%s' not found\n", name);
+		R_LOG_WARN ("Variable '%s' not found", name);
 	}
 	return NULL;
 }
@@ -316,11 +328,11 @@ R_API bool r_config_toggle(RConfig *cfg, const char *name) {
 		return false;
 	}
 	if (!r_config_node_is_bool (node)) {
-		eprintf ("(error: '%s' is not a boolean variable)\n", name);
+		R_LOG_ERROR ("Not a boolean variable '%s'", name);
 		return false;
 	}
 	if (r_config_node_is_ro (node)) {
-		eprintf ("(error: '%s' config key is read only)\n", name);
+		R_LOG_ERROR ("Key is readonly '%s'", name);
 		return false;
 	}
 	(void)r_config_set_b (cfg, name, !node->i_value);
@@ -397,29 +409,28 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 	RConfigNode *node = NULL;
 	char *ov = NULL;
 	ut64 oi;
-
 	r_return_val_if_fail (cfg && cfg->ht, NULL);
 	r_return_val_if_fail (!IS_NULLSTR (name), NULL);
 
 	node = r_config_node_get (cfg, name);
 	if (node) {
 		if (r_config_node_is_ro (node)) {
-			eprintf ("(error: '%s' config key is read only)\n", name);
+			R_LOG_ERROR ("Key '%s' is readonly", name);
 			return node;
 		}
 		oi = node->i_value;
 		if (node->value) {
 			ov = strdup (node->value);
 			if (!ov) {
-				goto beach;
+				return node;
 			}
 		} else {
-			free (node->value);
 			node->value = strdup ("");
 		}
+		R_DIRTY (cfg);
 		if (r_config_node_is_bool (node)) {
 			bool b = r_str_is_true (value);
-			node->i_value = b? 1: 0;
+			node->i_value = b;
 			char *value = strdup (r_str_bool (b));
 			if (value) {
 				free (node->value);
@@ -452,7 +463,7 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 		oi = UT64_MAX;
 		if (!cfg->lock) {
 			node = r_config_node_new (name, value);
-			if (node) {
+			if (R_LIKELY (node)) {
 				if (is_true_or_false (value)) {
 					node->flags |= CN_BOOL;
 					node->i_value = r_str_is_true (value)? 1: 0;
@@ -460,10 +471,10 @@ R_API RConfigNode* r_config_set(RConfig *cfg, const char *name, const char *valu
 				ht_pp_insert (cfg->ht, node->name, node);
 				r_list_append (cfg->nodes, node);
 			} else {
-				eprintf ("r_config_set: unable to create a new RConfigNode\n");
+				R_LOG_ERROR ("unable to create a new RConfigNode");
 			}
 		} else {
-			eprintf ("r_config_set: variable '%s' not found\n", name);
+			R_LOG_ERROR ("variable '%s' not found", name);
 		}
 	}
 
@@ -488,23 +499,24 @@ beach:
 /* r_config_desc takes a RConfig and a name,
  * r_config_node_desc takes a RConfigNode
  * Both set and return node->desc */
-R_API const char* r_config_desc(RConfig *cfg, const char *name, const char *desc) {
+R_API RConfigNode *r_config_desc(RConfig *cfg, const char *name, const char *desc) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	return r_config_node_desc (node, desc);
 }
 
-R_API const char* r_config_node_desc(RConfigNode *node, const char *desc) {
+R_API RConfigNode* r_config_node_desc(RConfigNode *node, const char *desc) {
 	r_return_val_if_fail (node, NULL);
 	if (desc) {
 		free (node->desc);
 		node->desc = strdup (desc);
 	}
-	return node->desc;
+	return node;
 }
 
 R_API bool r_config_rm(RConfig *cfg, const char *name) {
 	RConfigNode *node = r_config_node_get (cfg, name);
 	if (node) {
+		R_DIRTY (cfg);
 		ht_pp_delete (cfg->ht, node->name);
 		r_list_delete_data (cfg->nodes, node);
 		return true;
@@ -518,9 +530,9 @@ R_API void r_config_node_value_format_i(char *buf, size_t buf_size, const ut64 i
 		return;
 	}
 	if (i < 1024) {
-		snprintf (buf, buf_size, "%" PFMT64d "", i);
+		snprintf (buf, buf_size, "%" PFMT64d, i);
 	} else {
-		snprintf (buf, buf_size, "0x%08" PFMT64x "", i);
+		snprintf (buf, buf_size, "0x%08" PFMT64x, i);
 	}
 }
 
@@ -536,6 +548,7 @@ R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i)
 	char buf[128], *ov = NULL;
 	r_return_val_if_fail (cfg && name, NULL);
 	RConfigNode *node = r_config_node_get (cfg, name);
+	R_DIRTY (cfg);
 	if (node) {
 		if (r_config_node_is_ro (node)) {
 			node = NULL;
@@ -564,7 +577,7 @@ R_API RConfigNode* r_config_set_i(RConfig *cfg, const char *name, const ut64 i)
 				r_list_append (cfg->nodes, node);
 			}
 		} else {
-			eprintf ("(locked: no new keys can be created (%s))\n", name);
+			R_LOG_ERROR ("Cannot create a new '%s' key because config is locked", name);
 		}
 	}
 
@@ -583,7 +596,7 @@ beach:
 	return node;
 }
 
-static void __evalString(RConfig *cfg, char *name) {
+static void eval_config_string(RConfig *cfg, char *name) {
 	if (!*name) {
 		return;
 	}
@@ -603,7 +616,7 @@ static void __evalString(RConfig *cfg, char *name) {
 			if (v) {
 				cfg->cb_printf ("%s\n", v);
 			} else {
-				eprintf ("Invalid config key %s\n", name);
+				R_LOG_ERROR ("Invalid config key %s", name);
 			}
 		}
 	}
@@ -614,7 +627,7 @@ R_API bool r_config_eval(RConfig *cfg, const char *str, bool many) {
 
 	char *s = r_str_trim_dup (str);
 
-	if (!*s || !strcmp (s, "help")) {
+	if (!*s || !strcmp (s, "help")) { // 580 wtf is help here
 		r_config_list (cfg, NULL, 0);
 		free (s);
 		return false;
@@ -632,12 +645,12 @@ R_API bool r_config_eval(RConfig *cfg, const char *str, bool many) {
 		RListIter *iter;
 		char *name;
 		r_list_foreach (list, iter, name) {
-			__evalString (cfg, name);
+			eval_config_string (cfg, name);
 		}
 		free (s);
 		return true;
 	}
-	__evalString (cfg, s);
+	eval_config_string (cfg, s);
 	free (s);
 	return true;
 }
@@ -675,6 +688,7 @@ R_API RConfig* r_config_new(void *user) {
 	cfg->num = NULL;
 	cfg->lock = false;
 	cfg->cb_printf = (void *) printf;
+	R_DIRTY (cfg);
 	return cfg;
 }
 
@@ -692,11 +706,12 @@ R_API RConfig* r_config_clone(RConfig *cfg) {
 	}
 	c->lock = cfg->lock;
 	c->cb_printf = cfg->cb_printf;
+	R_DIRTY (c);
 	return c;
 }
 
 R_API void r_config_free(RConfig *cfg) {
-	if (cfg) {
+	if (R_LIKELY (cfg)) {
 		cfg->nodes->free = r_config_node_free; // damn
 		r_list_free (cfg->nodes);
 		ht_pp_free (cfg->ht);
@@ -713,7 +728,7 @@ R_API void r_config_visual_hit_i(RConfig *cfg, const char *name, int delta) {
 
 R_API void r_config_bump(RConfig *cfg, const char *key) {
 	char *orig = strdup (r_config_get (cfg, key));
-	if (orig) {
+	if (R_LIKELY (orig)) {
 		r_config_set (cfg, key, orig);
 		free (orig);
 	}
@@ -730,13 +745,15 @@ R_API void r_config_serialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db) {
 static bool load_config_cb(void *user, const char *k, const char *v) {
 	RConfig *config = user;
 	RConfigNode *node = r_config_node_get (config, k);
-	if (node) {
+	if (R_LIKELY (node)) {
 		r_config_set (config, k, v);
 	}
 	return true;
 }
 
+// TODO 580 -> return void
 R_API bool r_config_unserialize(R_NONNULL RConfig *config, R_NONNULL Sdb *db, R_NULLABLE char **err) {
+	r_return_val_if_fail (config && db, false);
 	sdb_foreach (db, load_config_cb, config);
 	return true;
 }
