@@ -1,5 +1,6 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
+#define R_LOG_ORIGIN "cfile"
 #include <r_core.h>
 #include <stdlib.h>
 #include <string.h>
@@ -13,10 +14,10 @@ static bool close_but_cb(void *user, void *data, ut32 id) {
        RCore *core = (RCore *)user;
        RIODesc *desc = (RIODesc *)data;
        if (core && desc && core->io->desc) {
-               if (desc->fd != core->io->desc->fd) {
-                       // TODO: use the API
-                       r_core_cmdf (core, "o-%d", desc->fd);
-               }
+	       if (desc->fd != core->io->desc->fd) {
+		       // TODO: use the API
+		       r_core_cmdf (core, "o-%d", desc->fd);
+	       }
        }
        return true;
 }
@@ -27,14 +28,16 @@ R_API bool r_core_file_close_all_but(RCore *core) {
        return true;
 }
 
-static bool __isMips (RAsm *a) {
-	return a && a->cur && a->cur->arch && strstr (a->cur->arch, "mips");
+static bool its_a_mips(RCore *core) {
+	RArchConfig *cfg = core->rasm->config;
+	return cfg && cfg->arch && !strcmp (cfg->arch, "mips");
 }
 
 static void loadGP(RCore *core) {
-	if (__isMips (core->rasm)) {
+	if (its_a_mips (core)) {
+		ut64 e0 = r_num_math (core->num, "entry0");
 		ut64 gp = r_num_math (core->num, "loc._gp");
-		if (!gp || gp == UT64_MAX) {
+		if ((!gp || gp == UT64_MAX) && (e0 && e0 != UT64_MAX)) {
 			r_config_set (core->config, "anal.roregs", "zero");
 			r_core_cmd0 (core, "10aes@entry0");
 			r_config_set (core->config, "anal.roregs", "zero,gp");
@@ -49,7 +52,6 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 	const bool isdebug = r_config_get_b (core->config, "cfg.debug");
 	char *path;
 	ut64 laddr = r_config_get_i (core->config, "bin.laddr");
-	RIODesc *file = NULL;
 	RIODesc *odesc = core->io ? core->io->desc : NULL;
 	RBinFile *bf = odesc ? r_bin_file_find_by_fd (core->bin, odesc->fd) : NULL;
 	char *ofilepath = NULL, *obinfilepath = (bf && bf->file)? strdup (bf->file): NULL;
@@ -77,12 +79,12 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 	}
 
 	if (r_sandbox_enable (0)) {
-		eprintf ("Cannot reopen in sandbox\n");
+		R_LOG_ERROR ("Cannot reopen in sandbox");
 		free (obinfilepath);
 		return false;
 	}
 	if (!odesc) {
-		eprintf ("No file opened to reopen\n");
+		R_LOG_ERROR ("No file opened to reopen");
 		free (ofilepath);
 		free (obinfilepath);
 		return false;
@@ -102,7 +104,7 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 		}
 	}
 	if (!ofilepath) {
-		eprintf ("Unknown file path");
+		R_LOG_ERROR ("Unknown file path");
 		free (obinfilepath);
 		return false;
 	}
@@ -110,15 +112,13 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 	// HACK: move last mapped address to higher place
 	// XXX - why does this hack work?
 	// when the new memory maps are created.
-	path = strdup (ofilepath);
+	path = r_str_trim_dup (ofilepath);
 	free (obinfilepath);
-	obinfilepath = strdup (ofilepath);
+	obinfilepath = r_str_trim_dup (ofilepath);
 
-	// r_str_trim (path);
-	file = r_core_file_open (core, path, perm, laddr);
+	RIODesc *file = r_core_file_open (core, path, perm, laddr);
 	if (isdebug) {
 		int newtid = newpid;
-		// XXX - select the right backend
 		if (core->io->desc) {
 			newpid = r_io_fd_get_pid (core->io, core->io->desc->fd);
 #if __linux__
@@ -149,7 +149,6 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 		r_io_fd_close (core->io, odesc->fd);
 		eprintf ("File %s reopened in %s mode\n", path,
 			(perm & R_PERM_W)? "read-write": "read-only");
-
 		if (loadbin && (loadbin == 2 || had_rbin_info)) {
 			ut64 baddr;
 			if (isdebug) {
@@ -162,7 +161,7 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 			ret = r_core_bin_load (core, obinfilepath, baddr);
 			r_core_bin_update_arch_bits (core);
 			if (!ret) {
-				eprintf ("Error: Failed to reload rbin for: %s", path);
+				R_LOG_ERROR ("Failed to reload rbin for: %s", path);
 			}
 			origoff = r_num_math (core->num, "entry0");
 		}
@@ -173,12 +172,11 @@ R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int loadb
 		}
 		// close old file
 	} else if (odesc) {
-		eprintf ("r_core_file_reopen: Cannot reopen file: %s with perms 0x%x,"
-			" attempting to open read-only.\n", path, perm);
+		R_LOG_ERROR ("Cannot reopen file: %s with perms 0x%x, attempting to open read-only", path, perm);
 		// lower it down back
 		//ofile = r_core_file_open (core, path, R_PERM_R, addr);
 	} else {
-		eprintf ("Cannot reopen\n");
+		R_LOG_ERROR ("Cannot reopen");
 	}
 	if (core->io->desc) {
 		core->switch_file_view = 1;
@@ -221,6 +219,7 @@ R_API void r_core_sysenv_end(RCore *core, const char *cmd) {
 	r_sys_setenv ("R2_FILE", NULL);
 	r_sys_setenv ("R2_BYTES", NULL);
 	r_sys_setenv ("R2_OFFSET", NULL);
+	r_sys_setenv ("R2_UTF8", NULL);
 
 	// remove temporary R2_CONFIG file
 	char *r2_config = r_sys_getenv ("R2_CONFIG");
@@ -231,7 +230,7 @@ R_API void r_core_sysenv_end(RCore *core, const char *cmd) {
 	}
 }
 
-R_API char *r_core_sysenv_begin(RCore * core, const char *cmd) {
+R_API char *r_core_sysenv_begin(RCore *core, const char *cmd) {
 	char *f, *ret = cmd? strdup (cmd): NULL;
 	RIODesc *desc = core->io->desc;
 	if (cmd && strstr (cmd, "R2_BYTES")) {
@@ -239,10 +238,11 @@ R_API char *r_core_sysenv_begin(RCore * core, const char *cmd) {
 		r_sys_setenv ("R2_BYTES", s);
 		free (s);
 	}
+	r_strf_buffer (64);
 	r_sys_setenv ("RABIN2_PDBSERVER", r_config_get (core->config, "pdb.server"));
 	if (desc && desc->name) {
 		r_sys_setenv ("R2_FILE", desc->name);
-		r_sys_setenv ("R2_SIZE", sdb_fmt ("%"PFMT64d, r_io_desc_size (desc)));
+		r_sys_setenv ("R2_SIZE", r_strf ("%"PFMT64d, r_io_desc_size (desc)));
 		if (cmd && strstr (cmd, "R2_BLOCK")) {
 			// replace BLOCK in RET string
 			if ((f = r_file_temp ("r2block"))) {
@@ -253,32 +253,36 @@ R_API char *r_core_sysenv_begin(RCore * core, const char *cmd) {
 			}
 		}
 	}
-	r_sys_setenv ("R2_OFFSET", sdb_fmt ("%"PFMT64d, core->offset));
-	r_sys_setenv ("R2_XOFFSET", sdb_fmt ("0x%08"PFMT64x, core->offset));
-	r_sys_setenv ("R2_ENDIAN", core->rasm->big_endian? "big": "little");
-	r_sys_setenv ("R2_BSIZE", sdb_fmt ("%d", core->blocksize));
-
+	r_sys_setenv ("R2_OFFSET", r_strf ("%"PFMT64d, core->offset));
+	r_sys_setenv ("R2_XOFFSET", r_strf ("0x%08"PFMT64x, core->offset));
+	r_sys_setenv ("R2_ENDIAN", core->rasm->config->big_endian? "big": "little");
+	r_sys_setenv ("R2_BSIZE", r_strf ("%d", core->blocksize));
+#if 0
 	// dump current config file so other r2 tools can use the same options
 	char *config_sdb_path = NULL;
 	int config_sdb_fd = r_file_mkstemp (NULL, &config_sdb_path);
 	if (config_sdb_fd >= 0) {
 		close (config_sdb_fd);
 	}
-
 	Sdb *config_sdb = sdb_new (NULL, config_sdb_path, 0);
 	r_config_serialize (core->config, config_sdb);
 	sdb_sync (config_sdb);
 	sdb_free (config_sdb);
 	r_sys_setenv ("R2_CONFIG", config_sdb_path);
-
+#endif
 	r_sys_setenv ("RABIN2_LANG", r_config_get (core->config, "bin.lang"));
 	r_sys_setenv ("RABIN2_DEMANGLE", r_config_get (core->config, "bin.demangle"));
 	r_sys_setenv ("R2_ARCH", r_config_get (core->config, "asm.arch"));
-	r_sys_setenv ("R2_BITS", sdb_fmt ("%"PFMT64u, r_config_get_i (core->config, "asm.bits")));
-	r_sys_setenv ("R2_COLOR", r_config_get_i (core->config, "scr.color")? "1": "0");
-	r_sys_setenv ("R2_DEBUG", r_config_get_i (core->config, "cfg.debug")? "1": "0");
-	r_sys_setenv ("R2_IOVA", r_config_get_i (core->config, "io.va")? "1": "0");
+	r_sys_setenv ("R2_BITS", r_strf ("%"PFMT64u, r_config_get_i (core->config, "asm.bits")));
+	char *s = sdb_itoa (r_config_get_i (core->config, "scr.color"), NULL, 10);
+	r_sys_setenv ("R2_COLOR", s);
+	free (s);
+	r_sys_setenv ("R2_UTF8", r_config_get_b (core->config, "scr.utf8")? "1": "0");
+	r_sys_setenv ("R2_DEBUG", r_config_get_b (core->config, "cfg.debug")? "1": "0");
+	r_sys_setenv ("R2_IOVA", r_config_get_b (core->config, "io.va")? "1": "0");
+#if 0
 	free (config_sdb_path);
+#endif
 	return ret;
 }
 
@@ -334,20 +338,20 @@ static bool setbpint(RCore *r, const char *mode, const char *sym) {
 #endif
 		return true;
 	}
-	eprintf ("Cannot set breakpoint at %s\n", sym);
+	R_LOG_ERROR ("Cannot set breakpoint at %s", sym);
 	return false;
 }
 #endif
 
 // XXX - need to handle index selection during debugging
-static int r_core_file_do_load_for_debug(RCore *r, ut64 baseaddr, const char *filenameuri) {
+static int r_core_file_do_load_for_debug(RCore *r, ut64 baseaddr, R_NULLABLE const char *filenameuri) {
 	RIODesc *desc = r->io->desc;
 	RBinFile *binfile = NULL;
 	RBinPlugin *plugin;
 	int xtr_idx = 0; // if 0, load all if xtr is used
 
 	// TODO : Honor file.path eval var too?
-	if (!strncmp ("dbg://", filenameuri, 6)) {
+	if (filenameuri && r_str_startswith (filenameuri, "dbg://")) {
 		filenameuri += 6;
 	}
 	if (!desc) {
@@ -364,13 +368,13 @@ static int r_core_file_do_load_for_debug(RCore *r, ut64 baseaddr, const char *fi
 		r_config_set_i (r->config, "bin.baddr", baseaddr);
 	}
 #endif
-	RBinOptions opt;
-	r_bin_options_init (&opt, fd, baseaddr, UT64_MAX, false);
+	RBinFileOptions opt;
+	r_bin_file_options_init (&opt, fd, baseaddr, UT64_MAX, false);
 	opt.xtr_idx = xtr_idx;
 	if (!r_bin_open (r->bin, filenameuri, &opt)) {
 		eprintf ("RBinLoad: Cannot open %s\n", filenameuri);
 		if (r_config_get_i (r->config, "bin.rawstr")) {
-			r_bin_options_init (&opt, fd, baseaddr, UT64_MAX, true);
+			r_bin_file_options_init (&opt, fd, baseaddr, UT64_MAX, true);
 			opt.xtr_idx = xtr_idx;
 			if (!r_bin_open (r->bin, filenameuri, &opt)) {
 				return false;
@@ -408,7 +412,7 @@ static int r_core_file_do_load_for_debug(RCore *r, ut64 baseaddr, const char *fi
 	}
 
 	if (plugin && !strcmp (plugin->name, "dex")) {
-		r_core_cmd0 (r, "\"(fix-dex,wx `ph sha1 $s-32 @32` @12 ; wx `ph adler32 $s-12 @12` @8)\"\n");
+		r_core_cmd0 (r, "\"(fix-dex;wx `ph sha1 $s-32 @32` @12 ; wx `ph adler32 $s-12 @12` @8)\"");
 	}
 
 	return true;
@@ -425,17 +429,17 @@ static int r_core_file_do_load_for_io_plugin(RCore *r, ut64 baseaddr, ut64 loada
 		return false;
 	}
 	r_io_use_fd (r->io, fd);
-	RBinOptions opt;
-	r_bin_options_init (&opt, fd, baseaddr, loadaddr, r->bin->rawstr);
+	RBinFileOptions opt;
+	r_bin_file_options_init (&opt, fd, baseaddr, loadaddr, r->bin->rawstr);
+	// opt.fd = fd;
 	opt.xtr_idx = xtr_idx;
 	if (!r_bin_open_io (r->bin, &opt)) {
-		//eprintf ("Failed to load the bin with an IO Plugin.\n");
 		return false;
 	}
 	binfile = r_bin_cur (r->bin);
 	if (r_core_bin_set_env (r, binfile)) {
-		if (!r->anal->sdb_cc->path) {
-			R_LOG_WARN ("No calling convention defined for this file, analysis may be inaccurate.\n");
+		if (r->anal->verbose && !sdb_const_get (r->anal->sdb_cc, "default.cc", 0)) {
+			R_LOG_WARN ("No calling convention defined for this file, analysis may be inaccurate");
 		}
 	}
 	plugin = r_bin_file_cur_plugin (binfile);
@@ -445,7 +449,7 @@ static int r_core_file_do_load_for_io_plugin(RCore *r, ut64 baseaddr, ut64 loada
 		if (!info) {
 			return false;
 		}
-		info->bits = r->rasm->bits;
+		info->bits = r->rasm->config->bits;
 		// set use of raw strings
 		r_core_bin_set_arch_bits (r, binfile->file, info->arch, info->bits);
 		// r_config_set_i (r->config, "io.va", false);
@@ -466,7 +470,7 @@ static int r_core_file_do_load_for_io_plugin(RCore *r, ut64 baseaddr, ut64 loada
 	}
 
 	if (plugin && !strcmp (plugin->name, "dex")) {
-		r_core_cmd0 (r, "\"(fix-dex,wx `ph sha1 $s-32 @32` @12 ; wx `ph adler32 $s-12 @12` @8)\"\n");
+		r_core_cmd0 (r, "\"(fix-dex;wx `ph sha1 $s-32 @32` @12 ; wx `ph adler32 $s-12 @12` @8)\"");
 	}
 	return true;
 }
@@ -475,7 +479,6 @@ static bool try_loadlib(RCore *core, const char *lib, ut64 addr) {
 	void *p = r_core_file_open (core, lib, 0, addr);
 	if (p) {
 		r_core_bin_load (core, lib, addr);
-		R_FREE (p);
 		return true;
 	}
 	return false;
@@ -483,16 +486,14 @@ static bool try_loadlib(RCore *core, const char *lib, ut64 addr) {
 
 R_API bool r_core_file_loadlib(RCore *core, const char *lib, ut64 libaddr) {
 	const char *dirlibs = r_config_get (core->config, "dir.libs");
-	bool free_libdir = true;
 #ifdef __WINDOWS__
 	char *libdir = r_str_r2_prefix (R2_LIBDIR);
+	if (!libdir) {
+		libdir = strdup (R2_LIBDIR);
+	}
 #else
 	char *libdir = strdup (R2_LIBDIR);
 #endif
-	if (!libdir) {
-		libdir = R2_LIBDIR;
-		free_libdir = false;
-	}
 	if (!dirlibs || !*dirlibs) {
 		dirlibs = "." R_SYS_DIR;
 	}
@@ -531,28 +532,11 @@ R_API bool r_core_file_loadlib(RCore *core, const char *lib, ut64 libaddr) {
 			libpath++;
 		}
 	}
-	if (free_libdir) {
-		free (libdir);
-	}
+	free (libdir);
 	return ret;
 }
 
-R_API int r_core_bin_rebase(RCore *core, ut64 baddr) {
-	if (!core || !core->bin || !core->bin->cur) {
-		return 0;
-	}
-	if (baddr == UT64_MAX) {
-		return 0;
-	}
-	RBinFile *bf = core->bin->cur;
-	bf->o->baddr = baddr;
-	bf->o->loadaddr = baddr;
-	r_bin_object_set_items (bf, bf->o);
-	return 1;
-}
-
 static void load_scripts_for(RCore *core, const char *name) {
-	// TODO:
 	char *file;
 	RListIter *iter;
 	char *hdir = r_str_newf (R_JOIN_2_PATHS (R2_HOME_BINRC, "bin-%s"), name);
@@ -618,30 +602,29 @@ static bool linkcb(void *user, void *data, ut32 id) {
 }
 
 R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
-	RIODesc *desc = r->io->desc;
+	r_return_val_if_fail (r && r->io, false);
 	ut64 laddr = r_config_get_i (r->config, "bin.laddr");
 	RBinFile *binfile = NULL;
 	RBinPlugin *plugin = NULL;
-	bool is_io_load;
 	const char *cmd_load;
+	RIODesc *desc = r->io->desc;
+	if (!desc && filenameuri) {
+		desc = r_io_desc_get_byuri (r->io, filenameuri);
+	}
 	if (!desc) {
-		return false;
-	}
-	// NULL deref guard
-	if (desc) {
-		is_io_load = desc && desc->plugin;
-		if (!filenameuri || !*filenameuri) {
-			filenameuri = desc->name;
+		// hack for openmany handlers
+		if (!filenameuri || *filenameuri == '-') {
+			// filenameuri = "malloc://512";
+			return false;
 		}
-	} else {
-		is_io_load = false;
+		r_core_file_open (r, filenameuri, 0, baddr);
+		desc = r->io->desc;
 	}
-
-	if (!filenameuri) {
-		eprintf ("r_core_bin_load: no file specified\n");
-		return false;
+	bool is_io_load = false;
+	if (desc && desc->plugin) {
+		is_io_load = true;
+	//	r_io_use_fd (r->io, desc->fd);
 	}
-
 	r->bin->minstrlen = r_config_get_i (r->config, "bin.minstr");
 	r->bin->maxstrbuf = r_config_get_i (r->config, "bin.maxstrbuf");
 	if (is_io_load) {
@@ -670,6 +653,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 	if (plugin && plugin->name) {
 		load_scripts_for (r, plugin->name);
 	}
+	r_core_bin_export_info (r, R_MODE_SET);
 	cmd_load = r_config_get (r->config, "cmd.load");
 	if (cmd_load && *cmd_load) {
 		r_core_cmd (r, cmd_load, 0);
@@ -677,11 +661,9 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 
 	if (plugin && plugin->name) {
 		if (!strcmp (plugin->name, "any")) {
-			if (r_str_startswith (desc->name, "rap") && strstr (desc->name, "://")) {
-				r_io_map_new (r->io, desc->fd, desc->perm, 0, laddr, UT64_MAX);
-			} else {
-				r_io_map_new (r->io, desc->fd, desc->perm, 0, laddr, r_io_desc_size (desc));
-			}
+			ut64 size = (desc->name && (r_str_startswith (desc->name, "rap") && strstr (desc->name, "://")))
+				? UT64_MAX : r_io_desc_size (desc);
+			r_io_map_add (r->io, desc->fd, desc->perm, 0, laddr, size);
 			// set use of raw strings
 			//r_config_set (r->config, "bin.rawstr", "true");
 			// r_config_set_i (r->config, "io.va", false);
@@ -697,7 +679,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 				}
 				//workaround to map correctly malloc:// and raw binaries
 				if (r_io_desc_is_dbg (desc) || (!obj->sections || !va)) {
-					r_io_map_new (r->io, desc->fd, desc->perm, 0, laddr, r_io_desc_size (desc));
+					r_io_map_add (r->io, desc->fd, desc->perm, 0, laddr, r_io_desc_size (desc));
 				}
 				RBinInfo *info = obj->info;
 				if (info) {
@@ -711,7 +693,7 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 		}
 	} else {
 		if (desc) {
-			r_io_map_new (r->io, desc->fd, desc->perm, 0, laddr, r_io_desc_size (desc));
+			r_io_map_add (r->io, desc->fd, desc->perm, 0, laddr, r_io_desc_size (desc));
 		}
 		if (binfile) {
 			r_core_bin_set_arch_bits (r, binfile->file,
@@ -723,13 +705,12 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 		desc->perm |= R_PERM_X;
 	}
 	if (plugin && plugin->name && !strcmp (plugin->name, "dex")) {
-		r_core_cmd0 (r, "\"(fix-dex,wx `ph sha1 $s-32 @32` @12 ;"
-			" wx `ph adler32 $s-12 @12` @8)\"\n");
+		r_core_cmd0 (r, "\"(fix-dex;wx `ph sha1 $s-32 @32` @12;wx `ph adler32 $s-12 @12` @8)\"");
 	}
 	if (!r_config_get_b (r->config, "cfg.debug")) {
 		loadGP (r);
 	}
-	if (r_config_get_i (r->config, "bin.libs")) {
+	if (r_config_get_b (r->config, "bin.libs")) {
 		const char *lib;
 		RListIter *iter;
 		RList *libs = r_bin_get_libs (r->bin);
@@ -738,21 +719,23 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 				continue;
 			}
 			eprintf ("[bin.libs] Opening %s\n", lib);
-			ut64 baddr = r_io_map_location (r->io, 0x200000);
-			if (baddr != UT64_MAX) {
+			ut64 baddr = (r->io->bits == 64)? 0x60000000000LL: 0x60000000;	// do we really need io->bits?
+			if (r_io_map_locate (r->io, &baddr, 0x200000, 1)) {
 				r_core_file_loadlib (r, lib, baddr);
 			}
 		}
 		r_core_cmd0 (r, "obb 0;s entry0");
 		r_config_set_i (r->config, "bin.at", true);
-		eprintf ("[bin.libs] Linking imports...\n");
+		R_LOG_INFO ("[bin.libs] Linking imports");
 		RBinImport *imp;
-		RList *imports = r_bin_get_imports (r->bin);
+		const RList *imports = r_bin_get_imports (r->bin);
 		r_list_foreach (imports, iter, imp) {
 			// PLT finding
-			RFlagItem *impsym = r_flag_get (r->flags, sdb_fmt ("sym.imp.%s", imp->name));
+			char *flagname = r_str_newf ("sym.imp.%s", imp->name);
+			RFlagItem *impsym = r_flag_get (r->flags, flagname);
+			free (flagname);
 			if (!impsym) {
-				//eprintf ("Cannot find '%s' import in the PLT\n", imp->name);
+				//R_LOG_ERROR ("Cannot find '%s' import in the PLT", imp->name);
 				continue;
 			}
 			ut64 imp_addr = impsym->offset;
@@ -764,15 +747,12 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 				ut64 a = linkdata.addr;
 				ut64 b = imp_addr;
 				r_core_cmdf (r, "ax 0x%08"PFMT64x" 0x%08"PFMT64x, a, b);
-			} else {
-				eprintf ("NO\n");
 			}
 		}
 	}
 
 	//If type == R_BIN_TYPE_CORE, we need to create all the maps
-	if (plugin && binfile && plugin->file_type
-		 && plugin->file_type (binfile) == R_BIN_TYPE_CORE) {
+	if (plugin && binfile && plugin->file_type && plugin->file_type (binfile) == R_BIN_TYPE_CORE) {
 		ut64 sp_addr = (ut64)-1;
 		RIOMap *stack_map = NULL;
 
@@ -786,9 +766,9 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 		}
 		if (binfile->o->regstate) {
 			if (r_reg_arena_set_bytes (r->anal->reg, binfile->o->regstate)) {
-				eprintf ("Setting up coredump: Problem while setting the registers\n");
+				R_LOG_INFO ("Setting up coredump: Problem while setting the registers");
 			} else {
-				eprintf ("Setting up coredump: Registers have been set\n");
+				R_LOG_INFO ("Setting up coredump: Registers have been set");
 				const char *regname = r_reg_get_name (r->anal->reg, R_REG_NAME_SP);
 				if (regname) {
 					RRegItem *reg = r_reg_get (r->anal->reg, regname, -1);
@@ -829,13 +809,11 @@ R_API bool r_core_bin_load(RCore *r, const char *filenameuri, ut64 baddr) {
 		goto beach;
 	}
 beach:
+	r_flag_space_set (r->flags, "*");
 	return true;
 }
 
 R_API RIODesc *r_core_file_open_many(RCore *r, const char *file, int perm, ut64 loadaddr) {
-	const bool openmany = r_config_get_i (r->config, "file.openmany");
-	int opened_count = 0;
-	RListIter *fd_iter, *iter2;
 	RIODesc *fd;
 
 	RList *list_fds = r_io_open_many (r->io, file, perm, 0644);
@@ -844,27 +822,33 @@ R_API RIODesc *r_core_file_open_many(RCore *r, const char *file, int perm, ut64
 		r_list_free (list_fds);
 		return NULL;
 	}
-
-	r_list_foreach_safe (list_fds, fd_iter, iter2, fd) {
-		opened_count++;
-		if (openmany && opened_count > 1) {
-			// XXX - Open Many should limit the number of files
-			// loaded in io plugin area this needs to be more premptive
-			// like down in the io plugin layer.
-			// start closing down descriptors
-			r_list_delete (list_fds, fd_iter);
-			continue;
+	RListIter *iter;
+	RIODesc *first = NULL;
+	bool incloadaddr = loadaddr == 0;
+	// r_config_set_b (r->config, "io.va", false);
+	r_list_foreach (list_fds, iter, fd) {
+		if (fd->uri) {
+			if (!first) {
+				first = fd;
+			}
+			r_io_use_fd (r->io, fd->fd);
+			// r_core_file_open (r, fd->uri, perm, loadaddr);
+			ut64 sz = r_io_fd_size (r->io, fd->fd);
+			r_core_bin_load (r, fd->uri, loadaddr);
+			if (incloadaddr && sz != UT64_MAX) {
+				const int rest = (sz % 4096);
+				const int pillow = 0x4000;
+				loadaddr += sz + rest + pillow;
+			}
 		}
-		r_core_bin_load (r, fd->name, loadaddr);
 	}
-	return NULL;
+	return first;
 }
 
 /* loadaddr is r2 -m (mapaddr) */
 R_API RIODesc *r_core_file_open(RCore *r, const char *file, int flags, ut64 loadaddr) {
 	r_return_val_if_fail (r && file, NULL);
 	ut64 prev = r_time_now_mono ();
-	const bool openmany = r_config_get_i (r->config, "file.openmany");
 
 	if (!strcmp (file, "-")) {
 		file = "malloc://512";
@@ -873,12 +857,12 @@ R_API RIODesc *r_core_file_open(RCore *r, const char *file, int flags, ut64 load
 	if (!flags) {
 		flags = R_PERM_R;
 	}
-	r->io->bits = r->rasm->bits; // TODO: we need an api for this
+	r->io->bits = r->rasm->config->bits; // TODO: we need an api for this
 	RIODesc *fd = r_io_open_nomap (r->io, file, flags, 0644);
 	if (r_cons_is_breaked()) {
 		goto beach;
 	}
-	if (!fd && openmany) {
+	if (!fd) {
 		// XXX - make this an actual option somewhere?
 		fd = r_core_file_open_many (r, file, flags, loadaddr);
 		if (fd) {
@@ -916,7 +900,7 @@ R_API RIODesc *r_core_file_open(RCore *r, const char *file, int flags, ut64 load
 		if (r->dbg->h && r->dbg->h->canstep) {
 			swstep = false;
 		}
-		r_config_set_i (r->config, "dbg.swstep", swstep);
+		r_config_set_b (r->config, "dbg.swstep", swstep);
 		// Set the correct debug handle
 		if (fd->plugin && fd->plugin->isdbg) {
 			char *dh = r_str_ndup (file, (strstr (file, "://") - file));
