@@ -1,42 +1,50 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #include "r_core.h"
 
 static const char *help_message_ci[] = {
 	"Usage: ci", "[sil] ([obid])", "Compare two bin objects",
-	"cis", " 0", "Compare symbols with current `ob 1` with given obid (0)",
-	"cii", " 0", "Compare imports",
-	"cil", " 0", "Compare libraries",
+	"cis", " 0", "compare symbols with current `ob 1` with given obid (0)",
+	"cii", " 0", "compare imports",
+	"cil", " 0", "compare libraries",
+	NULL
+};
+
+static const char *help_msg_cmp[] = {
+	"Usage: cmp", " [file] [file]", "Compare two ($alias) files, and change $? value",
+	"cmp", " ls ls.old", "compare contents of given files",
+	"cmp", " $a $b", "same as above but using alias files",
 	NULL
 };
 
 static const char *help_msg_c[] = {
 	"Usage:", "c[?dfx] [argument]", " # Compare",
-	"c", " [string]", "Compare a plain with escaped chars string",
-	"c*", " [string]", "Same as above, but printing r2 commands instead",
-	"c1", " [addr]", "Compare 8 bits from current offset",
-	"c2", " [value]", "Compare a word from a math expression",
-	"c4", " [value]", "Compare a doubleword from a math expression",
-	"c8", " [value]", "Compare a quadword from a math expression",
-	"cat", " [file]", "Show contents of file (see pwd, ls)",
-	"cc", " [at]", "Compares in two hexdump columns of block size",
-	"ccc", " [at]", "Same as above, but only showing different lines",
-	"ccd", " [at]", "Compares in two disasm columns of block size",
-	"ccdd", " [at]", "Compares decompiler output (e cmd.pdc=pdg|pdd)",
+	"c", " [string]", "compare a plain with escaped chars string",
+	"c*", " [string]", "same as above, but printing r2 commands instead",
+	"c1", " [addr]", "compare byte at addr with current offset",
+	"c2", "[*] [value]", "compare word at offset with given value",
+	"c4", "[*] [value]", "compare doubleword at offset with given value",
+	"c8", "[*] [value]", "compare quadword at offset with given value",
+	"cat", " [file]", "show contents of file (see pwd, ls)",
+	"cc", " [at]", "compares in two hexdump columns of block size",
+	"ccc", " [at]", "same as above, but only showing different lines",
+	"ccd", " [at]", "compares in two disasm columns of block size",
+	"ccdd", " [at]", "compares decompiler output (e cmd.pdc=pdg|pdd)",
 	"cd", " [dir]", "chdir",
 	// "cc", " [offset]", "code bindiff current block against offset"
 	// "cD", " [file]", "like above, but using radiff -b",
-	"cf", " [file]", "Compare contents of file at current seek",
-	"cg", "[?] [o] [file]", "Graphdiff current file and [file]",
-	"ci", "[?] [obid] ([obid2])", "Compare two bin-objects (symbols, imports, ...)",
-	"cl|cls|clear", "", "Clear screen, (clear0 to goto 0, 0 only)",
-	"cu", "[?] [addr] @at", "Compare memory hexdumps of $$ and dst in unified diff",
-	"cud", " [addr] @at", "Unified diff disasm from $$ and given address",
-	"cv", "[1248] [hexpairs] @at", "Compare 1,2,4,8-byte (silent return in $?)",
-	"cV", "[1248] [addr] @at", "Compare 1,2,4,8-byte address contents (silent, return in $?)",
-	"cw", "[?] [us?] [...]", "Compare memory watchers",
-	"cx", " [hexpair]", "Compare hexpair string (use '.' as nibble wildcard)",
-	"cx*", " [hexpair]", "Compare hexpair string (output r2 commands)",
+	"cf", " [file]", "compare contents of file at current seek",
+	"cg", "[?] [o] [file]", "graphdiff current file and [file]",
+	"ci", "[?] [obid] ([obid2])", "compare two bin-objects (symbols, imports, ...)",
+	"cl|cls|clear", "", "clear screen, (clear0 to goto 0, 0 only)",
+	"cmp", " [file] [file]", "compare two files",
+	"cu", "[?] [addr] @at", "compare memory hexdumps of $$ and dst in unified diff",
+	"cud", " [addr] @at", "unified diff disasm from $$ and given address",
+	"cv", "[1248] [hexpairs] @at", "compare 1,2,4,8-byte (silent return in $?)",
+	"cV", "[1248] [addr] @at", "compare 1,2,4,8-byte address contents (silent, return in $?)",
+	"cw", "[?][*dqjru] [addr]", "compare memory watchers",
+	"cx", " [hexpair]", "compare hexpair string (use '.' as nibble wildcard)",
+	"cx*", " [hexpair]", "compare hexpair string (output r2 commands)",
 	"cX", " [addr]", "Like 'cc' but using hexdiff output",
 	NULL
 };
@@ -44,12 +52,13 @@ static const char *help_msg_c[] = {
 R_API void r_core_cmpwatch_free(RCoreCmpWatcher *w) {
 	free (w->ndata);
 	free (w->odata);
+	free (w->cmd);
 	free (w);
 }
 
-R_API RCoreCmpWatcher *r_core_cmpwatch_get(RCore *core, ut64 addr) {
-	RListIter *iter;
+R_API R_BORROW RCoreCmpWatcher *r_core_cmpwatch_get(RCore *core, ut64 addr) {
 	RCoreCmpWatcher *w;
+	RListIter *iter;
 	r_list_foreach (core->watchers, iter, w) {
 		if (addr == w->addr) {
 			return w;
@@ -58,103 +67,208 @@ R_API RCoreCmpWatcher *r_core_cmpwatch_get(RCore *core, ut64 addr) {
 	return NULL;
 }
 
-R_API int r_core_cmpwatch_add(RCore *core, ut64 addr, int size, const char *cmd) {
+R_API bool r_core_cmpwatch_add(RCore *core, ut64 addr, int size, const char *cmd) {
 	RCoreCmpWatcher *cmpw;
-	if (size < 1) {
-		return false;
-	}
+	bool found = false;
+	r_return_val_if_fail (core && cmd && size > 0, false);
+
 	cmpw = r_core_cmpwatch_get (core, addr);
 	if (!cmpw) {
-		cmpw = R_NEW (RCoreCmpWatcher);
+		cmpw = R_NEW0 (RCoreCmpWatcher);
 		if (!cmpw) {
 			return false;
 		}
 		cmpw->addr = addr;
+	} else {
+		free (cmpw->odata);
+		free (cmpw->ndata);
+		free (cmpw->cmd);
+		found = true;
 	}
 	cmpw->size = size;
-	snprintf (cmpw->cmd, sizeof (cmpw->cmd), "%s", cmd);
-	cmpw->odata = NULL;
-	cmpw->ndata = malloc (size);
-	if (!cmpw->ndata) {
+	cmpw->cmd = r_str_new (cmd);
+	if (!cmpw->cmd) {
 		free (cmpw);
 		return false;
 	}
-	r_io_read_at (core->io, addr, cmpw->ndata, size);
-	r_list_append (core->watchers, cmpw);
+
+	cmpw->odata = NULL;
+	cmpw->ndata = malloc (size);
+	if (!cmpw->ndata) {
+		r_core_cmpwatch_free (cmpw);
+		return false;
+	}
+
+	if (r_io_nread_at (core->io, addr, cmpw->ndata, size) != size) {
+		r_core_cmpwatch_free (cmpw);
+		return false;
+	}
+
+	// Don't append a duplicate
+	if (!found) {
+		r_list_append (core->watchers, cmpw);
+	}
+
 	return true;
 }
 
-R_API int r_core_cmpwatch_del(RCore *core, ut64 addr) {
-	int ret = false;
+R_API bool r_core_cmpwatch_del(RCore *core, ut64 addr) {
+	bool ret = false;
 	RCoreCmpWatcher *w;
 	RListIter *iter, *iter2;
-	r_list_foreach_safe (core->watchers, iter, iter2, w) {
-		if (w->addr == addr || addr == UT64_MAX) {
+
+	if (addr == UT64_MAX) { // match all
+		r_list_foreach_safe (core->watchers, iter, iter2, w) {
 			r_list_delete (core->watchers, iter);
 			ret = true;
 		}
+		return ret;
+	}
+
+	// Can't use r_core_cmpwatch_get() here since we need the iter
+	r_list_foreach_safe (core->watchers, iter, iter2, w) {
+		if (w->addr == addr) {
+			/* Only one watcher per address - we can leave early */
+			r_list_delete (core->watchers, iter);
+			ret = true;
+			break;
+		}
 	}
 	return ret;
 }
 
-R_API int r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
-	char cmd[128];
+R_API bool r_core_cmpwatch_show(RCore *core, ut64 addr, int mode) {
 	RListIter *iter;
 	RCoreCmpWatcher *w;
-	r_list_foreach (core->watchers, iter, w) {
-		int is_diff = w->odata? memcmp (w->odata, w->ndata, w->size): 0;
-		switch (mode) {
-		case '*':
-			r_cons_printf ("cw 0x%08"PFMT64x " %d %s%s\n",
-				w->addr, w->size, w->cmd, is_diff? " # differs": "");
-			break;
-		case 'd': // diff
-			if (is_diff) {
-				r_cons_printf ("0x%08"PFMT64x " has changed\n", w->addr);
-			}
-			break;
-		case 'o': // old contents
-		// use tmpblocksize
-		default:
-			r_cons_printf ("0x%08"PFMT64x "%s\n", w->addr, is_diff? " modified": "");
-			snprintf (cmd, sizeof (cmd), "%s@%"PFMT64d "!%d",
-				w->cmd, w->addr, w->size);
-			r_core_cmd0 (core, cmd);
-			break;
-		}
-	}
-	return false;
-}
+	PJ *pj = NULL;
+	bool ret = false;
 
-R_API int r_core_cmpwatch_update(RCore *core, ut64 addr) {
-	RCoreCmpWatcher *w;
-	RListIter *iter;
-	r_list_foreach (core->watchers, iter, w) {
-		free (w->odata);
-		w->odata = w->ndata;
-		w->ndata = malloc (w->size);
-		if (!w->ndata) {
+	if (mode == 'j') {
+		pj = r_core_pj_new (core);
+		if (!pj) {
 			return false;
 		}
-		r_io_read_at (core->io, w->addr, w->ndata, w->size);
+		pj_a (pj);
 	}
-	return !r_list_empty (core->watchers);
+
+	r_list_foreach (core->watchers, iter, w) {
+		bool changed = w->odata? memcmp (w->odata, w->ndata, w->size): false;
+		if (addr != UT64_MAX && addr != w->addr) {
+			continue;
+		}
+		switch (mode) {
+		case '*': // print watchers as r2 commands
+			r_cons_printf ("cw 0x%08" PFMT64x " %d %s%s\n",
+					w->addr, w->size, w->cmd,
+					changed? " # differs": "");
+			break;
+		case 'q': // quiet
+			if (changed) {
+				r_cons_printf ("0x%08" PFMT64x " has changed\n", w->addr);
+			}
+			break;
+		case 'j': // "cw"
+			{
+			char *cmd_output = r_core_cmd_strf (core, "%s %d @%" PFMT64d,
+					w->cmd, w->size, w->addr);
+			pj_o (pj);
+			pj_kn (pj, "addr", w->addr);
+			pj_kb (pj, "changed", changed);
+			pj_ks (pj, "cmd", w->cmd);
+			pj_ks (pj, "cmd_out", r_str_get (cmd_output));
+			pj_end (pj);
+			free (cmd_output);
+			}
+			break;
+		default:
+			r_cons_printf ("0x%08" PFMT64x "%s\n", w->addr, changed? " modified": "");
+			r_core_cmdf (core, "%s %d @%" PFMT64d, w->cmd, w->size, w->addr);
+			break;
+		}
+
+		ret = true;
+	}
+
+	if (pj) {
+		char *out;
+		pj_end (pj);
+		out = pj_drain (pj);
+		r_cons_println (out);
+		free (out);
+	}
+
+	return ret;
 }
 
-R_API int r_core_cmpwatch_revert(RCore *core, ut64 addr) {
+static bool update_watcher(RIO *io, RCoreCmpWatcher *w) {
+	r_return_val_if_fail (io && w, false);
+
+	free (w->odata);
+	w->odata = w->ndata;
+	w->ndata = malloc (w->size);
+	if (!w->ndata) {
+		return false;
+	}
+	r_io_read_at (io, w->addr, w->ndata, w->size);
+	return true;
+}
+
+/* Replace old data with current new data, then read IO into new data */
+R_API bool r_core_cmpwatch_update(RCore *core, ut64 addr) {
 	RCoreCmpWatcher *w;
-	int ret = false;
 	RListIter *iter;
+	bool ret = false;
+
+	if (addr != UT64_MAX) {
+		w = r_core_cmpwatch_get (core, addr);
+		if (w) {
+			return update_watcher (core->io, w);
+		}
+		return false;
+	}
+
 	r_list_foreach (core->watchers, iter, w) {
-		if (w->addr == addr || addr == UT64_MAX) {
-			if (w->odata) {
-				free (w->ndata);
-				w->ndata = w->odata;
-				w->odata = NULL;
-				ret = true;
-			}
+		if (update_watcher (core->io, w)) {
+			ret = true;
 		}
 	}
+
+	return ret;
+}
+
+static bool revert_watcher(RCoreCmpWatcher *w) {
+	r_return_val_if_fail (w, false);
+	if (w->odata) {
+		free (w->ndata);
+		w->ndata = w->odata;
+		w->odata = NULL;
+	}
+
+	return true;
+}
+
+/* Mark the current old state as new, discarding the original new state */
+R_API bool r_core_cmpwatch_revert(RCore *core, ut64 addr) {
+	RCoreCmpWatcher *w;
+	RListIter *iter;
+	bool ret = false;
+
+	if (addr != UT64_MAX) {
+		w = r_core_cmpwatch_get (core, addr);
+		if (w) {
+			return revert_watcher (w);
+		}
+
+		return false;
+	}
+
+
+	r_list_foreach (core->watchers, iter, w) {
+		if (revert_watcher (w)) {
+			ret = true;
+		}
+	}
+
 	return ret;
 }
 
@@ -304,59 +418,186 @@ static int radare_compare(RCore *core, const ut8 *f, const ut8 *d, int len, int
 	return len - eq;
 }
 
-static void cmd_cmp_watcher(RCore *core, const char *input) {
-	char *p, *q, *r = NULL;
-	int size = 0;
-	ut64 addr = 0;
+/* Returns 0 if operation succeeded, 1 otherwise */
+static int cmd_cmp_watcher(RCore *core, const char *input) {
+	static const char *help_msg_cw[] = {
+		"Usage: cw", "[args]", "Manage compare watchers; See if and how memory changes",
+		"cw??", "", "Show more info about watchers",
+		"cw ", "addr sz cmd", "Add a compare watcher",
+		"cw", "[*qj] [addr]", "Show compare watchers (*=r2 commands, q=quiet, j=json)",
+		"cwd", " [addr]", "Delete watcher",
+		"cwr", " [addr]", "Revert watcher",
+		"cwu", " [addr]", "Update watcher",
+		NULL
+	};
+	static const char *verbose_help_cw =
+		"Watchers are used to record memory at 2 different points in time, then\n"
+		"report if and how it changed. First, create one with `cw addr sz cmd`. This\n"
+		"will record sz bytes at addr. To record the second state, use `cwu`. Now, when\n"
+		"you run `cw`, the watcher will report if the bytes changed and run the command given\n"
+		"at creation with the size and address. You may overwrite any watcher by creating\n"
+		"another at the same address. This will discard the existing watcher completely.\n"
+		"\n"
+		"When you create a watcher, the data read from memory is marked as \"new\". Updating\n"
+		"the watcher with `cwu` will mark this data as \"old\", and then read the \"new\" data.\n"
+		"`cwr` will mark the current \"old\" state as being \"new\", letting you reuse it as\n"
+		"your new base state when updating with `cwu`. Any existing \"new\" state from running\n"
+		"`cwu` previously is lost in this process. Watched memory areas may overlap with no ill\n"
+		"effects, but may have unexpected results if you update some but not others.\n"
+		"\n"
+		"Showing a watcher without updating will still run the command, but it will not report\n"
+		"changes.\n"
+		"\n"
+		"When an address is an optional argument, the command will apply to all watchers if\n"
+		"you don't pass one.\n"
+		"\n"
+		"For more details and examples, see section 4.10 of the radare2 book.\n";
+
+	ut64 addr = UT64_MAX;
+	int ret = 0;
+
 	switch (*input) {
-	case ' ':
-		p = strdup (input + 1);
-		q = strchr (p, ' ');
-		if (q) {
-			*q++ = 0;
-			addr = r_num_math (core->num, p);
-			r = strchr (q, ' ');
-			if (r) {
-				*r++ = 0;
-				size = atoi (q);
-			}
-			r_core_cmpwatch_add (core, addr, size, r);
-			// eprintf ("ADD (%llx) %d (%s)\n", addr, size, r);
-		} else {
-			eprintf ("Missing parameters\n");
+	case ' ': { // "cw "
+		int argc;
+		char **argv = r_str_argv (input + 1, &argc);
+		if (!argv) {
+			return 1;
 		}
-		free (p);
+		if (argc == 1) { // "cw [addr]"
+			addr = r_num_math (core->num, argv[0]);
+			r_core_cmpwatch_show (core, addr, 0);
+		} else if (argc == 3) { // "cw addr sz cmd"
+			addr = r_num_math (core->num, argv[0]);
+			ut64 size = r_num_math (core->num, argv[1]);
+
+			if (size < 1) {
+				ret = 1;
+				eprintf ("Can't create a watcher with size less than 1.\n");
+				goto out_free_argv;
+			}
+			if (size > INT_MAX) {
+				ret = 1;
+				eprintf ("Can't create a watcher with size larger than an int.\n");
+				goto out_free_argv;
+			}
+
+			if (!r_core_cmpwatch_add (core, addr, (int)size, argv[2])) {
+				ret = 1;
+				eprintf ("Failed to add watcher.\n");
+			}
+		} else {
+			r_core_cmd_help_match (core, help_msg_cw, "cw ", true);
+		}
+
+out_free_argv:
+		r_str_argv_free (argv);
 		break;
-	case 'r':
-		addr = input[1]? r_num_math (core->num, input + 1): UT64_MAX;
-		r_core_cmpwatch_revert (core, addr);
-		break;
-	case 'u':
-		addr = input[1]? r_num_math (core->num, input + 1): UT64_MAX;
-		r_core_cmpwatch_update (core, addr);
-		break;
-	case '*':
-		r_core_cmpwatch_show (core, UT64_MAX, '*');
-		break;
-	case '\0':
-		r_core_cmpwatch_show (core, UT64_MAX, 0);
-		break;
-	case '?': {
-			  const char *help_message[] = {
-				  "Usage: cw", "", "Watcher commands",
-				  "cw", "", "List all compare watchers",
-				  "cw", " addr", "List all compare watchers",
-				  "cw", " addr sz cmd", "Add a memory watcher",
-				  // "cws", " [addr]", "Show watchers",
-				  "cw", "*", "List compare watchers in r2 cmds",
-				  "cwr", " [addr]", "Reset/revert watchers",
-				  "cwu", " [addr]", "Update watchers",
-				  NULL
-			  };
-			  r_core_cmd_help (core, help_message);
-		  }
-		  break;
 	}
+	case 'd': // "cwd"
+		if (input[1] == '?') {
+			r_core_cmd_help_match (core, help_msg_cw, "cwd", true);
+			return 0;
+		}
+
+		if (input[1]) {
+			addr = r_num_math (core->num, input + 2);
+		}
+
+		if (addr == UT64_MAX &&
+				!r_cons_yesno ('n', "Delete all watchers? (y/N)")) {
+			return 1;
+		}
+
+		if (!r_core_cmpwatch_del (core, addr) && addr) {
+			ret = 1;
+			if (addr == UT64_MAX) {
+				eprintf ("No watchers exist.\n");
+			} else {
+				eprintf ("No watcher exists at address %" PFMT64x ".\n", addr);
+			}
+		}
+		break;
+	case 'r': // "cwr"
+		if (input[1] == '?') {
+			r_core_cmd_help_match (core, help_msg_cw, "cwr", true);
+			return 0;
+		}
+
+		if (input[1]) {
+			addr = r_num_math (core->num, input + 2);
+		}
+
+		if (addr == UT64_MAX &&
+				!r_cons_yesno ('n', "Revert all watchers? (y/N)")) {
+			return 1;
+		}
+
+		if (!r_core_cmpwatch_revert (core, addr)) {
+			ret = 0;
+			if (addr == UT64_MAX) {
+				eprintf ("No watchers exist.\n");
+			} else {
+				eprintf ("No watcher exists at address %" PFMT64x ".\n", addr);
+			}
+		}
+		break;
+	case 'u': // "cwu"
+		if (input[1] == '?') {
+			r_core_cmd_help_match (core, help_msg_cw, "cwu", true);
+			return 0;
+		}
+
+		if (input[1]) {
+			addr = r_num_math (core->num, input + 2);
+		}
+
+		if (!r_core_cmpwatch_update (core, addr)) {
+			ret = 1;
+			if (addr == UT64_MAX) {
+				eprintf ("No watchers exist.\n");
+			} else {
+				eprintf ("No watcher exists at address %" PFMT64x ".\n", addr);
+			}
+		}
+		break;
+	case '*': // "cw*"
+	case 'q': // "cwq"
+	case 'j': // "cwj"
+	case '\0': { // "cw"
+		int mode = *input;
+		if (*input && input[1]) {
+			addr = r_num_math (core->num, input + 2);
+		}
+
+		if (!r_core_cmpwatch_show (core, addr, mode)) {
+			ret = 1;
+
+			/* Skip error message for json, it will still show [] */
+			if (mode == 'j') {
+				break;
+			}
+
+			if (addr == UT64_MAX) {
+				eprintf ("No watchers exist.\n");
+			} else {
+				eprintf ("No watcher exists at address %" PFMT64x ".\n", addr);
+			}
+		}
+		break;
+	}
+	case '?': // "cw?"
+	default:
+		if (input[1] == '?') {
+			// this command really needs explaining
+			// so it's strongly signposted
+			r_cons_printf ("%s", verbose_help_cw);
+		} else {
+			r_core_cmd_help (core, help_msg_cw);
+		}
+		break;
+	}
+
+	return ret;
 }
 
 static int cmd_cmp_disasm(RCore *core, const char *input, int mode) {
@@ -543,43 +784,59 @@ static int cmd_cp(void *data, const char *input) {
 	return false;
 }
 
-static void __core_cmp_bits(RCore *core, ut64 addr) {
-	const bool scr_color = r_config_get_i (core->config, "scr.color");
+/* Show the bits for the bytes at addr and offset.
+ * If scr.color is enabled, when bytes differ 1 is colored graph_true and 0 is
+ * colored graph_false.
+ */
+static int cmp_bits(RCore *core, ut64 addr) {
+	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
+	const bool use_color = r_config_get_b (core->config, "scr.color");
+	const char *color_end = use_color? Color_RESET: "";
 	int i;
 	ut8 a, b;
-	r_io_read_at (core->io, core->offset, &a, 1);
-	r_io_read_at (core->io, addr, &b, 1);
-	RConsPrintablePalette *pal = &r_cons_singleton ()->context->pal;
-	const char *color = scr_color? pal->offset: "";
-	const char *color_end = scr_color? Color_RESET: "";
-	if (r_config_get_i (core->config, "hex.header")) {
-		char *n = r_str_newf ("0x%08"PFMT64x, core->offset);
-		const char *extra = r_str_pad (' ', strlen (n) - 10);
-		free (n);
-		r_cons_printf ("%s- offset -%s  7 6 5 4 3 2 1 0%s\n", color, extra, color_end);
-	}
-	color = scr_color? pal->graph_false: "";
-	color_end = scr_color? Color_RESET: "";
+	bool a_bits[8], b_bits[8];
+	const char *a_colors[8], *b_colors[8];
 
-	r_cons_printf ("%s0x%08"PFMT64x"%s  ", color, core->offset, color_end);
-	for (i = 7; i >= 0; i--) {
-		bool b0 = (a & 1<<i)? 1: 0;
-		bool b1 = (b & 1<<i)? 1: 0;
-		color = scr_color? (b0 == b1)? "": b0? pal->graph_true:pal->graph_false: "";
-		color_end = scr_color ? Color_RESET: "";
-		r_cons_printf ("%s%d%s ", color, b0, color_end);
+	r_io_nread_at (core->io, core->offset, &a, 1);
+	r_io_nread_at (core->io, addr, &b, 1);
+
+	/* Print offset header if enabled */
+	if (r_config_get_i (core->config, "hex.header")) {
+		const char *color = use_color? pal->offset: "";
+		char *n = r_str_newf ("0x%08" PFMT64x, core->offset);
+		const char *padding = r_str_pad (' ', strlen (n) - 10);
+		free (n);
+		r_cons_printf ("%s- offset -%s  7 6 5 4 3 2 1 0%s\n", color, padding, color_end);
 	}
-	color = scr_color? pal->graph_true: "";
-	color_end = scr_color? Color_RESET: "";
-	r_cons_printf ("\n%s0x%08"PFMT64x"%s  ", color, addr, color_end);
+
+	/* Set up bits and colors */
 	for (i = 7; i >= 0; i--) {
-		bool b0 = (a & 1<<i)? 1: 0;
-		bool b1 = (b & 1<<i)? 1: 0;
-		color = scr_color? (b0 == b1)? "": b1? pal->graph_true: pal->graph_false: "";
-		color_end = scr_color ? Color_RESET: "";
-		r_cons_printf ("%s%d%s ", color, b1, color_end);
+		a_bits[i] = a & (1 << i);
+		b_bits[i] = b & (1 << i);
+
+		if (use_color && a_bits[i] != b_bits[i]) {
+			a_colors[i] = a_bits[i]? pal->graph_true: pal->graph_false;
+			b_colors[i] = b_bits[i]? pal->graph_true: pal->graph_false;
+		} else {
+			a_colors[i] = "";
+			b_colors[i] = "";
+		}
+	}
+
+	r_cons_printf ("%s0x%08" PFMT64x "%s  ", use_color? pal->graph_false: "", core->offset, color_end);
+	for (i = 7; i >= 0; i--) {
+		r_cons_printf ("%s%d%s%s", a_colors[i], a_bits[i], color_end, i? " ": "");
+	}
+
+	r_cons_printf ("\n%s0x%08" PFMT64x "%s  ", use_color? pal->graph_true: "", addr, color_end);
+	for (i = 7; i >= 0; i--) {
+		r_cons_printf ("%s%d%s%s", b_colors[i], b_bits[i], color_end, i? " ": "");
 	}
 	r_cons_newline ();
+
+	// 0 if equal, 1 if not equal
+	// same return pattern as ?==
+	return a != b;
 }
 
 static const RList *symbols_of(RCore *core, int id0) {
@@ -739,30 +996,84 @@ static void _core_cmp_info(RCore *core, const char *input) {
 	}
 }
 
+static void cmd_curl(RCore *core, const char *arg) {
+	if (r_sys_getenv_asbool ("R2_CURL")) {
+		r_sys_cmdf ("curl %s", arg);
+	} else {
+		if (strstr (arg, "http") && strstr (arg, "://")) {
+			int len;
+			char *s = r_socket_http_get (arg, NULL, &len);
+			if (s) {
+				r_cons_write (s, len);
+				free (s);
+			}
+		} else {
+			eprintf ("Usage: curl [http-url]\n");
+		}
+	}
+}
+
+static char *myslurp(RCore *core, const char *a, ut64 *sz) {
+	if (*a == '$') {
+		RCmdAliasVal *v = r_cmd_alias_get (core->rcmd, a + 1);
+		if (v) {
+			char *v_str = r_cmd_alias_val_strdup (v);
+			*sz = strlen (v_str);
+			return v_str;
+		}
+	} else {
+		size_t ssz;
+		char *rc = r_file_slurp (a, &ssz);
+		*sz = ssz;
+		return rc;
+	}
+	*sz = 0;
+	return NULL;
+}
+
+static int cmd_cmp_posix(RCore *core, const char *a, const char *b) {
+	ut64 sa, sb;
+	char *ba = myslurp (core, a, &sa);
+	char *bb = myslurp (core, b, &sb);
+	int res = 0;
+	if (!ba || !bb) {
+		eprintf ("One or more files can't be read.\n");
+		res = 1;
+	} else {
+		if (sa == sb) {
+			res = memcmp (ba, bb, sa)? 1: 0;
+		} else {
+			res = 1;
+		}
+	}
+	free (ba);
+	free (bb);
+	// return 0 if both files are the same
+	return res;
+}
+
 static int cmd_cmp(void *data, const char *input) {
-	static char *oldcwd = NULL;
+	static R_TH_LOCAL char *oldcwd = NULL;
 	int ret = 0, i, mode = 0;
 	RCore *core = (RCore *)data;
 	ut64 val = UT64_MAX;
 	char *filled;
 	ut8 *buf;
-	ut16 v16;
-	ut32 v32;
-	ut64 v64;
+	utAny wordcmp;
 	FILE *fd;
 	const ut8* block = core->block;
 
 	switch (*input) {
-	case 'p':
+	case 'p': // "cp"
 		return cmd_cp (data, input + 1);
 		break;
-	case 'a': // "cat"
-		if (input[1] == 't') {
+	case 'a': // "ca"
+		if (input[1] == 't') { // "cat"
 			const char *path = r_str_trim_head_ro (input + 2);
 			if (*path == '$' && !path[1]) {
 				eprintf ("No alias name given.\n");
 			} else if (*path == '$') {
-				RCmdAliasVal *v = r_cmd_alias_get (core->rcmd, path+1);
+				RCmdAliasVal *v = r_cmd_alias_get (core->rcmd, path + 1);
 				if (v) {
 					char *v_str = r_cmd_alias_val_strdup (v);
 					r_cons_println (v_str);
@@ -770,7 +1081,7 @@ static int cmd_cmp(void *data, const char *input) {
 				} else {
 					eprintf ("No such alias \"$%s\"\n", path+1);
 				}
-			} else {
+			} else if (*path) {
 				if (r_fs_check (core->fs, path)) {
 					r_core_cmdf (core, "mg %s", path);
 				} else {
@@ -780,13 +1091,17 @@ static int cmd_cmp(void *data, const char *input) {
 						free (res);
 					}
 				}
+			} else {
+				r_core_cmd_help_match (core, help_msg_c, "cat", true);
 			}
+		} else { // "ca"
+			r_core_cmd_help_match (core, help_msg_c, "cat", true);
 		}
 		break;
 	case 'w':
-		cmd_cmp_watcher (core, input + 1);
+		return cmd_cmp_watcher (core, input + 1);
 		break;
-	case '*':
+	case '*': // c*"
 		if (!input[2]) {
 			eprintf ("Usage: cx* 00..22'\n");
 			return 0;
@@ -795,16 +1110,14 @@ static int cmd_cmp(void *data, const char *input) {
 		val = radare_compare (core, block, (ut8 *) input + 2,
 			strlen (input + 2) + 1, '*');
 		break;
-	case ' ':
-	{
+	case ' ': { // "c"
 		char *str = strdup (input + 1);
 		int len = r_str_unescape (str);
 		val = radare_compare (core, block, (ut8 *) str, len, 0);
 		free (str);
+		break;
 	}
-	break;
-	case 'j':
-	{
+	case 'j': // "cj"
 		if (input[1] != ' ') {
 			eprintf ("Usage: cj [string]\n");
 		} else {
@@ -813,9 +1126,8 @@ static int cmd_cmp(void *data, const char *input) {
 			val = radare_compare (core, block, (ut8 *) str, len, 'j');
 			free (str);
 		}
-	}
-	break;
-	case 'x':
+		break;
+	case 'x': // "cx"
 		switch (input[1]) {
 		case ' ':
 			mode = 0;
@@ -869,7 +1181,7 @@ static int cmd_cmp(void *data, const char *input) {
 			free (buf);
 		}
 		break;
-	case 'f':
+	case 'f': // "cf"
 		if (input[1] != ' ') {
 			eprintf ("Please. use 'cf [file]'\n");
 			return false;
@@ -939,21 +1251,41 @@ static int cmd_cmp(void *data, const char *input) {
 			free (home);
 		}
 		break;
-	case '1': // "c1"
-		__core_cmp_bits (core, r_num_math (core->num, input + 1));
-		break;
-	case '2': // "c2"
-		v16 = (ut16) r_num_math (core->num, input + 1);
-		val = radare_compare (core, block, (ut8 *) &v16, sizeof (v16), 0);
-		break;
-	case '4': // "c4"
-		v32 = (ut32) r_num_math (core->num, input + 1);
-		val = radare_compare (core, block, (ut8 *) &v32, sizeof (v32), 0);
-		break;
-	case '8': // "c8"
-		v64 = (ut64) r_num_math (core->num, input + 1);
-		val = radare_compare (core, block, (ut8 *) &v64, sizeof (v64), 0);
+	case '1':
+	case '2':
+	case '4':
+	case '8': {
+		const char width = *input++;
+		const char mode = *input == '*'? '*': 0;
+		const char *arg;
+		utAny cmp_val;
+
+		arg = *input? r_str_trim_head_ro (input + 1): NULL;
+
+		if (input[0] == '?' || R_STR_ISEMPTY (arg)) {
+			r_core_cmd_help_match_spec (core, help_msg_c, "c", width, true);
+			break;
+		}
+
+		if (width == '1') {
+			if (mode == '*') {
+				R_LOG_ERROR ("c1 does not support * mode");
+				r_core_cmd_help_match (core, help_msg_c, "c1", true);
+			} else {
+				val = cmp_bits (core, r_num_math (core->num, arg));
+			}
+		} else if (width == '2') {
+			cmp_val.v16 = (ut16) r_num_math (core->num, arg);
+			val = radare_compare (core, block, (ut8 *) &cmp_val.v16, sizeof (wordcmp.v16), mode);
+		} else if (width == '4') {
+			cmp_val.v32 = (ut32) r_num_math (core->num, arg);
+			val = radare_compare (core, block, (ut8 *) &cmp_val.v32, sizeof (wordcmp.v32), mode);
+		} else if (width == '8') {
+			cmp_val.v64 = r_num_math (core->num, arg);
+			val = radare_compare (core, block, (ut8 *) &cmp_val.v64, sizeof (wordcmp.v64), mode);
+		}
 		break;
+	}
 	case 'c': // "cc"
 		if (input[1] == '?') { // "cc?"
 			r_core_cmd0 (core, "c?~cc");
@@ -1018,8 +1350,8 @@ static int cmd_cmp(void *data, const char *input) {
 			const char *help_message[] = {
 				"Usage: cg", "", "Graph code commands",
 				"cg", "", "diff ratio among functions (columns: off-A, match-ratio, off-B)",
-				"cgf", "[fcn]", "Compare functions (curseek vs fcn)",
-				"cgo", "", "Opcode-bytes code graph diff",
+				"cgf", "[fcn]", "compare functions (curseek vs fcn)",
+				"cgo", "", "opcode-bytes code graph diff",
 				NULL
 			};
 			r_core_cmd_help (core, help_message);
@@ -1060,6 +1392,11 @@ static int cmd_cmp(void *data, const char *input) {
 	}
 	break;
 	case 'u': // "cu"
+		if (r_str_startswith (input, "url")) {
+			const char *arg = r_str_trim_head_ro (input + 3);
+			cmd_curl (core, arg);
+			break;
+		}
 		switch (input[1]) {
 		case '.':
 		case ' ':
@@ -1081,13 +1418,13 @@ static int cmd_cmp(void *data, const char *input) {
 		default: {
 			const char *help_msg[] = {
 				"Usage: cu", " [offset]", "# Prints unified comparison to make hexpatches",
-				"cu", " $$+1 > p", "Compare hexpairs from  current seek and +1",
-				"cu1", " $$+1 > p", "Compare bytes from current seek and +1",
-				"cu2", " $$+1 > p", "Compare words (half, 16bit) from current seek and +1",
-				"cu4", " $$+1 > p", "Compare dwords from current seek and +1",
-				"cu8", " $$+1 > p", "Compare qwords from current seek and +1",
-				"cud", " $$+1 > p", "Compare disasm current seek and +1",
-				"wu", " p", "Apply unified hex patch (see output of cu)",
+				"cu", " $$+1 > p", "compare hexpairs from  current seek and +1",
+				"cu1", " $$+1 > p", "compare bytes from current seek and +1",
+				"cu2", " $$+1 > p", "compare words (half, 16bit) from current seek and +1",
+				"cu4", " $$+1 > p", "compare dwords from current seek and +1",
+				"cu8", " $$+1 > p", "compare qwords from current seek and +1",
+				"cud", " $$+1 > p", "compare disasm current seek and +1",
+				"wu", " p", "apply unified hex patch (see output of cu)",
 				NULL
 			};
 			r_core_cmd_help (core, help_msg);
@@ -1112,41 +1449,47 @@ static int cmd_cmp(void *data, const char *input) {
 		switch (sz) {
 		case '1': { // "cv1"
 			ut8 n = (ut8) r_num_math (core->num, input + 2);
-			core->num->value = 1;
 			if (block[0] == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
-				core->num->value = 0;
+				r_core_return_value (core, 0);
+			} else {
+				r_core_return_value (core, 1);
 			}
 			break;
 		}
 		case '2': { // "cv2"
 			ut16 n = (ut16) r_num_math (core->num, input + 2);
-			core->num->value = 1;
 			if (core->blocksize >= 2 && *(ut16*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
-				core->num->value = 0;
+				r_core_return_value (core, 0);
+			} else {
+				r_core_return_value (core, 1);
 			}
 			break;
 		}
 		case '4': { // "cv4"
 			ut32 n = (ut32) r_num_math (core->num, input + 2);
-			core->num->value = 1;
 			if (core->blocksize >= 4 && *(ut32*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
-				core->num->value = 0;
+				r_core_return_value (core, 0);
+			} else {
+				r_core_return_value (core, 1);
 			}
 			break;
 		}
 		case '8': { // "cv8"
 			ut64 n = (ut64) r_num_math (core->num, input + 2);
-			core->num->value = 1;
 			if (core->blocksize >= 8 && *(ut64*)block == n) {
 				r_cons_printf ("0x%08"PFMT64x "\n", core->offset);
-				core->num->value = 0;
+				r_core_return_value (core, 0);
+			} else {
+				r_core_return_value (core, 1);
 			}
 			break;
 		}
 		default:
+			r_core_return_value (core, 1);
+			// fallthrough
 		case '?':
 			eprintf ("Usage: cv[1248] [num]\n"
 				"Show offset if current value equals to the one specified\n"
@@ -1170,14 +1513,18 @@ static int cmd_cmp(void *data, const char *input) {
 			}
 		} else if (sz == '?') {
 			eprintf ("Usage: cV[1248] [addr] @ addr2\n"
-				"Compare n bytes from one address to current one and return in $? 0 or 1\n");
+				"compare n bytes from one address to current one and return in $? 0 or 1\n");
 		}
 		sz -= '0';
 		if (sz > 0) {
 			ut64 at = r_num_math (core->num, input + 2);
 			ut8 buf[8] = {0};
-			r_io_read_at (core->io, at, buf, sizeof (buf));
-			core->num->value = memcmp (buf, core->block, sz)? 1: 0;
+			if (r_io_read_at (core->io, at, buf, sizeof (buf)) < 1) {
+				r_core_return_value (core, -1);
+				break;
+			}
+			int val = memcmp (buf, core->block, R_MIN (core->blocksize, sz))? 1: 0;
+			r_core_return_value (core, val);
 		}
 		break;
 	}
@@ -1191,12 +1538,27 @@ static int cmd_cmp(void *data, const char *input) {
 			r_cons_clear00 ();
 		}
 		break;
+	case 'm': // "cmp"
+		if (input[1] != 'p' || strchr (input, '?')) {
+			r_core_cmd_help (core, help_msg_cmp);
+		} else {
+			int argc;
+			char **argv = r_str_argv (r_str_trim_head_ro (input + 2), &argc);
+			if (argc == 2) {
+				int res = cmd_cmp_posix (core, argv[0], argv[1]);
+				r_core_return_value (core, res);
+			} else {
+				r_core_cmd_help (core, help_msg_cmp);
+			}
+			free (argv);
+		}
+		break;
 	default:
 		r_core_cmd_help (core, help_msg_c);
 		break;
 	}
 	if (val != UT64_MAX) {
-		core->num->value = val;
+		r_core_return_value (core, val);
 	}
 	return 0;
 }
