@@ -67,12 +67,12 @@ static const char *help_msg_CC[] = {
 static const char *help_msg_CL[] = {
 	"Usage: CL", ".j-", "@addr - manage code-line references (loaded via bin.dbginfo and shown when asm.dwarf)",
 	"CL", "", "list all code line information (virtual address <-> source file:line)",
-	"CLj", "", "Same as above but in JSON format (See dir.source to change the path to find the referenced lines)",
-	"CL*", "", "Same as above but in r2 commands format",
-	"CL.", "", "Show list all code line information (virtual address <-> source file:line)",
-	"CL-", "*", "Remove all the cached codeline information",
-	"CL", " addr file:line", "Register new file:line source details, r2 will slurp the line",
-	"CL", " addr base64:text", "Register new source details for given address using base64",
+	"CLj", "", "same as above but in JSON format (See dir.source to change the path to find the referenced lines)",
+	"CL*", "", "same as above but in r2 commands format",
+	"CL.", "", "show list all code line information (virtual address <-> source file:line)",
+	"CL-", "*", "remove all the cached codeline information",
+	"CL", " addr file:line", "register new file:line source details, r2 will slurp the line",
+	"CL", " addr base64:text", "register new source details for given address using base64",
 	NULL
 };
 
@@ -123,7 +123,7 @@ static const char *help_msg_Cvb[] = {
 	"Cvb", "", "list all base pointer args/vars comments in human friendly format",
 	"Cvb*", "", "list all base pointer args/vars comments in r2 format",
 	"Cvb-", "[name]", "delete comments for var/arg at current offset for base pointer",
-	"Cvb", " [name]", "Show comments for var/arg at current offset for base pointer",
+	"Cvb", " [name]", "show comments for var/arg at current offset for base pointer",
 	"Cvb", " [name] [comment]", "add/append comment for the variable with the current name",
 	"Cvb!", "[name]", "edit comment using cfg editor",
 	NULL
@@ -135,7 +135,7 @@ static const char *help_msg_Cvr[] = {
 	"Cvr", "", "list all register based args comments in human friendly format",
 	"Cvr*", "", "list all register based args comments in r2 format",
 	"Cvr-", "[name]", "delete comments for register based arg for that name",
-	"Cvr", "[name]", "Show comments for register based arg for that name",
+	"Cvr", "[name]", "show comments for register based arg for that name",
 	"Cvr", "[name] [comment]", "add/append comment for the variable",
 	"Cvr!", "[name]", "edit comment using cfg editor",
 	NULL
@@ -146,7 +146,7 @@ static const char *help_msg_Cvs[] = {
 	"Cvs!", "[name]", "edit comment using cfg editor",
 	"Cvs", "", "list all stack based args/vars comments in human friendly format",
 	"Cvs", "[name] [comment]", "add/append comment for the variable",
-	"Cvs", "[name]", "Show comments for stack pointer var/arg with that name",
+	"Cvs", "[name]", "show comments for stack pointer var/arg with that name",
 	"Cvs*", "", "list all stack based args/vars comments in r2 format",
 	"Cvs-", "[name]", "delete comments for stack pointer var/arg with that name",
 	"Cvs?", "", "show this help",
@@ -157,7 +157,7 @@ static int remove_meta_offset(RCore *core, ut64 offset) {
 	char aoffset[64];
 	char *aoffsetptr = sdb_itoa (offset, aoffset, 16);
 	if (!aoffsetptr) {
-		eprintf ("Failed to convert %"PFMT64x" to a key", offset);
+		eprintf ("Failed to convert %"PFMT64x" to a key\n", offset);
 		return -1;
 	}
 	return sdb_unset (core->bin->cur->sdb_addrinfo, aoffsetptr, 0);
@@ -411,9 +411,13 @@ static int cmd_meta_lineinfo(RCore *core, const char *input) {
 		if (use_json) {
 			pj = r_core_pj_new (core);
 			pj_a (pj);
-			sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo_json, pj);
+			if (core->bin->cur && core->bin->cur->sdb_addrinfo) {
+				sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo_json, pj);
+			}
 		} else {
-			sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, NULL);
+			if (core->bin->cur && core->bin->cur->sdb_addrinfo) {
+				sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, NULL);
+			}
 		}
 		if (filter_count == 0) {
 			print_meta_offset (core, offset, pj);
@@ -446,7 +450,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		} else if (input[2] == ' ') {
 			const char *fn = input + 2;
 			const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, addr);
-			while (*fn== ' ')fn++;
+			fn = r_str_trim_head_ro (fn);
 			if (comment && *comment) {
 				// append filename in current comment
 				char *nc = r_str_newf ("%s ,(%s)", comment, fn);
@@ -557,6 +561,14 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			}
 		} else {
 			r_meta_set_string (core->anal, R_META_TYPE_COMMENT, addr, nc);
+			if (r_config_get_b (core->config, "cmd.undo")) {
+				char *a = r_str_newf ("CC-0x%08"PFMT64x, addr);
+				char *b = r_str_newf ("CC %s@0x%08"PFMT64x, nc, addr);
+				RCoreUndo *uc = r_core_undo_new (core->offset, b, a);
+				r_core_undo_push (core, uc);
+				free (a);
+				free (b);
+			}
 		}
 		free (nc);
 		}
