@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #include "r_types.h"
 #include "r_config.h"
@@ -11,79 +11,87 @@ static void __core_cmd_search_backward_prelude(RCore *core, bool doseek, bool fo
 
 static const char *help_msg_s[] = {
 	"Usage: s", "", " # Help for the seek commands. See ?$? to see all variables",
-	"s", "", "Print current address",
-	"s.", "[?]hexoff", "Seek honoring a base from core->offset",
-	"s:", "pad", "Print current address with N padded zeros (defaults to 8)",
-	"s", " addr", "Seek to address",
-	"s-", "", "Undo seek",
-	"s-*", "", "Reset undo seek history",
-	"s-", " n", "Seek n bytes backward",
-	"s--", "[n]", "Seek blocksize bytes backward (/=n)",
-	"s+", "", "Redo seek",
-	"s+", " n", "Seek n bytes forward",
-	"s++", "[n]", "Seek blocksize bytes forward (/=n)",
-	"s[j*=!]", "", "List undo seek history (JSON, =list, *r2, !=names, s==)",
-	"s/", " DATA", "Search for next occurrence of 'DATA'",
-	"s/x", " 9091", "Search for next occurrence of \\x90\\x91",
-	"sa", " [[+-]a] [asz]", "Seek asz (or bsize) aligned to addr",
-	"sb", "", "Seek aligned to bb start",
-	"sC", "[?] string", "Seek to comment matching given string",
-	"sf", "", "Seek to next function (f->addr+f->size)",
-	"sf", " function", "Seek to address of specified function",
-	"sf.", "", "Seek to the beginning of current function",
-	"sg/sG", "", "Seek begin (sg) or end (sG) of section or file",
-	"sl", "[?] [+-]line", "Seek to line",
-	"sn/sp", " ([nkey])", "Seek to next/prev location, as specified by scr.nkey",
-	"snp", "", "Seek to next function prelude",
-	"spp", "", "Seek to prev function prelude",
-	"so", " [N]", "Seek to N next opcode(s)",
-	"sr", " pc", "Seek to register",
-	"ss", "[?]", "Seek silently (without adding an entry to the seek history)",
+	"s", "", "print current address",
+	"s.", "[?]hexoff", "seek honoring a base from core->offset",
+	"s:", "pad", "print current address with N padded zeros (defaults to 8)",
+	"s", " addr", "seek to address",
+	"s-", "", "undo seek",
+	"s-*", "", "reset undo seek history",
+	"s-", " n", "seek n bytes backward",
+	"s--", "[n]", "seek blocksize bytes backward (/=n)",
+	"s+", "", "redo seek",
+	"s+", " n", "seek n bytes forward",
+	"s++", "[n]", "seek blocksize bytes forward (/=n)",
+	"s[j*=!]", "", "list undo seek history (JSON, =list, *r2, !=names, s==)",
+	"s/", " DATA", "search for next occurrence of 'DATA'",
+	"s/x", " 9091", "search for next occurrence of \\x90\\x91",
+	"sa", " [[+-]a] [asz]", "seek asz (or bsize) aligned to addr",
+	"sb", "", "seek aligned to bb start",
+	"sC", "[?] string", "seek to comment matching given string",
+	"sf", "", "seek to next function (f->addr+f->size)",
+	"sf", " function", "seek to address of specified function",
+	"sf.", "", "seek to the beginning of current function",
+	"sg/sG", "", "seek begin (sg) or end (sG) of section or file",
+	"sh", "", "open a basic shell (aims to support basic posix syntax)",
+	"sl", "[?] [+-]line", "seek to line",
+	"sn/sp", " ([nkey])", "seek to next/prev location, as specified by scr.nkey",
+	"snp", "", "seek to next function prelude",
+	"spp", "", "seek to prev function prelude",
+	"so", " [N]", "seek to N next opcode(s)",
+	"sr", " pc", "seek to register",
+	"ss", "[?]", "seek silently (without adding an entry to the seek history)",
 	// "sp [page]  seek page N (page = block)",
 	NULL
 };
 
 static const char *help_msg_sdot[] = {
-	"Usage:", "s.", "seek here or there (near seeks)",
-	"s.", "", "Seek here, same as 's $$'",
-	"s..", "32a8", "Seek to the same address but replacing the lower nibbles",
+	"Usage:", "s.", "Seek here or there (near seeks)",
+	"s.", "", "seek here, same as 's $$'",
+	"s..", "32a8", "seek to the same address but replacing the lower nibbles",
+	NULL
+};
+
+static const char *help_msg_sh[] = {
+	"Usage:", "sh", "r2's posix shell compatible subset",
+	"sh", "", "enters a posix shell subset repl (requires scr.interactive)",
+	"sh", " [cmd]", "run the given line and update $?",
 	NULL
 };
 
 static const char *help_msg_sC[] = {
 	"Usage:", "sC", "Comment grep",
-	"sC", "*", "List all comments",
-	"sC", " str", "Seek to the first comment matching 'str'",
+	"sC", "*", "list all comments",
+	"sC", " str", "seek to the first comment matching 'str'",
 	NULL
 };
 
 static const char *help_msg_sn[] = {
 	"Usage:", "sn[p]", "",
-	"sn", " [line]", "Seek to next address",
-	"snp", "", "Seek to next prelude",
+	"sn", " [line]", "seek to next address",
+	"snp", "", "seek to next prelude",
 	NULL
 };
 
 static const char *help_msg_sp[] = {
 	"Usage:", "sp[p]", "",
-	"sp", " [line]", "Seek to previous address",
-	"spp", "", "Seek to previous prelude",
+	"sp", " [line]", "seek to previous address",
+	"spp", "", "seek to previous prelude",
 	NULL
 };
 
 static const char *help_msg_sl[] = {
 	"Usage:", "sl+ or sl- or slc", "",
-	"sl", " [line]", "Seek to absolute line",
-	"sl", "[+-][line]", "Seek to relative line",
-	"slc", "", "Clear line cache",
-	"sll", "", "Show total number of lines",
-	"sleep", " [seconds]", "Sleep for an specific amount of time (support decimal values)",
+	"sl", " [line]", "seek to absolute line",
+	"sl", "[+-][line]", "seek to relative line",
+	"slc", "", "clear line cache",
+	"sll", "", "show total number of lines",
+	"sleep", " [seconds]", "sleep for an specific amount of time (support decimal values)",
 	NULL
 };
 
 static const char *help_msg_ss[] = {
 	"Usage: ss", "", " # Seek silently (not recorded in the seek history)",
-	"s?", "", "Works with all s subcommands",
+	"s?", "", "works with all s subcommands",
 	NULL
 };
 
@@ -296,6 +304,7 @@ static int cmd_seek_opcode_backward(RCore *core, int numinstr) {
 			}
 			val += op.size;
 			addr = prev_addr;
+			r_asm_op_fini (&op);
 		}
 	}
 	r_core_seek (core, addr, true);
@@ -335,7 +344,7 @@ static void cmd_seek_opcode(RCore *core, const char *input) {
 	int val = (n < 0)
 		? cmd_seek_opcode_backward (core, -n)
 		: cmd_seek_opcode_forward (core, n);
-	core->num->value = val;
+	r_core_return_value (core, val);
 }
 
 static int cmd_seek(void *data, const char *input) {
@@ -419,10 +428,20 @@ static int cmd_seek(void *data, const char *input) {
 			r_core_cmd_help (core, help_msg_sC);
 		}
 		break;
+	case '0': // "s0"
+	case '1': // "s1"
+	case '2': // "s2"
+	case '3': // "s3"
+	case '4': // "s4"
+	case '5': // "s5"
+	case '6': // "s6"
+	case '7': // "s7"
+	case '8': // "s8"
+	case '9': // "s9"
 	case ' ': // "s "
 	{
-		ut64 addr = r_num_math (core->num, input + 1);
-		if (core->num->nc.errors) {
+		ut64 addr = r_num_math (core->num, r_str_trim_head_ro (input));
+		if (core->num->nc.errors) { // TODO expose an api for this char *r_num_failed();
 			if (r_cons_singleton ()->context->is_interactive) {
 				eprintf ("Cannot seek to unknown address '%s'\n", core->num->nc.calc_buf);
 			}
@@ -486,8 +505,8 @@ static int cmd_seek(void *data, const char *input) {
 			for (input++; *input == '.'; input++) {
 				;
 			}
-			r_core_seek_base (core, input);
 			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+			r_core_seek_base (core, input);
 		} else {
 			// just re-read the current block
 			r_core_seek (core, core->offset, true);
@@ -781,6 +800,33 @@ static int cmd_seek(void *data, const char *input) {
 		}
 	}
 	break;
+	case 'h': // "sh"
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_sh);
+		} else {
+			char *arg = r_str_trim_dup (input + 1);
+			if (R_STR_ISNOTEMPTY (arg)) {
+				int rc = r_sys_tem (arg);
+				r_core_return_value (core, (rc > 0)? rc: 1);
+			} else {
+				if (r_config_get_b (core->config, "scr.interactive")) {
+					// open shell
+					r_line_set_prompt ("sh> ");
+					for (;;) {
+						const char *line = r_line_readline ();
+						if (!line || !strcmp (line, "exit")) {
+							break;
+						}
+						int rc = r_sys_tem (line);
+						r_core_return_value (core, (rc > 0)? rc: 1);
+					}
+				} else {
+					R_LOG_WARN ("enable scr.interactive to use this new shell prompt");
+				}
+			}
+			free (arg);
+		}
+		break;
 	case 'l': // "sl"
 	{
 		int sl_arg = r_num_math (core->num, input + 1);
