@@ -1,52 +1,61 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
+/* radare - LGPL - Copyright 2009-2022 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
 
-#include <string.h>
-#include "r_anal.h"
-#include "r_cons.h"
-#include "r_core.h"
-#include <sdb.h>
+#include <r_core.h>
 
 static const char *help_msg_t[] = {
 	"Usage: t", "", "# cparse types commands",
-	"t", "", "List all loaded types",
+	"t", "", "list all loaded types",
 	"tj", "", "List all loaded types as json",
-	"t", " <type>", "Show type in 'pf' syntax",
-	"t*", "", "List types info in r2 commands",
-	"t-", " <name>", "Delete types by its name",
-	"t-*", "", "Remove all types",
-	"tail", " [filename]", "Output the last part of files",
-	"tc", " [type.name]", "List all/given types in C output format",
-	"te", "[?]", "List all loaded enums",
-	"td", "[?] <string>", "Load types from string",
-	"tf", "", "List all loaded functions signatures",
-	"tk", " <sdb-query>", "Perform sdb query",
-	"tl", "[?]", "Show/Link type to an address",
+	"t", " <type>", "show type in 'pf' syntax",
+	"t*", "", "list types info in r2 commands",
+	"t-", " <name>", "delete types by its name",
+	"t-*", "", "remove all types",
+	"tail", " [filename]", "output the last part of files",
+	"tac", " [filename]", "the infamous reverse cat command",
+	"tc", " [type.name]", "list all/given types in C output format",
+	"te", "[?]", "list all loaded enums",
+	"td", "[?] <string>", "load types from string",
+	"tf", "", "list all loaded functions signatures",
+	"tk", " <sdb-query>", "perform sdb query",
+	"tl", "[?]", "show/Link type to an address",
 	"tn", "[?] [-][addr]", "manage noreturn function attributes and marks",
-	"to", " -", "Open cfg.editor to load types",
-	"to", " <path>", "Load types from C header file",
-	"toe", " [type.name]", "Open cfg.editor to edit types",
-	"tos", " <path>", "Load types from parsed Sdb database",
-	"touch", " <file>", "Create or update timestamp in file",
+	"to", " -", "open cfg.editor to load types",
+	"to", " <path>", "load types from C header file",
+	"toe", " [type.name]", "open cfg.editor to edit types",
+	"tos", " <path>", "load types from parsed Sdb database",
+	"touch", " <file>", "create or update timestamp in file",
 	"tp", "  <type> [addr|varname]", "cast data at <address> to <type> and print it (XXX: type can contain spaces)",
-	"tpv", " <type> @ [value]", "Show offset formatted for given type",
-	"tpx", " <type> <hexpairs>", "Show value for type with specified byte sequence (XXX: type can contain spaces)",
-	"ts", "[?]", "Print loaded struct types",
-	"tu", "[?]", "Print loaded union types",
-	"tx", "[f?]", "Type xrefs",
-	"tt", "[?]", "List all loaded typedefs",
+	"tpv", " <type> @ [value]", "show offset formatted for given type",
+	"tpx", " <type> <hexpairs>", "show value for type with specified byte sequence (XXX: type can contain spaces)",
+	"ts", "[?]", "print loaded struct types",
+	"tu", "[?]", "print loaded union types",
+	"tx", "[f?]", "type xrefs",
+	"tt", "[?]", "list all loaded typedefs",
+	NULL
+};
+
+static const char *help_msg_tx[] = {
+	"Usage: tx", "[flg] [...]", "",
+	"tx.", "", "same as txf",
+	"txf", " ([addr])", "list all types used in the current or given function (same as tx.)",
+	"txl","","list all types used by any function",
+	"txg", "", "render the type xrefs graph (usage .txg;aggv)",
+	"tx", " int32_t", "list functions names using this type",
+	"txt", " int32_t", "same as 'tx type'",
+	"tx", "", "list functions and the types they use",
 	NULL
 };
 
 static const char *help_msg_tcc[] = {
 	"Usage: tcc", "[-name]", "# type function calling conventions (see also afc? and arcc)",
-	"tcc", "", "List all calling convcentions",
-	"tcc", " r0 pascal(r0,r1,r2)", "Define signature for pascall cc (see also arcc)",
-	"tcc", "-pascal", "Remove the pascal cc",
-	"tcc-*", "", "Unregister all the calling conventions",
-	"tcck", "", "List calling conventions in k=v",
-	"tccl", "", "List cc signatures (return ccname (arg0, arg1, ..) err;)",
-	"tccj", "", "List them in JSON",
-	"tcc*", "", "List them as r2 commands",
+	"tcc", "", "list all calling convcentions",
+	"tcc", " r0 pascal(r0,r1,r2)", "define signature for pascall cc (see also arcc)",
+	"tcc", "-pascal", "remove the pascal cc",
+	"tcc-*", "", "unregister all the calling conventions",
+	"tcck", "", "list calling conventions in k=v",
+	"tccl", "", "list cc signatures (return ccname (arg0, arg1, ..) err;)",
+	"tccj", "", "list them in JSON",
+	"tcc*", "", "list them as r2 commands",
 	NULL
 };
 
@@ -57,67 +66,67 @@ static const char *help_msg_t_minus[] = {
 
 static const char *help_msg_tf[] = {
 	"Usage: tf[...]", "", "",
-	"tf", "", "List all function definitions loaded",
-	"tf", " <name>", "Show function signature",
-	"tfc", " <name>", "Show function signature in C syntax",
-	"tfcj", " <name>", "Same as above but in JSON",
-	"tfj", "", "List all function definitions in JSON",
-	"tfj", " <name>", "Show function signature in JSON",
+	"tf", "", "list all function definitions loaded",
+	"tf", " <name>", "show function signature",
+	"tfc", " <name>", "show function signature in C syntax",
+	"tfcj", " <name>", "same as above but in JSON",
+	"tfj", "", "list all function definitions in JSON",
+	"tfj", " <name>", "show function signature in JSON",
 	NULL
 };
 
 static const char *help_msg_to[] = {
 	"Usage: to[...]", "", "",
-	"to", " -", "Open cfg.editor to load types",
-	"to", " <path>", "Load types from C header file",
-	"tos", " <path>", "Load types from parsed Sdb database",
-	"touch", " <file>", "Create or update timestamp in file",
+	"to", " -", "open cfg.editor to load types",
+	"to", " <path>", "load types from C header file",
+	"tos", " <path>", "load types from parsed Sdb database",
+	"touch", " <file>", "create or update timestamp in file",
 	NULL
 };
 
 static const char *help_msg_tp[] = {
 	"Usage: tp[...]", "", "",
 	"tp", "  <type> [addr|varname]", "cast data at <address> to <type> and print it (XXX: type can contain spaces)",
-	"tpv", " <type> @ [value]", "Show offset formatted for given type",
-	"tpx", " <type> <hexpairs>", "Show value for type with specified byte sequence (XXX: type can contain spaces)",
+	"tpv", " <type> @ [value]", "show offset formatted for given type",
+	"tpx", " <type> <hexpairs>", "show value for type with specified byte sequence (XXX: type can contain spaces)",
 	NULL
 };
 
 static const char *help_msg_tc[] = {
 	"Usage: tc[...]", " [cctype]", "",
-	"tc", " [type.name]", "List all/given loaded types in C output format with newlines",
-	"tcd", "", "List all loaded types in C output format without newlines",
-	"tcc", "?", "Manage calling conventions types",
+	"tc", " [type.name]", "list all/given loaded types in C output format with newlines",
+	"tcd", "", "list all loaded types in C output format without newlines",
+	"tcc", "?", "manage calling conventions types",
 	"tc?", "", "show this help",
 	NULL
 };
 
 static const char *help_msg_td[] = {
 	"Usage:", "\"td [...]\"", "",
-	"td", "[string]", "Load types from string",
+	"td", "[string]", "load types from string",
 	NULL
 };
 
 static const char *help_msg_te[] = {
 	"Usage: te[...]", "", "",
-	"te", "", "List all loaded enums",
-	"te", " <enum>", "Print all values of enum for given name",
-	"tej", "", "List all loaded enums in json",
-	"tej", " <enum>", "Show enum in json",
-	"te", " <enum> <value>", "Show name for given enum number",
-	"teb", " <enum> <name>", "Show matching enum bitfield for given name",
-	"tec", "<name>", "List all/given loaded enums in C output format with newlines",
-	"ted", "", "List all loaded enums in C output format without newlines",
+	"te", "", "list all loaded enums",
+	"te", " <enum>", "print all values of enum for given name",
+	"tej", "", "list all loaded enums in json",
+	"tej", " <enum>", "show enum in json",
+	"te", " <enum> <value>", "show name for given enum number",
+	"teb", " <enum> <name>", "show matching enum bitfield for given name",
+	"tec", "<name>", "list all/given loaded enums in C output format with newlines",
+	"ted", "", "list all loaded enums in C output format without newlines",
 	"te?", "", "show this help",
 	NULL
 };
 
 static const char *help_msg_tt[] = {
 	"Usage: tt[...]", "", "",
-	"tt", "", "List all loaded typedefs",
-	"tt", " <typename>", "Show name for given type alias",
-	"ttj", "", "Show typename and type alias in json",
-	"ttc", "<name>", "Show typename and type alias in C output format",
+	"tt", "", "list all loaded typedefs",
+	"tt", " <typename>", "show name for given type alias",
+	"ttj", "", "show typename and type alias in json",
+	"ttc", "<name>", "show typename and type alias in C output format",
 	"tt?", "", "show this help",
 	NULL
 };
@@ -149,29 +158,29 @@ static const char *help_msg_tn[] = {
 
 static const char *help_msg_ts[] = {
 	"Usage: ts[...]", " [type]", "",
-	"ts", "", "List all loaded structs",
-	"ts", " [type]", "Show pf format string for given struct",
-	"tsj", "", "List all loaded structs in json",
-	"tsj", " [type]", "Show pf format string for given struct in json",
-	"ts*", "", "Show pf.<name> format string for all loaded structs",
-	"ts*", " [type]", "Show pf.<name> format string for given struct",
-	"tsc", "<name>", "List all/given loaded structs in C output format with newlines",
-	"tsd", "", "List all loaded structs in C output format without newlines",
-	"tss", " [type]", "Display size of struct",
+	"ts", "", "list all loaded structs",
+	"ts", " [type]", "show pf format string for given struct",
+	"tsj", "", "list all loaded structs in json",
+	"tsj", " [type]", "show pf format string for given struct in json",
+	"ts*", "", "show pf.<name> format string for all loaded structs",
+	"ts*", " [type]", "show pf.<name> format string for given struct",
+	"tsc", "<name>", "list all/given loaded structs in C output format with newlines",
+	"tsd", "", "list all loaded structs in C output format without newlines",
+	"tss", " [type]", "display size of struct",
 	"ts?", "", "show this help",
 	NULL
 };
 
 static const char *help_msg_tu[] = {
 	"Usage: tu[...]", "", "",
-	"tu", "", "List all loaded unions",
-	"tu", " [type]", "Show pf format string for given union",
-	"tuj", "", "List all loaded unions in json",
-	"tuj", " [type]", "Show pf format string for given union in json",
-	"tu*", "", "Show pf.<name> format string for all loaded unions",
-	"tu*", " [type]", "Show pf.<name> format string for given union",
-	"tuc", "<name>", "List all/given loaded unions in C output format with newlines",
-	"tud", "", "List all loaded unions in C output format without newlines",
+	"tu", "", "list all loaded unions",
+	"tu", " [type]", "show pf format string for given union",
+	"tuj", "", "list all loaded unions in json",
+	"tuj", " [type]", "show pf format string for given union in json",
+	"tu*", "", "show pf.<name> format string for all loaded unions",
+	"tu*", " [type]", "show pf.<name> format string for given union",
+	"tuc", "<name>", "list all/given loaded unions in C output format with newlines",
+	"tud", "", "list all loaded unions in C output format without newlines",
 	"tu?", "", "show this help",
 	NULL
 };
@@ -269,7 +278,7 @@ static void cmd_tcc(RCore *core, const char *input) {
 	case ' ':
 		if (strchr (input, '(')) {
 			if (!r_anal_cc_set (core->anal, input + 1)) {
-				eprintf ("Invalid syntax in cc signature.");
+				R_LOG_ERROR ("Invalid syntax in cc signature");
 			}
 		} else {
 			const char *ccname = r_str_trim_head_ro (input + 1);
@@ -311,11 +320,41 @@ static void showFormat(RCore *core, const char *name, int mode) {
 			}
 			free (fmt);
 		} else {
-			eprintf ("Cannot find '%s' type\n", name);
+			R_LOG_ERROR ("Cannot find '%s' type", name);
 		}
 	}
 }
 
+static int cmd_tac(void *data, const char *_input) { // "tac"
+	char *input = strdup (_input);
+	char *arg = strchr (input, ' ');
+	if (arg) {
+		arg = (char *)r_str_trim_head_ro (arg + 1);
+	}
+	switch (*input) {
+	case '?': // "tac?"
+		eprintf ("Usage: tac [file]\n");
+		break;
+	default: // "tac"
+		if (R_STR_ISNOTEMPTY (arg)) {
+			char *data = r_file_slurp (arg, NULL);
+			RList *lines = r_str_split_list (data, "\n", 0);
+			RListIter *iter;
+			char *line;
+			r_list_foreach_prev (lines, iter, line) {
+				r_cons_printf ("%s\n", line);
+			}
+			r_list_free (lines);
+			free (data);
+		} else {
+			eprintf ("Usage: tac [file]\n");
+		}
+		break;
+	}
+	free (input);
+	return 0;
+}
+
 static int cmd_tail(void *data, const char *_input) { // "tail"
 	char *input = strdup (_input);
 	RCore *core = (RCore *)data;
@@ -695,11 +734,11 @@ static bool print_link_readable_cb(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	char *fmt = r_type_format (core->anal->sdb_types, v);
 	if (!fmt) {
-		eprintf ("Can't fint type %s", v);
+		eprintf ("Can't find type %s\n", v);
 		return 1;
 	}
 	r_cons_printf ("(%s)\n", v);
-	r_core_cmdf (core, "pf %s @ 0x%s\n", fmt, k + strlen ("link."));
+	r_core_cmdf (core, "pf %s @ 0x%s", fmt, k + strlen ("link."));
 	return true;
 }
 
@@ -708,11 +747,11 @@ static bool print_link_readable_json_cb(void *p, const char *k, const char *v) {
 	RCore *core = (RCore *)p;
 	char *fmt = r_type_format (core->anal->sdb_types, v);
 	if (!fmt) {
-		eprintf ("Can't fint type %s", v);
+		R_LOG_ERROR ("Can't find type %s", v);
 		return true;
 	}
 	r_cons_printf ("{\"%s\":", v);
-	r_core_cmdf (core, "pfj %s @ 0x%s\n", fmt, k + strlen ("link."));
+	r_core_cmdf (core, "pfj %s @ 0x%s", fmt, k + strlen ("link."));
 	r_cons_printf ("}");
 	return true;
 }
@@ -1118,10 +1157,10 @@ static int cmd_type(void *data, const char *input) {
 		break;
 	case 's': { // "ts"
 		switch (input[1]) {
-		case '?':
+		case '?': // "ts?"
 			r_core_cmd_help (core, help_msg_ts);
 			break;
-		case '*':
+		case '*': // "ts*"
 			if (input[2] == ' ') {
 				showFormat (core, r_str_trim_head_ro (input + 2), 1);
 			} else {
@@ -1137,7 +1176,7 @@ static int cmd_type(void *data, const char *input) {
 		case ' ':
 			showFormat (core, r_str_trim_head_ro (input + 1), 0);
 			break;
-		case 's':
+		case 's': // "tss"
 			if (input[2] == ' ') {
 				r_cons_printf ("%" PFMT64u "\n", (r_type_get_bitsize (TDB, input + 3) / 8));
 			} else {
@@ -1175,7 +1214,7 @@ static int cmd_type(void *data, const char *input) {
 			*member_name++ = 0;
 		}
 		if (name && (r_type_kind (TDB, name) != R_TYPE_ENUM)) {
-			eprintf ("%s is not an enum\n", name);
+			R_LOG_ERROR ("%s is not an enum", name);
 			free (name);
 			break;
 		}
@@ -1293,8 +1332,20 @@ static int cmd_type(void *data, const char *input) {
 		}
 		break;
 	}
-	case ' ':
-		showFormat (core, input + 1, 0);
+	case ' ': // "t "
+		  {
+			  const char *token = r_str_trim_head_ro (input + 1);
+			  const char *typdef = sdb_const_get (core->anal->sdb_types, token, 0);
+			  // Tresolve typedef if any
+			  if (typdef && !strcmp (typdef, "typedef")) {
+				  r_strf_var (a, 128, "typedef.%s", token);
+				  const char *tokendef = sdb_const_get (core->anal->sdb_types, a, 0);
+				  if (tokendef) {
+					  token = tokendef;
+				  }
+			  }
+			  showFormat (core, token, 0);
+		  }
 		break;
 	// t* - list all types in 'pf' syntax
 	case 'j': // "tj"
@@ -1352,7 +1403,7 @@ static int cmd_type(void *data, const char *input) {
 				if (arg) {
 					r_file_touch (arg + 1);
 				} else {
-					eprintf ("Usage: touch [filename]");
+					eprintf ("Usage: touch [filename]\n");
 				}
 			} else if (input[1] == 's') {
 				const char *dbpath = input + 3;
@@ -1392,9 +1443,7 @@ static int cmd_type(void *data, const char *input) {
 			// TODO #7967 help refactor: move to detail
 			r_core_cmd_help (core, help_msg_td);
 			r_cons_printf ("Note: The td command should be put between double quotes\n"
-				"Example: \"td struct foo {int bar;int cow;};\""
-				"\nt");
-
+				"Example: \"td struct foo {int bar;int cow;};\"\n");
 		} else if (input[1] == ' ') {
 			char *tmp = r_str_newf ("%s;", input + 2);
 			if (!tmp) {
@@ -1408,11 +1457,11 @@ static int cmd_type(void *data, const char *input) {
 				free (out);
 			}
 			if (error_msg) {
-				eprintf ("%s", error_msg);
+				R_LOG_ERROR ("%s", error_msg);
 				free (error_msg);
 			}
 		} else {
-			eprintf ("Invalid use of td. See td? for help\n");
+			R_LOG_ERROR ("Invalid use of td. See td? for help");
 		}
 		break;
 	case 'x': {
@@ -1485,7 +1534,7 @@ static int cmd_type(void *data, const char *input) {
 				r_list_free (uniqList);
 			}
 			break;
-		case 't':
+		case 't': // "txt"
 		case ' ': // "tx " -- show which function use given type
 			type = (char *)r_str_trim_head_ro (input + 2);
 			r_list_foreach (core->anal->fcns, iter, fcn) {
@@ -1499,14 +1548,7 @@ static int cmd_type(void *data, const char *input) {
 			}
 			break;
 		default:
-			eprintf ("Usage: tx[flg] [...]\n");
-			eprintf (" txf | tx.      list all types used in this function\n");
-			eprintf (" txf 0xaddr     list all types used in function at 0xaddr\n");
-			eprintf (" txl            list all types used by any function\n");
-			eprintf (" txg            render the type xrefs graph (usage .txg;aggv)\n");
-			eprintf (" tx int32_t     list functions names using this type\n");
-			eprintf (" txt int32_t    same as 'tx type'\n");
-			eprintf (" tx             list functions and the types they use\n");
+			r_core_cmd_help (core, help_msg_tx);
 			break;
 		}
 		break;
@@ -1514,16 +1556,19 @@ static int cmd_type(void *data, const char *input) {
 	// ta: moved to anal hints (aht)- just for tail, at the moment
 	case 'a': // "ta"
 		switch (input[1]) {
-		case 'i': { // "tai"
+		case 'c': // "tac"
+			cmd_tac (core, input);
+			break;
+		case 'i': // "tai"
 			if (input[2] == 'l') {
 				cmd_tail (core, input);
 			} else {
 				eprintf ("Usage: tail [number] [file]\n");
 			}
 			break;
-		}
 		default:
-			eprintf ("[WARNING] \"ta\" is deprecated. Use \"aht\" instead.\n");
+			R_LOG_WARN ("`ta` command is deprecated. Use \"aht\" instead");
+			break;
 		}
 		break;
 	// tl - link a type to an address
@@ -1623,7 +1668,7 @@ static int cmd_type(void *data, const char *input) {
 			char *fmt = r_type_format (TDB, type_name);
 			if (fmt && *fmt) {
 				ut64 val = core->offset;
-				r_core_cmdf (core, "pf %s @v:0x%08" PFMT64x "\n", fmt, val);
+				r_core_cmdf (core, "pf %s @v:0x%08" PFMT64x, fmt, val);
 			} else {
 				eprintf ("Usage: tpv [type] @ [value]\n");
 			}
@@ -1652,7 +1697,6 @@ static int cmd_type(void *data, const char *input) {
 				}
 				if (input[1] == 'x' && arg) { // "tpx"
 					r_core_cmdf (core, "pf %s @x:%s", fmt, arg);
-					// eprintf ("pf %s @x:%s", fmt, arg);
 				} else {
 					ut64 addr = arg ? r_num_math (core->num, arg): core->offset;
 					ut64 original_addr = addr;
@@ -1665,20 +1709,28 @@ static int cmd_type(void *data, const char *input) {
 							}
 						}
 					}
+					int type_size = r_type_get_bitsize (core->anal->sdb_types, type) / 8;
+					int obs = core->blocksize;
+					if (type_size > obs) {
+						r_core_block_size (core, type_size);
+					}
 					if (addr != UT64_MAX) {
-						r_core_cmdf (core, "pf %s @ 0x%08" PFMT64x, fmt, addr);
+						r_core_cmdf (core, "pf %s @ 0x%08"PFMT64x, fmt, addr);
 					} else if (original_addr == 0) {
-						r_core_cmdf (core, "pf %s @ 0x%08" PFMT64x, fmt, original_addr);
+						r_core_cmdf (core, "pf %s @ 0x%08"PFMT64x, fmt, original_addr);
+					}
+					if (type_size > obs) {
+						r_core_block_size (core, obs);
 					}
 				}
 				free (fmt);
 				free (type);
 			} else {
-				eprintf ("Usage: tp?\n");
+				r_core_cmd_help (core, help_msg_tp);
 			}
 			free (tmp);
 		} else { // "tp"
-			eprintf ("Usage: tp?\n");
+			r_core_cmd_help (core, help_msg_tp);
 		}
 		break;
 	case '-': // "t-"
@@ -1729,7 +1781,7 @@ static int cmd_type(void *data, const char *input) {
 			break;
 		}
 		break;
-	case 't': {
+	case 't': { // "tt"
 		if (!input[1] || input[1] == 'j') {
 			PJ *pj = NULL;
 			if (input[1] == 'j') {
