@@ -60,7 +60,7 @@ bool file_copyrf(const char *src, const char *dst) {
 				r_sys_mkdirp (dstp);
 			} else {
 				if (!file_copyp (path, dstp)) {
-				eprintf ("Failed to copy the file: %s to %s\n",
+				R_LOG_ERROR ("Failed to copy the file: %s to %s",
 						path, dstp);
 				ret = false;
 				//continue copying files don't break
@@ -69,7 +69,7 @@ bool file_copyrf(const char *src, const char *dst) {
 			free (dstp);
 		} else {
 			ret = false;
-			eprintf ("Failed to copy the file: %s\n", path);
+			R_LOG_ERROR ("Failed to copy the file: %s", path);
 		}
 	}
 	return ret;
@@ -134,7 +134,7 @@ static bool repo_exists(const char *path) {
 			break;
 		}
 		if (!r_file_is_directory (files[i]) && !r_file_exists (files[i])) {
-			eprintf ("Error: Corrupt repo: %s doesn't exist\n",
+			R_LOG_ERROR ("Corrupt repo: %s doesn't exist",
 					files[i]);
 			r = false;
 			break;
@@ -192,13 +192,13 @@ static void free_blobs(RList *blobs) {
 	r_list_free (blobs);
 }
 
-static char *absp2rp(const char *rp, const char *absp) {
+static char *absp2rp(Rvc *rvc, const char *absp) {
 	char *p;
-	char *arp = r_file_abspath (rp);
+	char *arp = r_file_abspath (rvc->path);
 	if (!arp) {
 		return NULL;
 	}
-	if (r_str_len_utf8 (arp) < r_str_len_utf8 (rp)) {
+	if (r_str_len_utf8 (arp) < r_str_len_utf8 (rvc->path)) {
 		free (arp);
 		return NULL;
 	}
@@ -212,8 +212,8 @@ static char *absp2rp(const char *rp, const char *absp) {
 	return ret;
 }
 
-char *rp2absp(const char *rp, const char *path) {
-	char *arp = r_file_abspath (rp);
+char *rp2absp(Rvc *rvc, const char *path) {
+	char *arp = r_file_abspath (rvc->path);
 	if (!arp) {
 		return NULL;
 	}
@@ -229,31 +229,28 @@ char *rp2absp(const char *rp, const char *path) {
 
 //TODO:Make the tree related functions abit cleaner & more efficient
 
-static RList *get_commits(const char *rp, const size_t max_num) {
+static RList *get_commits(Rvc *rvc, const size_t max_num) {
 	char *i;
 	RList *ret = r_list_new ();
 	if (!ret) {
 		return NULL;
 	}
-	Sdb *db = vcdb_open(rp);
-	i = sdb_get (db, sdb_const_get (db, CURRENTB, 0), 0);
+	i = sdb_get (rvc->db, sdb_const_get (rvc->db, CURRENTB, 0), 0);
 	if (!i) {
 		r_list_free (ret);
-		sdb_unlink (db);
-		sdb_free (db);
-		return NULL;
+		ret = NULL;
+		goto ret;
 	}
 	if (!strcmp (i, NULLVAL)) {
-		sdb_unlink (db);
-		sdb_free (db);
-		return ret;
+		goto ret;
 	}
 	while (true) {
 		if (!r_list_prepend (ret, i)) {
 			r_list_free (ret);
+			ret = NULL;
 			break;
 		}
-		i = sdb_get (db, i, 0);
+		i = sdb_get (rvc->db, i, 0);
 		if (!i) {
 			r_list_free (ret);
 			ret = NULL;
@@ -263,8 +260,7 @@ static RList *get_commits(const char *rp, const size_t max_num) {
 			break;
 		}
 	}
-	sdb_unlink (db);
-	sdb_free (db);
+ret:
 	return ret;
 }
 
@@ -274,9 +270,6 @@ static bool in_rvc_ignore(const RList *ignore, const char *rpf) {
 	RListIter *iter;
 	char *p;
 	bool ret = false;
-	if (!ignore) {
-		return false;
-	}
 	r_list_foreach (ignore, iter, p) {
 		char *stripped = strip_sys_dir (p);
 		if (stripped) {
@@ -303,7 +296,7 @@ static bool update_blobs(const RList *ignore, RList *blobs, const RList *nh) {
 			continue;
 		}
 		blob->fhash = r_str_new (nh->tail->data);
-		return blob->fhash;
+		return (bool) blob->fhash;
 	}
 	blob = R_NEW (RvcBlob);
 	if (!blob) {
@@ -312,22 +305,22 @@ static bool update_blobs(const RList *ignore, RList *blobs, const RList *nh) {
 	blob->fhash = r_str_new (nh->tail->data);
 	blob->fname = r_str_new (nh->head->data);
 	if (!blob->fhash || !blob->fname) {
-		free (blob->fhash);
-		free (blob->fname);
-		free (blob);
-		return false;
+		goto fail_ret;
 	}
 	if (!r_list_append (blobs, blob)) {
-		free (blob->fhash);
-		free (blob->fname);
-		free (blob);
-		return false;
+		goto fail_ret;
 	}
 	return true;
+fail_ret:
+	free (blob->fhash);
+	free (blob->fname);
+	free (blob);
+	return false;
+
 }
 
-static int branch_exists(const char *rp, const char *bname) {
-	RList *branches = r_vc_get_branches (rp);
+static int branch_exists(Rvc *rvc, const char *bname) {
+	RList *branches = r_vc_get_branches (rvc);
 	if (!branches) {
 		return -1;
 	}
@@ -345,53 +338,42 @@ static int branch_exists(const char *rp, const char *bname) {
 	return ret;
 }
 
-static RList *get_blobs(const char *rp, RList *ignore) {
-	RList *commits = get_commits (rp, 0);
+static RList *get_blobs(Rvc *rvc, RList *ignore) {
+	RList *commits = get_commits (rvc, 0);
 	if (!commits) {
 		return NULL;
 	}
 	RList *ret = r_list_new ();
 	if (!ret) {
-		r_list_free (commits);
+		goto ret;
 	}
 	RListIter *i;
 	char *hash;
 	r_list_foreach (commits, i, hash) {
-		char *commit_path = r_file_new (rp, ".rvc", "commits",
+		char *commit_path = r_file_new (rvc->path, ".rvc", "commits",
 				hash, NULL);
 		if (!commit_path) {
-			free_blobs (ret);
-			ret = NULL;
-			break;
+			goto fail_ret;
 		}
 		char *content = r_file_slurp (commit_path, 0);
 		free (commit_path);
 		if (!content) {
-			free_blobs (ret);
-			ret = NULL;
-			break;
+			goto fail_ret;
 		}
 		RList *lines = r_str_split_duplist (content, "\n", true);
 		free (content);
 		if (!lines) {
-			free_blobs (ret);
-			ret = NULL;
-			break;
+			goto fail_ret;
 		}
 		RListIter *j;
 		char *ln;
 		bool found = false;
 		r_list_foreach (lines, j, ln) {
 			if (!found) {
-				found = !r_str_cmp (ln, COMMIT_BLOB_SEP, r_str_len_utf8 (COMMIT_BLOB_SEP));
+				found = !r_str_cmp (ln, COMMIT_BLOB_SEP,
+						r_str_len_utf8 (COMMIT_BLOB_SEP));
 				continue;
 			}
-			RvcBlob *blob = R_NEW (RvcBlob);
-			if (!blob) {
-				free_blobs (ret);
-				ret = NULL;
-				break;
-			}
 			RList *kv = r_str_split_list (ln, "=", 2);
 			if (!kv) {
 				free_blobs (ret);
@@ -407,24 +389,28 @@ static RList *get_blobs(const char *rp, RList *ignore) {
 		}
 		r_list_free (lines);
 	}
+ret:
 	r_list_free (commits);
 	return ret;
+fail_ret:
+	free_blobs(ret);
+	return NULL;
 }
 
-static bool rm_empty_dir(const char *rp) {
-	char *rvc = r_file_new (rp, ".rvc", NULL);
-	if (!rvc) {
+static bool rm_empty_dir(Rvc *rvc) {
+	char *path = r_file_new (rvc->path, ".rvc", NULL);
+	if (!path) {
 		return false;
 	}
-	RList *files = r_file_lsrf (rp);
+	RList *files = r_file_lsrf (rvc->path);
 	RListIter *iter;
 	const char *f;
 	r_list_foreach (files, iter, f) {
-		if (r_str_cmp (f, rvc, r_str_len_utf8 (rvc))) {
+		if (r_str_cmp (f, path, r_str_len_utf8 (path))) {
 			rmdir (f);
 		}
 	}
-	free (rvc);
+	free (path);
 	r_list_free (files);
 	return true;
 }
@@ -493,9 +479,9 @@ static RList *repo_files(const char *dir) {
 	return ret;
 }
 
-static RList *load_rvc_ignore(const char *rp) {
+static RList *load_rvc_ignore(Rvc *rvc) {
 	RList *ignore = NULL;
-	char *path = r_file_new (rp, IGNORE_NAME, NULL);
+	char *path = r_file_new (rvc->path, IGNORE_NAME, NULL);
 	if (!path) {
 		return false;
 	}
@@ -510,17 +496,17 @@ static RList *load_rvc_ignore(const char *rp) {
 }
 
 //shit function:
-R_API RList *r_vc_get_uncommitted(const char *rp) {
-	RList *ignore = load_rvc_ignore (rp);
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+R_API RList *r_vc_get_uncommitted(Rvc *rvc) {
+	RList *ignore = load_rvc_ignore (rvc);
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
-	RList *blobs = get_blobs(rp, ignore);
+	RList *blobs = get_blobs(rvc, ignore);
 	if (!blobs) {
 		return NULL;
 	}
-	RList *files = repo_files (rp);
+	RList *files = repo_files (rvc->path);
 	if (!files) {
 		free_blobs (blobs);
 		return NULL;
@@ -534,7 +520,7 @@ R_API RList *r_vc_get_uncommitted(const char *rp) {
 	RListIter *iter;
 	RvcBlob *blob;
 	r_list_foreach (blobs, iter, blob) {
-		char *blob_absp = rp2absp (rp, blob->fname);
+		char *blob_absp = rp2absp (rvc, blob->fname);
 		if (!blob_absp) {
 			goto fail_ret;
 		}
@@ -582,7 +568,7 @@ R_API RList *r_vc_get_uncommitted(const char *rp) {
 	free_blobs (blobs);
 	blobs = NULL;
 	r_list_foreach (files, iter, file) {
-		char *rfp = absp2rp (rp, file);
+		char *rfp = absp2rp (rvc, file);
 		if (!rfp) {
 			goto fail_ret;
 		}
@@ -609,8 +595,8 @@ fail_ret:
 	return NULL;
 }
 
-static char *find_blob_hash(const char *rp, const char *fname) {
-	RList *blobs = get_blobs (rp, load_rvc_ignore(rp));
+static char *find_blob_hash(Rvc *rvc, const char *fname) {
+	RList *blobs = get_blobs (rvc, load_rvc_ignore(rvc));
 	if (blobs) {
 		RListIter *i;
 		RvcBlob *b;
@@ -625,7 +611,7 @@ static char *find_blob_hash(const char *rp, const char *fname) {
 	return NULL;
 }
 
-static char *write_commit(const char *rp, const char *message, const char *author, RList *blobs) {
+static char *write_commit(Rvc *rvc, const char *message, const char *author, RList *blobs) {
 	RvcBlob *blob;
 	RListIter *iter;
 	char *content = r_str_newf ("message=%s\nauthor=%s\ntime=%" PFMT64x "\n"
@@ -646,7 +632,7 @@ static char *write_commit(const char *rp, const char *message, const char *autho
 		free (content);
 		return false;
 	}
-	char *commit_path = r_file_new (rp, ".rvc","commits", commit_hash, NULL);
+	char *commit_path = r_file_new (rvc->path, ".rvc","commits", commit_hash, NULL);
 	if (!commit_path || !r_file_dump (commit_path, (const ut8*)content, -1, false)) {
 		free (content);
 		free (commit_hash);
@@ -656,7 +642,7 @@ static char *write_commit(const char *rp, const char *message, const char *autho
 	return commit_hash;
 }
 
-static RvcBlob *bfadd(const char *rp, const char *fname) {
+static RvcBlob *bfadd(Rvc *rvc, const char *fname) {
 	RvcBlob *ret = R_NEW (RvcBlob);
 	if (!ret) {
 		return NULL;
@@ -666,7 +652,7 @@ static RvcBlob *bfadd(const char *rp, const char *fname) {
 		free (ret);
 		return NULL;
 	}
-	ret->fname = absp2rp (rp, absp);
+	ret->fname = absp2rp (rvc, absp);
 	if (!ret->fname) {
 		free (ret);
 		free (absp);
@@ -685,7 +671,7 @@ static RvcBlob *bfadd(const char *rp, const char *fname) {
 		free (absp);
 		goto fail_ret;
 	}
-	char *bpath = r_file_new (rp, ".rvc", "blobs", ret->fhash, NULL);
+	char *bpath = r_file_new (rvc->path, ".rvc", "blobs", ret->fhash, NULL);
 	if (!bpath) {
 		goto fail_ret;
 	}
@@ -706,12 +692,12 @@ fail_ret:
 	return NULL;
 }
 
-static RList *blobs_add(const char *rp, const RList *files) {
+static RList *blobs_add(Rvc *rvc, const RList *files) {
 	RList *ret = r_list_new ();
 	if (!ret) {
 		return NULL;
 	}
-	RList *uncommitted = r_vc_get_uncommitted (rp);
+	RList *uncommitted = r_vc_get_uncommitted (rvc);
 	if (!uncommitted) {
 		free (ret);
 		return NULL;
@@ -733,7 +719,7 @@ static RList *blobs_add(const char *rp, const RList *files) {
 				continue;
 			}
 			found = true;
-			RvcBlob *b = bfadd (rp, ucp);
+			RvcBlob *b = bfadd (rvc, ucp);
 			if (!b) {
 				free (absp);
 				goto fail_ret;
@@ -749,8 +735,8 @@ static RList *blobs_add(const char *rp, const RList *files) {
 		}
 		if (!found) {
 			eprintf ("File %s is already committed\n", path);
-			free (absp);
 		}
+		free (absp);
 	}
 	return ret;
 fail_ret:
@@ -759,10 +745,10 @@ fail_ret:
 	return NULL;
 }
 
-R_API bool r_vc_commit(const char *rp, const char *message, const char *author, const RList *files) {
+R_API bool r_vc_commit(Rvc *rvc, const char *message, const char *author, const RList *files) {
 	char *commit_hash;
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
 	if (R_STR_ISEMPTY (message)) {
@@ -785,12 +771,13 @@ R_API bool r_vc_commit(const char *rp, const char *message, const char *author,
 	}
 	const char *m;
 	for (m = message; *m; m++) {
-		if (*m < ' ') {
-			eprintf ("commit messages must not contain unprintable charecters\n");
+		if (*m < ' ' && *m != '\n') {
+			eprintf ("commit messages must not contain unprintable charecters %c\n",
+					*m);
 			return false;
 		}
 	}
-	RList *blobs = blobs_add (rp, files);
+	RList *blobs = blobs_add (rvc, files);
 	if (!blobs) {
 		return false;
 	}
@@ -799,54 +786,41 @@ R_API bool r_vc_commit(const char *rp, const char *message, const char *author,
 		eprintf ("Nothing to commit\n");
 		return false;
 	}
-	commit_hash = write_commit (rp, message, author, blobs);
+	commit_hash = write_commit (rvc, message, author, blobs);
 	if (!commit_hash) {
 		free_blobs (blobs);
 		return false;
 	}
 	{
 		const char *current_branch;
-		Sdb *db = vcdb_open (rp) ;
-		current_branch = sdb_const_get (db, CURRENTB, 0);
-		if (sdb_set (db, commit_hash, sdb_const_get (db, current_branch, 0), 0) < 0) {
-			sdb_unlink (db);
-			sdb_free (db);
+		current_branch = sdb_const_get (rvc->db, CURRENTB, 0);
+		if (sdb_set (rvc->db, commit_hash, sdb_const_get (rvc->db, current_branch, 0), 0) < 0) {
 			free_blobs (blobs);
 			free (commit_hash);
 			return false;
 		}
-		if (sdb_set (db, current_branch, commit_hash, 0) < 0) {
-			sdb_unlink (db);
-			sdb_free (db);
+		if (sdb_set (rvc->db, current_branch, commit_hash, 0) < 0) {
 			free_blobs (blobs);
 			free (commit_hash);
 			return false;
 		}
-		sdb_sync (db);
-		sdb_unlink (db);
-		sdb_free (db);
 	}
 	free (commit_hash);
 	free_blobs (blobs);
 	return true;
 }
 
-R_API RList *r_vc_get_branches(const char *rp) {
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+R_API RList *r_vc_get_branches(Rvc *rvc) {
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return NULL;
 	}
-	Sdb *db = vcdb_open (rp);
 	RList *ret = r_list_new ();
 	if (!ret) {
-		sdb_unlink (db);
-		sdb_free (db);
 		return NULL;
 	}
-	SdbList *keys = sdb_foreach_list (db, false);
+	SdbList *keys = sdb_foreach_list (rvc->db, false);
 	if (!keys) {
-		sdb_unlink (db);
-		sdb_free (db);
 		r_list_free (ret);
 		return NULL;
 	}
@@ -865,16 +839,14 @@ R_API RList *r_vc_get_branches(const char *rp) {
 		}
 	}
 	ls_free (keys);
-	sdb_unlink (db);
-	sdb_free (db);
 	return ret;
 }
 
-R_API bool r_vc_branch(const char *rp, const char *bname) {
+R_API bool r_vc_branch(Rvc *rvc, const char *bname) {
 	const char *current_branch;
 	const char *commits;
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
 	if (!is_valid_branch_name (bname)) {
@@ -882,7 +854,7 @@ R_API bool r_vc_branch(const char *rp, const char *bname) {
 		return false;
 	}
 	{
-		int ret = branch_exists (rp, bname);
+		int ret = branch_exists (rvc, bname);
 		if (ret < 0) {
 			return false;
 		} else if (ret) {
@@ -890,87 +862,84 @@ R_API bool r_vc_branch(const char *rp, const char *bname) {
 			return false;
 		}
 	}
-	Sdb *db = vcdb_open (rp);
-	current_branch = sdb_const_get (db, CURRENTB, 0);
+	current_branch = sdb_const_get (rvc->db, CURRENTB, 0);
 	if (!current_branch) {
-		sdb_unlink (db);
-		sdb_free (db);
 		return false;
 	}
-	commits = sdb_const_get (db, current_branch, 0);
+	commits = sdb_const_get (rvc->db, current_branch, 0);
 	char *nbn = r_str_newf (BPREFIX "%s", bname);
 	if (!nbn) {
-		sdb_unlink (db);
-		sdb_free (db);
 		return false;
 	}
-	sdb_set (db, nbn, commits, 0);
+	sdb_set (rvc->db, nbn, commits, 0);
 	free (nbn);
-	sdb_sync (db);
-	sdb_unlink (db);
-	sdb_free (db);
 	return true;
 }
 
-R_API bool r_vc_new(const char *path) {
-	Sdb *db;
+R_API Rvc *r_vc_new(const char *path) {
 	char *commitp, *blobsp;
-	char *vcp = r_file_new (path, ".rvc", NULL);
-	if (r_file_is_directory (vcp)) {
-		eprintf ("A repository already exists in %s\n", path);
-		free (vcp);
-		return false;
-
+	if (repo_exists (path)) {
+		eprintf("A repo already exists in %s", path);
+		return NULL;
 	}
-	if (!vcp) {
-		return false;
+	Rvc *rvc = R_NEW(Rvc);
+	if (!rvc) {
+		eprintf("Failed to create repo\n");
+		return NULL;
 	}
-	commitp = r_file_new (vcp, "commits", NULL);
-	blobsp = r_file_new (vcp, "blobs", NULL);
+	rvc->path = r_str_new (path);
+	if (!rvc->path) {
+		free (rvc);
+		return NULL;
+	}
+	commitp = r_file_new (rvc->path, ".rvc", "commits", NULL);
+	blobsp = r_file_new (rvc->path, ".rvc","blobs", NULL);
 	if (!commitp || !blobsp) {
 		free (commitp);
 		free (blobsp);
-		free (vcp);
+		free (rvc->path);
+		free (rvc);
 		return false;
 	}
 	if (!r_sys_mkdirp (commitp) || !r_sys_mkdir (blobsp)) {
-		eprintf ("Can't create The RVC repo directory");
+		eprintf ("Can't create The RVC repo directory\n");
 		free (commitp);
-		free (vcp);
+		free (rvc->path);
+		free (rvc);
 		free (blobsp);
 		return false;
 	}
 	free (commitp);
 	free (blobsp);
-	db = sdb_new (vcp, DBNAME, 0);
-	free (vcp);
-	if (!db) {
+	rvc->db = sdb_new (rvc->path, "/.rvc/" DBNAME, 0);
+	if (!rvc->db) {
 		eprintf ("Can't create The RVC branches database");
+		free (rvc->path);
+		free (rvc);
 		return false;
 	}
-	if (!sdb_set (db, FIRST_BRANCH, NULLVAL, 0)) {
-		sdb_unlink (db);
-		sdb_free (db);
+	if (!sdb_set (rvc->db, FIRST_BRANCH, NULLVAL, 0)) {
+		sdb_unlink (rvc->db);
+		sdb_free (rvc->db);
+		free (rvc->path);
+		free (rvc);
 		return false;
 	}
-	if (!sdb_set (db, CURRENTB, FIRST_BRANCH, 0)) {
-		sdb_unlink (db);
-		sdb_free (db);
+	if (!sdb_set (rvc->db, CURRENTB, FIRST_BRANCH, 0)) {
+		sdb_unlink (rvc->db);
+		sdb_free (rvc->db);
 		return false;
 	}
-	sdb_sync (db);
-	sdb_unlink (db);
-	sdb_free (db);
-	return true;
+	return rvc;
 }
 
-R_API bool r_vc_checkout(const char *rp, const char *bname) {
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+R_API bool r_vc_checkout(Rvc *rvc, const char *bname) {
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
 	{
-		int ret = branch_exists (rp, bname);
+		int ret = branch_exists (rvc, bname);
 		if (ret < 0) {
 			return false;
 		}
@@ -979,7 +948,7 @@ R_API bool r_vc_checkout(const char *rp, const char *bname) {
 			return false;
 		}
 	}
-	RList *uncommitted = r_vc_get_uncommitted (rp);
+	RList *uncommitted = r_vc_get_uncommitted (rvc);
 	RListIter *i;
 	char *file;
 	if (!uncommitted) {
@@ -997,59 +966,47 @@ R_API bool r_vc_checkout(const char *rp, const char *bname) {
 	r_list_free (uncommitted);
 	//Must set to NULL to avoid double r_list_free on fail_ret
 	uncommitted = NULL;
-	Sdb *db = vcdb_open (rp) ;
-	if (!db) {
-		return false;
-	}
 	const char *oldb;
 	{
 		char *fbname = r_str_newf (BPREFIX "%s", bname);
 		if (!fbname) {
-			sdb_unlink (db);
-			sdb_free (db);
 			return false;
 		}
-		oldb = sdb_const_get (db, CURRENTB, 0);
-		sdb_set (db, CURRENTB, fbname, 0);
+		oldb = sdb_const_get (rvc->db, CURRENTB, 0);
+		sdb_set (rvc->db, CURRENTB, fbname, 0);
 		free (fbname);
-		if (!sdb_sync (db)) {
-			sdb_unlink (db);
-			sdb_free (db);
+		if (!sdb_sync (rvc->db)) {
 			return false;
 		}
 	}
-	if (!r_vc_reset (rp)) {
+	if (!r_vc_reset (rvc)) {
 		goto fail_ret;
 	}
-	if (!rm_empty_dir (rp)) {
+	if (!rm_empty_dir (rvc)) {
 		goto fail_ret;
 	}
-	sdb_sync (db);
-	sdb_unlink (db);
-	sdb_free (db);
+	sdb_sync (rvc->db);
 	return true;
 fail_ret:
 	r_list_free (uncommitted);
-	sdb_set (db, CURRENTB, oldb, 0);
-	sdb_sync (db);
-	sdb_unlink (db);
-	sdb_free (db);
+	sdb_set (rvc->db, CURRENTB, oldb, 0);
+	sdb_sync (rvc->db);
 	return false;
 }
 
-R_API RList *r_vc_log(const char *rp) {
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+R_API RList *r_vc_log(Rvc *rvc) {
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
-	RList *commits = get_commits (rp, 0);
+	RList *commits = get_commits (rvc, 0);
 	if (!commits) {
 		return NULL;
 	}
 	RListIter *iter;
 	char *ch;
 	r_list_foreach_prev (commits, iter, ch) {
-		char *cp = r_file_new (rp, ".rvc", "commits", ch, NULL);
+		char *cp = r_file_new (rvc->path, ".rvc", "commits", ch, NULL);
 		if (!cp) {
 			goto fail_ret;
 		}
@@ -1076,38 +1033,36 @@ fail_ret:
 	return NULL;
 }
 
-R_API char *r_vc_current_branch(const char *rp) {
-	if (!repo_exists (rp)) {
-		eprintf ("No valid repo in %s\n", rp);
+R_API char *r_vc_current_branch(Rvc *rvc) {
+	if (!repo_exists (rvc->path)) {
+		eprintf ("No valid repo in %s\n", rvc->path);
 		return false;
 	}
-	Sdb *db = vcdb_open (rp);
-	if (!db) {
+	if (!rvc->db) {
 		return NULL;
 	}
 	//TODO: return consistently either BPREFIX.bname or bname
-	char *ret = r_str_new (sdb_const_get (db, CURRENTB, 0) + r_str_len_utf8 (BPREFIX));
-	sdb_unlink (db);
-	sdb_close (db);
-	sdb_free (db);
+	char *ret = r_str_new (sdb_const_get (rvc->db, CURRENTB, 0) + r_str_len_utf8 (BPREFIX));
 	return ret;
 }
 
 R_API bool r_vc_clone(const char *src, const char *dst) {
-	printf ("src, dst: %s, %s\n", src, dst);
 	char *drp = r_file_new (dst, ".rvc", NULL);
 	bool ret = false;
 	if (drp) {
 		char *srp = r_file_new (src, ".rvc", NULL);
 		if (srp) {
 			if (file_copyrf (srp, drp)) {
-				if (r_vc_reset (dst)) {
-					ret = true;
-				} else {
-					eprintf("Failed to reset\n");
+				Rvc *dst_repo = r_vc_open (dst);
+				if (dst_repo) {
+					if (r_vc_reset (dst_repo)) {
+						ret = true;
+					} else {
+						R_LOG_ERROR("Failed to reset");
+					}
 				}
 			} else {
-				eprintf ("Failed to copy files\n");
+				R_LOG_ERROR ("Failed to copy files");
 			}
 			free (srp);
 		}
@@ -1118,11 +1073,34 @@ R_API bool r_vc_clone(const char *src, const char *dst) {
 
 // GIT commands as APIs
 
-R_API bool r_vc_git_init(const char *path) {
+//TODO: unify the rvc and git apis
+
+R_API Rvc *r_vc_git_open(const char *path) {
+	char *git_path = r_file_new (path, ".git", NULL);
+	if (!git_path || !r_file_is_directory (git_path)) {
+		free (git_path);
+		return NULL;
+	}
+	free (git_path);
+	Rvc *vc = R_NEW (Rvc);
+	if (!vc) {
+		return NULL;
+	}
+	vc->path = r_str_new (path);
+	if (!vc->path) {
+		free (vc);
+		return NULL;
+	}
+	vc->db = NULL;
+	vc->type = VC_GIT;
+	return vc;
+}
+
+R_API Rvc *r_vc_git_init(const char *path) {
 	char *escpath = r_str_escape (path);
 	int ret = r_sys_cmdf ("git init \"%s\"", escpath);
 	free (escpath);
-	return !ret;
+	return !ret? r_vc_git_open (path) : NULL;
 }
 
 R_API bool r_vc_git_branch(const char *path, const char *name) {
@@ -1185,12 +1163,12 @@ R_API bool r_vc_git_commit(const char *path, const char *message) {
 	return res == 0;
 }
 
-R_API bool r_vc_reset(const char *rp) {
-	if (!repo_exists (rp)) {
+R_API bool r_vc_reset(Rvc *rvc) {
+	if (!repo_exists (rvc->path)) {
 		return false;
 	}
 	bool ret = true;
-	RList *uncommitted = r_vc_get_uncommitted (rp);
+	RList *uncommitted = r_vc_get_uncommitted (rvc);
 	if (!uncommitted) {
 		return false;
 	}
@@ -1199,12 +1177,12 @@ R_API bool r_vc_reset(const char *rp) {
 	r_list_foreach (uncommitted, iter, fp) {
 		char *blobp;
 		{
-			char *p = absp2rp (rp, fp);
+			char *p = absp2rp (rvc, fp);
 			if (!p) {
 				ret = false;
 				break;
 			}
-			char *b = find_blob_hash (rp, p);
+			char *b = find_blob_hash (rvc, p);
 			if (!b || !strcmp (b, "-")) {
 				free (p);
 				if (!r_file_rm (fp)) {
@@ -1214,7 +1192,7 @@ R_API bool r_vc_reset(const char *rp) {
 				continue;
 
 			}
-			blobp = r_file_new (rp, ".rvc", "blobs", b, NULL);
+			blobp = r_file_new (rvc->path, ".rvc", "blobs", b, NULL);
 			free (b);
 		}
 		if (!blobp) {
@@ -1233,15 +1211,19 @@ R_API bool r_vc_reset(const char *rp) {
 
 //Access both git and rvc functionality from one set of functions
 
-R_API bool rvc_git_init(const RCore *core, const char *rp) {
+R_API Rvc *rvc_git_init(const RCore *core, const char *path) {
 	if (!strcmp (r_config_get (core->config, "prj.vc.type"), "git")) {
-		return r_vc_git_init (rp);
+		return r_vc_git_init (path);
 	}
 	printf ("rvc is just for testing please don't use it\n");
-	return r_vc_new (rp);
+	Rvc *rvc = r_vc_new (path);
+	if (!rvc || !r_vc_save (rvc)) {
+		return NULL;
+	}
+	return rvc;
 }
 
-R_API bool rvc_git_commit(RCore *core, const char *rp, const char *message, const char *author, const RList *files) {
+R_API bool rvc_git_commit(RCore *core, Rvc *rvc, const char *message, const char *author, const RList *files) {
 	const char *m = r_config_get (core->config, "prj.vc.message");
 	if (!*m) {
 		if (!r_cons_is_interactive ()) {
@@ -1250,41 +1232,44 @@ R_API bool rvc_git_commit(RCore *core, const char *rp, const char *message, cons
 		}
 	}
 	message = R_STR_ISEMPTY (message)? m : message;
-	if (!strcmp (r_config_get (core->config, "prj.vc.type"), "rvc")) {
+	if (rvc->type == VC_RVC) {
 		author = author? author : r_config_get (core->config, "cfg.user");
 		eprintf ("rvc is just for testing please don't use it\n");
-		return r_vc_commit (rp, message, author, files);
+		r_vc_commit (rvc, message, author, files);
+		return r_vc_save (rvc);
 	}
 	char *path;
 	RListIter *iter;
 	r_list_foreach (files, iter, path) {
-		if (!r_vc_git_add (rp, path)) {
+		if (!r_vc_git_add (rvc->path, path)) {
 			return false;
 		}
 	}
-	return r_vc_git_commit (rp, message);
+	return r_vc_git_commit (rvc->path, message);
 }
 
-R_API bool rvc_git_branch(const RCore *core, const char *rp, const char *bname) {
-	if (!strcmp (r_config_get (core->config, "prj.vc.type"), "rvc")) {
+R_API bool rvc_git_branch(Rvc *rvc, const char *bname) {
+	if (rvc->type == VC_RVC) {
 		eprintf ("rvc is just for testing please don't use it\n");
-		return r_vc_branch (rp, bname);
+		r_vc_branch (rvc, bname);
+		return r_vc_save(rvc);
 	}
-	return !r_vc_git_branch (rp, bname);
+	return !r_vc_git_branch (rvc->path, bname);
 }
 
-R_API bool rvc_git_checkout(const RCore *core, const char *rp, const char *bname) {
-	if (!strcmp (r_config_get (core->config, "prj.vc.type"), "rvc")) {
+R_API bool rvc_git_checkout(const RCore *core, Rvc *rvc, const char *bname) {
+	if (rvc->type == VC_GIT) {
 		eprintf ("rvc is just for testing please don't use it\n");
-		return r_vc_checkout (rp, bname);
+		r_vc_checkout (rvc, bname);
+		return r_vc_save(rvc);
 	}
-	return r_vc_git_checkout (rp, bname);
+	return r_vc_git_checkout (rvc->path, bname);
 }
 
-R_API bool rvc_git_repo_exists(const RCore *core, const char *rp) {
+R_API bool rvc_git_repo_exists(const RCore *core, const char *path) {
 	char *frp = !strcmp (r_config_get (core->config, "prj.vc.type"), "rvc")?
-		r_file_new (rp, ".rvc", NULL):
-		r_file_new (rp, ".git", NULL);
+		r_file_new (path, ".rvc", NULL):
+		r_file_new (path, ".git", NULL);
 	if (frp) {
 		bool ret = r_file_is_directory (frp);
 		free (frp);
@@ -1292,3 +1277,36 @@ R_API bool rvc_git_repo_exists(const RCore *core, const char *rp) {
 	}
 	return false;
 }
+
+R_API Rvc *r_vc_open(const char *rp) {
+	Rvc *repo = R_NEW(Rvc);
+	if (repo) {
+		repo->path = r_str_new (rp);
+		if (repo->path) {
+			repo->db = vcdb_open (rp) ;
+			if (repo->db) {
+				return repo;
+			}
+			free (repo->path);
+		}
+		free(repo);
+	}
+	return NULL;
+
+}
+
+R_API bool r_vc_save(Rvc *vc) {
+	sdb_sync(vc->db);
+	return true;
+}
+
+R_API void r_vc_close(Rvc *vc, bool save) {
+	if (vc) {
+		if (save) {
+			r_vc_save(vc);
+		}
+		sdb_close (vc->db);
+		free (vc->path);
+		free (vc);
+	}
+}
