@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2019 - GustavoLCR */
+/* radare - LGPL - Copyright 2019-2021 - GustavoLCR */
 
 #include <r_core.h>
-#include <TlHelp32.h>
-#include <windows_heap.h>
-#include "..\..\debug\p\native\maps\windows_maps.h"
+#include <tlhelp32.h>
+#include "heap/r_windows.h"
+#include "../debug/p/native/maps/windows_maps.h"
 
 /*
 *	Viewer discretion advised: Spaghetti code ahead
@@ -74,7 +74,7 @@ static size_t RtlpLFHKeyOffset = 0;
 	}\
 	hb->dwFlags |= ((flags) >> SHIFT) << SHIFT;
 
-static bool __is_windows_ten() {
+static bool __is_windows_ten(void) {
 	int major = 0;
 	RSysInfo *info = r_sys_info ();
 	if (info && info->version) {
@@ -120,7 +120,7 @@ static char *get_type(WPARAM flags) {
 	return r_str_newf ("%s %s%s", state, heaptype, type);
 }
 
-static bool init_func() {
+static bool init_func(void) {
 	HANDLE ntdll = LoadLibrary (TEXT ("ntdll.dll"));
 	if (!ntdll) {
 		return false;
@@ -256,7 +256,7 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 	RDebugMap *map;
 	bool found = false;
 	const char ntdll[] = "ntdll.dll";
-	static ut64 lastNdtllAddr = 0;
+	static R_TH_LOCAL ut64 lastNdtllAddr = 0;
 	r_list_foreach (modules, it, map) {
 		if (!strncmp(map->name, ntdll, sizeof (ntdll))) {
 			found = true;
@@ -264,21 +264,22 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 		}
 	}
 	if (!found) {
-		eprintf ("ntdll.dll not loaded.");
+		eprintf ("ntdll.dll not loaded.\n");
 		r_list_free (modules);
 		return false;
 	}
 	bool doopen = lastNdtllAddr != map->addr;
-	char *ntdllopen = dbg->corebind.cmdstrf (dbg->corebind.core, "ob~%s", ntdll);
+	char *ntdllopen = dbg->coreb.cmdstrf (dbg->coreb.core, "ob~%s", ntdll);
 	if (*ntdllopen) {
 		char *saddr = strtok (ntdllopen, " ");
-		for (int i = 0; i < 3; i++) {
+		size_t i;
+		for (i = 0; i < 3; i++) {
 			saddr = strtok (NULL, " ");
 		}
 		if (doopen) {
 			// Close to reopen at the right address
 			int fd = atoi (ntdllopen);
-			dbg->corebind.cmdstrf (dbg->corebind.core, "o-%d", fd);
+			dbg->coreb.cmdstrf (dbg->coreb.core, "o-%d", fd);
 			RtlpHpHeapGlobalsOffset = RtlpLFHKeyOffset = 0;
 		}
 	}
@@ -286,19 +287,19 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 	if (doopen) {
 		char *ntdllpath = r_lib_path ("ntdll");
 		eprintf ("Opening %s\n", ntdllpath);
-		dbg->corebind.cmdf (dbg->corebind.core, "o %s 0x%"PFMT64x"", ntdllpath, map->addr);
+		dbg->coreb.cmdf (dbg->coreb.core, "o %s 0x%"PFMT64x, ntdllpath, map->addr);
 		lastNdtllAddr = map->addr;
 		free (ntdllpath);
 	}
 	r_list_free (modules);
 
 	if (!RtlpHpHeapGlobalsOffset || !RtlpLFHKeyOffset) {
-		char *res = dbg->corebind.cmdstrf (dbg->corebind.core, "idpi~RtlpHpHeapGlobals");
+		char *res = dbg->coreb.cmdstrf (dbg->coreb.core, "idpi~RtlpHpHeapGlobals");
 		if (!*res) {
 			// Try downloading the pdb
 			free (res);
-			dbg->corebind.cmd (dbg->corebind.core, "idpd");
-			res = dbg->corebind.cmdstrf (dbg->corebind.core, "idpi~RtlpHpHeapGlobals");
+			dbg->coreb.cmd (dbg->coreb.core, "idpd");
+			res = dbg->coreb.cmdstrf (dbg->coreb.core, "idpi~RtlpHpHeapGlobals");
 		}
 		if (*res) {
 			RtlpHpHeapGlobalsOffset = r_num_math (NULL, res);
@@ -307,7 +308,7 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 			return false;
 		}
 		free (res);
-		res = dbg->corebind.cmdstrf (dbg->corebind.core, "idpi~RtlpLFHKey");
+		res = dbg->coreb.cmdstrf (dbg->coreb.core, "idpi~RtlpLFHKey");
 		if (*res) {
 			RtlpLFHKeyOffset = r_num_math (NULL, res);
 		}
@@ -316,10 +317,10 @@ static bool GetHeapGlobalsOffset(RDebug *dbg, HANDLE h_proc) {
 
 	if (doopen) {
 		// Close ntdll.dll
-		char *res = dbg->corebind.cmdstrf (dbg->corebind.core, "o~%s", ntdll);
+		char *res = dbg->coreb.cmdstrf (dbg->coreb.core, "o~%s", ntdll);
 		int fd = atoi (res);
 		free (res);
-		dbg->corebind.cmdf (dbg->corebind.core, "o-%d", fd);
+		dbg->coreb.cmdf (dbg->coreb.core, "o-%d", fd);
 	}
 	return true;
 }
@@ -340,7 +341,7 @@ static bool GetLFHKey(RDebug *dbg, HANDLE h_proc, bool segment, WPARAM *lfhKey)
 	}
 	if (!ReadProcessMemory (h_proc, (PVOID)lfhKeyLocation, lfhKey, sizeof (WPARAM), NULL)) {
 		r_sys_perror ("ReadProcessMemory");
-		eprintf ("LFH key not found.\n");
+		R_LOG_WARN ("LFH key not found");
 		*lfhKey = 0;
 		return false;
 	}
@@ -478,7 +479,7 @@ static PDEBUG_BUFFER InitHeapInfo(RDebug *dbg, DWORD mask) {
 		}
 		HANDLE h_proc = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dbg->pid);
 		if (!h_proc) {
-			R_LOG_ERROR ("OpenProcess failed\n");
+			R_LOG_ERROR ("OpenProcess failed");
 			free (heapInfo);
 			RtlDestroyQueryDebugBuffer (db);
 			return NULL;
@@ -542,7 +543,8 @@ static bool __lfh_segment_loop(HANDLE h_proc, PHeapBlockBasicInfo *blocks, SIZE_
 		ReadProcessMemory (h_proc, (void *)next, &subsegment, sizeof (HEAP_LFH_SUBSEGMENT), NULL);
 		subsegment.BlockOffsets.EncodedData ^= (DWORD)lfhKey ^ ((DWORD)next >> 0xC);
 		WPARAM mask = 1, offset = 0;
-		for (int l = 0; l < subsegment.BlockCount; l++) {
+		int l;
+		for (l = 0; l < subsegment.BlockCount; l++) {
 			if (!mask) {
 				mask = 1;
 				offset++;
@@ -590,7 +592,8 @@ static bool GetSegmentHeapBlocks(RDebug *dbg, HANDLE h_proc, PVOID heapBase, PHe
 
 	// LFH
 	byte numBuckets = _countof (segheapHeader.LfhContext.Buckets);
-	for (int j = 0; j < numBuckets; j++) {
+	int j;
+	for (j = 0; j < numBuckets; j++) {
 		if ((WPARAM)segheapHeader.LfhContext.Buckets[j] & 1) {
 			continue;
 		}
@@ -647,7 +650,8 @@ static bool GetSegmentHeapBlocks(RDebug *dbg, HANDLE h_proc, PVOID heapBase, PHe
 	WPARAM RtlpHpHeapGlobal;
 	ReadProcessMemory (h_proc, (PVOID)RtlpHpHeapGlobalsOffset, &RtlpHpHeapGlobal, sizeof (WPARAM), &bytesRead);
 	// Backend Blocks (And VS)
-	for (int i = 0; i < 2; i++) {
+	int i;
+	for (i = 0; i < 2; i++) {
 		HEAP_SEG_CONTEXT ctx = segheapHeader.SegContexts[i];
 		WPARAM ctxFirstEntry = (WPARAM)heapBase + offsetof (SEGMENT_HEAP, SegContexts) + sizeof (HEAP_SEG_CONTEXT) * i + offsetof (HEAP_SEG_CONTEXT, SegmentListHead);
 		HEAP_PAGE_SEGMENT pageSegment;
@@ -723,12 +727,12 @@ static PDEBUG_BUFFER GetHeapBlocks(DWORD pid, RDebug *dbg) {
 	HANDLE h_proc = NULL;
 	PDEBUG_BUFFER db = InitHeapInfo (dbg, PDI_HEAPS);
 	if (!db || !db->HeapInformation) {
-		R_LOG_ERROR ("InitHeapInfo Failed\n");
+		R_LOG_ERROR ("InitHeapInfo Failed");
 		goto err;
 	}
 	h_proc = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
 	if (!h_proc) {
-		R_LOG_ERROR ("OpenProcess failed\n");
+		R_LOG_ERROR ("OpenProcess failed");
 		goto err;
 	}
 
@@ -736,7 +740,7 @@ static PDEBUG_BUFFER GetHeapBlocks(DWORD pid, RDebug *dbg) {
 	if (!GetLFHKey (dbg, h_proc, false, &lfhKey)) {
 		RtlDestroyQueryDebugBuffer (db);
 		CloseHandle (h_proc);
-		eprintf ("GetHeapBlocks: Failed to get LFH key.\n");
+		R_LOG_ERROR ("GetHeapBlocks: Failed to get LFH key");
 		return NULL;
 	}
 
@@ -754,7 +758,7 @@ static PDEBUG_BUFFER GetHeapBlocks(DWORD pid, RDebug *dbg) {
 		SIZE_T allocated = 128 * sizeof (HeapBlockBasicInfo);
 		PHeapBlockBasicInfo blocks = calloc (allocated, 1);
 		if (!blocks) {
-			R_LOG_ERROR ("Memory Allocation failed\n");
+			R_LOG_ERROR ("Memory Allocation failed");
 			goto err;
 		}
 
@@ -836,7 +840,9 @@ static PDEBUG_BUFFER GetHeapBlocks(DWORD pid, RDebug *dbg) {
 					ReadProcessMemory (h_proc, userdata.BusyBitmap.Buffer, bitmap, bitmapsz, &bytesRead);
 					WPARAM mask = 1;
 					// Walk through the busy bitmap
-					for (int j = 0, offset = 0; j < userdata.BusyBitmap.SizeOfBitMap; j++) {
+					int j;
+					size_t offset;
+					for (j = 0, offset = 0; j < userdata.BusyBitmap.SizeOfBitMap; j++) {
 						if (!mask) {
 							mask = 1;
 							offset++;
@@ -930,7 +936,8 @@ err:
 		CloseHandle (h_proc);
 	}
 	if (db) {
-		for (int i = 0; i < heapInfo->count; i++) {
+		int i;
+		for (i = 0; i < heapInfo->count; i++) {
 			PDEBUG_HEAP_INFORMATION heap = &heapInfo->heaps[i];
 			free_extra_info (heap);
 			R_FREE (heap->Blocks);
@@ -947,12 +954,12 @@ static PHeapBlock GetSingleSegmentBlock(RDebug *dbg, HANDLE h_proc, PSEGMENT_HEA
 	*/
 	PHeapBlock hb = R_NEW0 (HeapBlock);
 	if (!hb) {
-		R_LOG_ERROR ("GetSingleSegmentBlock: Allocation failed.\n");
+		R_LOG_ERROR ("GetSingleSegmentBlock: Allocation failed");
 		return NULL;
 	}
 	PHeapBlockExtraInfo extra = R_NEW0 (HeapBlockExtraInfo);
 	if (!extra) {
-		R_LOG_ERROR ("GetSingleSegmentBlock: Allocation failed.\n");
+		R_LOG_ERROR ("GetSingleSegmentBlock: Allocation failed");
 		goto err;
 	}
 	hb->extraInfo = extra;
@@ -1051,7 +1058,7 @@ static PHeapBlock GetSingleBlock(RDebug *dbg, ut64 offset) {
 	PHeapBlockExtraInfo extra = NULL;
 
 	if (!hb) {
-		R_LOG_ERROR ("GetSingleBlock: Allocation failed.\n");
+		R_LOG_ERROR ("GetSingleBlock: Allocation failed");
 		return NULL;
 	}
 	HANDLE h_proc = OpenProcess (PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dbg->pid);
@@ -1065,13 +1072,14 @@ static PHeapBlock GetSingleBlock(RDebug *dbg, ut64 offset) {
 	}
 	extra = R_NEW0 (HeapBlockExtraInfo);
 	if (!extra) {
-		R_LOG_ERROR ("GetSingleBlock: Allocation failed.\n");
+		R_LOG_ERROR ("GetSingleBlock: Allocation failed");
 		goto err;
 	}
 	WPARAM NtLFHKey;
 	GetLFHKey (dbg, h_proc, false, &NtLFHKey);
 	PHeapInformation heapInfo = db->HeapInformation;
-	for (int i = 0; i < heapInfo->count; i++) {
+	int i;
+	for (i = 0; i < heapInfo->count; i++) {
 		DEBUG_HEAP_INFORMATION heap = heapInfo->heaps[i];
 		if (is_segment_heap (h_proc, heap.Base)) {
 			free (hb);
@@ -1158,8 +1166,8 @@ err:
 	return NULL;
 }
 
-static RTable *__new_heapblock_tbl() {
-	RTable *tbl = r_table_new ();
+static RTable *__new_heapblock_tbl(void) {
+	RTable *tbl = r_table_new ("heap");
 	r_table_add_column (tbl, r_table_type ("number"), "HeaderAddress", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "UserAddress", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Size", -1);
@@ -1177,14 +1185,14 @@ static void w32_list_heaps(RCore *core, const char format) {
 			db = GetHeapBlocks (pid, core->dbg);
 		}
 		if (!db) {
-			eprintf ("Couldn't get heap info.\n");
+			R_LOG_WARN ("Couldn't get heap info");
 			return;
 		}
 	}
 	PHeapInformation heapInfo = db->HeapInformation;
 	CHECK_INFO (heapInfo);
 	int i;
-	RTable *tbl = r_table_new ();
+	RTable *tbl = r_table_new ("heaps");
 	r_table_add_column (tbl, r_table_type ("number"), "Address", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Blocks", -1);
 	r_table_add_column (tbl, r_table_type ("number"), "Allocated", -1);
@@ -1199,7 +1207,7 @@ static void w32_list_heaps(RCore *core, const char format) {
 			pj_kN (pj, "address", (ut64)heap.Base);
 			pj_kN (pj, "count", (ut64)heap.BlockCount);
 			pj_kN (pj, "allocated", (ut64)heap.Allocated);
-			pj_kN (pj, "commited", (ut64)heap.Committed);
+			pj_kN (pj, "committed", (ut64)heap.Committed);
 			pj_end (pj);
 			break;
 		default:
@@ -1231,7 +1239,7 @@ static void w32_list_heaps_blocks(RCore *core, const char format) {
 		db = InitHeapInfo (core->dbg, PDI_HEAPS | PDI_HEAP_BLOCKS);
 	}
 	if (!db) {
-		eprintf ("Couldn't get heap info.\n");
+		R_LOG_ERROR ("Couldn't get heap info");
 		return;
 	}
 	PHeapInformation heapInfo = db->HeapInformation;
@@ -1246,7 +1254,7 @@ static void w32_list_heaps_blocks(RCore *core, const char format) {
 		switch (format) {
 		case 'f':
 			if (heapInfo->heaps[i].BlockCount > 50000) {
-				go = r_cons_yesno ('n', "Are you sure you want to add %"PFMT64u" flags? (y/N)", heapInfo->heaps[i].BlockCount);
+				go = r_cons_yesno ('n', "Are you sure you want to add %lu flags? (y/N)", heapInfo->heaps[i].BlockCount);
 			}
 			break;
 		case 'j':
@@ -1269,7 +1277,7 @@ static void w32_list_heaps_blocks(RCore *core, const char format) {
 				switch (format) {
 				case 'f':
 				{
-					char *name = r_str_newf ("alloc.%"PFMT64x"", address);
+					char *name = r_str_newf ("alloc.%"PFMT64x, address);
 					r_flag_set (core->flags, name, address, block->dwSize);
 					free (name);
 					break;
