@@ -97,7 +97,7 @@ static int r_debug_handle_signals(RDebug *dbg) {
 #if __KFBSD__
 	return bsd_handle_signals (dbg);
 #else
-	eprintf ("Warning: signal handling is not supported on this platform\n");
+	R_LOG_WARN ("signal handling is not supported on this platform");
 	return 0;
 #endif
 }
@@ -239,7 +239,7 @@ static bool r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
 		r_list_foreach (dbg->threads, it, th) {
 			ret = r_debug_ptrace (dbg, PTRACE_CONT, th->pid, 0, 0);
 			if (ret) {
-				eprintf ("Error: (%d) is running or dead.\n", th->pid);
+				R_LOG_ERROR ("(%d) is running or dead", th->pid);
 			}
 		}
 	} else {
@@ -301,7 +301,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	RW32Dw *wrap = dbg->user;
 
 	if (pid == -1) {
-		eprintf ("ERROR: r_debug_native_wait called with pid -1\n");
+		R_LOG_ERROR ("r_debug_native_wait called with pid -1");
 		return R_DEBUG_REASON_ERROR;
 	}
 
@@ -314,23 +314,23 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 			}
 
 			/* Check if autoload PDB is set, and load PDB information if yes */
-			RCore *core = dbg->corebind.core;
-			bool autoload_pdb = dbg->corebind.cfggeti (core, "pdb.autoload");
+			RCore *core = dbg->coreb.core;
+			bool autoload_pdb = dbg->coreb.cfggeti (core, "pdb.autoload");
 			if (autoload_pdb) {
 				PLIB_ITEM lib = r->lib;
-				dbg->corebind.cmdf (core, "\"o \\\"%s\\\" 0x%p\"", lib->Path, lib->BaseOfDll);
-				char *o_res = dbg->corebind.cmdstrf (core, "o~+%s", lib->Name);
+				dbg->coreb.cmdf (core, "\"o \\\"%s\\\" 0x%p\"", lib->Path, lib->BaseOfDll);
+				char *o_res = dbg->coreb.cmdstrf (core, "o~+%s", lib->Name);
 				int fd = atoi (o_res);
 				free (o_res);
 				if (fd) {
-					char *pdb_file = dbg->corebind.cmdstr (core, "i~dbg_file");
+					char *pdb_file = dbg->coreb.cmdstr (core, "i~dbg_file");
 					if (pdb_file && (r_str_trim (pdb_file), *pdb_file)) {
 						if (!r_file_exists (pdb_file + 9)) {
-							dbg->corebind.cmdf (core, "idpd");
+							dbg->coreb.cmdf (core, "idpd");
 						}
-						dbg->corebind.cmdf (core, "idp");
+						dbg->coreb.cmdf (core, "idp");
 					}
-					dbg->corebind.cmdf (core, "o-%d", fd);
+					dbg->coreb.cmdf (core, "o-%d", fd);
 				}
 			}
 			r_debug_info_free (r);
@@ -428,7 +428,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	RDebugReasonType reason = R_DEBUG_REASON_UNKNOWN;
 
 	if (pid == -1) {
-		eprintf ("ERROR: r_debug_native_wait called with pid -1\n");
+		R_LOG_ERROR ("r_debug_native_wait called with pid -1");
 		return R_DEBUG_REASON_ERROR;
 	}
 
@@ -441,7 +441,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	RDebugReasonType reason = R_DEBUG_REASON_UNKNOWN;
 
 	if (pid == -1) {
-		eprintf ("ERROR: r_debug_native_wait called with pid -1\n");
+		R_LOG_ERROR ("r_debug_native_wait called with pid -1");
 		return R_DEBUG_REASON_ERROR;
 	}
 
@@ -537,18 +537,18 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 #endif
 		} else if (status == 1) {
 			/* XXX(jjd): does this actually happen? */
-			eprintf ("EEK DEAD DEBUGEE!\n");
+			R_LOG_ERROR ("EEK DEAD DEBUGEE!");
 			reason = R_DEBUG_REASON_DEAD;
 		} else if (status == 0) {
 			/* XXX(jjd): does this actually happen? */
-			eprintf ("STATUS=0?!?!?!?\n");
+			R_LOG_ERROR ("STATUS=0?!?!?!?");
 			reason = R_DEBUG_REASON_DEAD;
 		} else {
 			if (ret != pid) {
 				reason = R_DEBUG_REASON_NEW_PID;
 			} else {
 				/* ugh. still don't know :-/ */
-				eprintf ("CRAP. returning from wait without knowing why...\n");
+				R_LOG_ERROR ("CRAP. returning from wait without knowing why");
 			}
 		}
 	}
@@ -786,11 +786,11 @@ static int thp_mode(void) {
 }
 #endif
 
-static int linux_map_thp(RDebug *dbg, ut64 addr, int size) {
+static bool linux_map_thp(RDebug *dbg, ut64 addr, int size) {
 #if !defined(__ANDROID__) && defined(MADV_HUGEPAGE)
 	RBuffer *buf = NULL;
 	char code[1024];
-	int ret = true;
+	bool ret = true;
 	char *asm_list[] = {
 		"x86", "x86.as",
 		"x64", "x86.as",
@@ -800,7 +800,7 @@ static int linux_map_thp(RDebug *dbg, ut64 addr, int size) {
 	const size_t thpsize = 1<<21;
 
 	if (size % thpsize) {
-		eprintf ("size not a power of huge pages size\n");
+		R_LOG_ERROR ("size not a power of huge pages size");
 		return false;
 	}
 #if __linux__
@@ -808,7 +808,7 @@ static int linux_map_thp(RDebug *dbg, ut64 addr, int size) {
 	// even though the address might not have the 'hg'
 	// vmflags
 	if (thp_mode () != 1) {
-		eprintf ("transparent huge page mode is not in madvise mode\n");
+		R_LOG_ERROR ("transparent huge page mode is not in madvise mode");
 		return false;
 	}
 #endif
@@ -823,19 +823,22 @@ static int linux_map_thp(RDebug *dbg, ut64 addr, int size) {
 	r_egg_setup (dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
 	r_egg_load (dbg->egg, code, 0);
 	if (!r_egg_compile (dbg->egg)) {
-		eprintf ("Cannot compile.\n");
+		R_LOG_ERROR ("Cannot compile");
 		goto err_linux_map_thp;
 	}
 	if (!r_egg_assemble_asm (dbg->egg, asm_list)) {
-		eprintf ("r_egg_assemble: invalid assembly\n");
+		R_LOG_ERROR ("r_egg_assemble: invalid assembly");
 		goto err_linux_map_thp;
 	}
 	buf = r_egg_get_bin (dbg->egg);
 	if (buf) {
+		ut64 tmpsz, retval;
 		r_reg_arena_push (dbg->reg);
-		ut64 tmpsz;
 		const ut8 *tmp = r_buf_data (buf, &tmpsz);
-		ret = r_debug_execute (dbg, tmp, tmpsz, 1) == 0;
+		if (!r_debug_execute (dbg, tmp, tmpsz, &retval, true, false)) {
+			R_LOG_ERROR ("Failed to execute code");
+		}
+		ret = (retval == 0);
 		r_reg_arena_pop (dbg->reg);
 	}
 err_linux_map_thp:
@@ -877,11 +880,11 @@ static RDebugMap* linux_map_alloc(RDebug *dbg, ut64 addr, int size, bool thp) {
 	r_egg_setup (dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
 	r_egg_load (dbg->egg, code, 0);
 	if (!r_egg_compile (dbg->egg)) {
-		eprintf ("Cannot compile.\n");
+		R_LOG_ERROR ("Cannot compile");
 		goto err_linux_map_alloc;
 	}
 	if (!r_egg_assemble_asm (dbg->egg, asm_list)) {
-		eprintf ("r_egg_assemble: invalid assembly\n");
+		R_LOG_ERROR ("r_egg_assemble: invalid assembly");
 		goto err_linux_map_alloc;
 	}
 	buf = r_egg_get_bin (dbg->egg);
@@ -891,13 +894,15 @@ static RDebugMap* linux_map_alloc(RDebug *dbg, ut64 addr, int size, bool thp) {
 		r_reg_arena_push (dbg->reg);
 		ut64 tmpsz;
 		const ut8 *tmp = r_buf_data (buf, &tmpsz);
-		map_addr = r_debug_execute (dbg, tmp, tmpsz, 1);
+		if (!r_debug_execute (dbg, tmp, tmpsz, &map_addr, true, false)) {
+			R_LOG_ERROR ("Failed to execute code");
+			goto err_linux_map_alloc;
+		}
 		r_reg_arena_pop (dbg->reg);
-		if (map_addr != (ut64)-1) {
+		if (map_addr < UT64_MAX) {
 			if (thp) {
 				if (!linux_map_thp (dbg, map_addr, size)) {
-					// Not overly dramatic
-					eprintf ("map promotion to huge page failed\n");
+					R_LOG_WARN ("map promotion to huge page failed");
 				}
 			}
 			r_debug_map_sync (dbg);
@@ -911,7 +916,7 @@ err_linux_map_alloc:
 static int linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 	RBuffer *buf = NULL;
 	char code[1024];
-	int ret = 0;
+	ut64 ret = 0;
 	char *asm_list[] = {
 		"x86", "x86.as",
 		"x64", "x86.as",
@@ -928,11 +933,11 @@ static int linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 	r_egg_setup (dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
 	r_egg_load (dbg->egg, code, 0);
 	if (!r_egg_compile (dbg->egg)) {
-		eprintf ("Cannot compile.\n");
+		R_LOG_ERROR ("Cannot compile");
 		goto err_linux_map_dealloc;
 	}
 	if (!r_egg_assemble_asm (dbg->egg, asm_list)) {
-		eprintf ("r_egg_assemble: invalid assembly\n");
+		R_LOG_ERROR ("r_egg_assemble: invalid assembly");
 		goto err_linux_map_dealloc;
 	}
 	buf = r_egg_get_bin (dbg->egg);
@@ -940,11 +945,13 @@ static int linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 		r_reg_arena_push (dbg->reg);
 		ut64 tmpsz;
 		const ut8 *tmp = r_buf_data (buf, &tmpsz);
-		ret = r_debug_execute (dbg, tmp, tmpsz, 1) == 0;
+		if (!r_debug_execute (dbg, tmp, tmpsz, &ret, true, false)) {
+			R_LOG_ERROR ("Failed to execute code");
+		}
 		r_reg_arena_pop (dbg->reg);
 	}
 err_linux_map_dealloc:
-	return ret;
+	return (int)ret;
 }
 #endif
 
@@ -1479,11 +1486,14 @@ static RList *xnu_desc_list(int pid) {
 #define xwr2rwx(x) ((x&1)<<2) | (x&2) | ((x&4)>>2)
 	RDebugDesc *desc;
 	RList *ret = r_list_new ();
+	if (!ret) {
+		return NULL;
+	}
 	struct vnode_fdinfowithpath vi;
 	int i, nb, type = 0;
 	int maxfd = getMaxFiles();
 
-	for (i=0 ; i<maxfd; i++) {
+	for (i = 0 ; i < maxfd; i++) {
 		nb = proc_pidfdinfo (pid, i, PROC_PIDFDVNODEPATHINFO, &vi, sizeof (vi));
 		if (nb < 1) {
 			continue;
@@ -1541,11 +1551,11 @@ static int r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perm
 	r_egg_setup(dbg->egg, dbg->arch, 8 * dbg->bits, 0, 0);
 	r_egg_load (dbg->egg, code, 0);
 	if (!r_egg_compile (dbg->egg)) {
-		eprintf ("Cannot compile.\n");
+		R_LOG_ERROR ("Cannot compile");
 		return false;
 	}
 	if (!r_egg_assemble (dbg->egg)) {
-		eprintf ("r_egg_assemble: invalid assembly\n");
+		R_LOG_ERROR ("r_egg_assemble: invalid assembly");
 		return false;
 	}
 	buf = r_egg_get_bin (dbg->egg);
@@ -1553,7 +1563,9 @@ static int r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perm
 		r_reg_arena_push (dbg->reg);
 		ut64 tmpsz;
 		const ut8 *tmp = r_buf_data (buf, &tmpsz);
-		r_debug_execute (dbg, tmp, tmpsz, 1);
+		if (!r_debug_execute (dbg, tmp, tmpsz, NULL, true, false)) {
+			R_LOG_ERROR ("Failed to execute code");
+		}
 		r_reg_arena_pop (dbg->reg);
 		return true;
 	}
@@ -1574,7 +1586,7 @@ static int r_debug_setup_ownership(int fd, RDebug *dbg) {
 	RDebugInfo *info = r_debug_info (dbg, NULL);
 
 	if (!info) {
-		eprintf ("Error while getting debug info.\n");
+		R_LOG_ERROR ("getting debug info");
 		return -1;
 	}
 	fchown (fd, info->uid, info->gid);
@@ -1616,7 +1628,11 @@ RDebugPlugin r_debug_plugin_native = {
 #elif __aarch64__ || __arm64__
 	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
 	.arch = "arm",
+#if __APPLE__
+	.canstep = true,
+#else
 	.canstep = false,
+#endif
 #elif __arm__
 	.bits = R_SYS_BITS_16 | R_SYS_BITS_32 | R_SYS_BITS_64,
 	.arch = "arm",
