@@ -7,10 +7,14 @@
 #if __x86_64__ || __i386__ || __arm__ || __arm64__
 #include <sys/uio.h>
 #include <sys/ptrace.h>
-#include <asm/ptrace.h>
 #include "linux_coredump.h"
 #include "linux_ptrace.h"
 
+/* Required for ARM_VFPREGS_SIZE on musl */
+#if __arm__ || __arm64__
+#include <asm/ptrace.h>
+#endif
+
 /* For compatibility */
 #if __x86_64__ || __arm64__
 typedef Elf64_auxv_t elf_auxv_t;
@@ -75,6 +79,7 @@ static char *prpsinfo_get_psargs(char *buffer, int len) {
 
 static prpsinfo_t *linux_get_prpsinfo(RDebug *dbg, proc_per_process_t *proc_data) {
 	const char *prog_states = "RSDTZW"; /* fs/binfmt_elf.c from kernel */
+	r_strf_buffer (64);
 	const char *basename = NULL;
 	char *buffer, *pfname = NULL, *ppsargs = NULL, *file = NULL;
 	prpsinfo_t *p;
@@ -83,16 +88,16 @@ static prpsinfo_t *linux_get_prpsinfo(RDebug *dbg, proc_per_process_t *proc_data
 
 	p = R_NEW0 (prpsinfo_t);
 	if (!p) {
-		eprintf ("Couldn't allocate memory for prpsinfo_t\n");
+		R_LOG_ERROR ("Couldn't allocate memory for prpsinfo_t");
 		return NULL;
 	}
 
 	p->pr_pid = mypid = dbg->pid;
 	/* Start filling pr_fname and pr_psargs */
-	file = sdb_fmt ("/proc/%d/cmdline", mypid);
+	file = r_strf ("/proc/%d/cmdline", mypid);
 	buffer = r_file_slurp (file, &len);
 	if (!buffer) {
-		eprintf ("buffer NULL\n");
+		R_LOG_ERROR ("buffer NULL");
 		goto error;
 	}
 	buffer[len] = 0;
@@ -133,7 +138,8 @@ error:
 static proc_per_thread_t *get_proc_thread_content(int pid, int tid) {
 	char *temp_p_sigpend, *temp_p_sighold, *p_sigpend, *p_sighold;
 	size_t size;
-	const char * file = sdb_fmt ("/proc/%d/task/%d/stat", pid, tid);
+	r_strf_buffer (64);
+	const char * file = r_strf ("/proc/%d/task/%d/stat", pid, tid);
 
 	char *buff = r_file_slurp (file, &size);
 	if (!buff) {
@@ -159,8 +165,8 @@ static proc_per_thread_t *get_proc_thread_content(int pid, int tid) {
 		free (buff);
 	}
 
-        /* /proc/[pid]/status for uid, gid, sigpend and sighold */
-	file = sdb_fmt ("/proc/%d/task/%d/status", pid, tid);
+	/* /proc/[pid]/status for uid, gid, sigpend and sighold */
+	file = r_strf ("/proc/%d/task/%d/status", pid, tid);
 	buff = r_file_slurp (file, &size);
 	if (!buff) {
 		free (t);
@@ -225,7 +231,7 @@ static prstatus_t *linux_get_prstatus(RDebug *dbg, int pid, int tid, proc_conten
 	p->pr_cstime.tv_usec = (proc_data->per_thread->cstime % 1000) / 1000;
 
 	if (r_debug_ptrace (dbg, PTRACE_GETREGS, tid, NULL, &regs) < 0) {
-		perror ("PTRACE_GETREGS");
+		r_sys_perror ("PTRACE_GETREGS");
 		R_FREE (proc_data->per_thread);
 		free (p);
 		return NULL;
@@ -239,7 +245,7 @@ static elf_fpregset_t *linux_get_fp_regset(RDebug *dbg, int pid) {
 	elf_fpregset_t *p = R_NEW0 (elf_fpregset_t);
 	if (p) {
 		if (r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, p) < 0) {
-			perror ("PTRACE_GETFPREGS");
+			r_sys_perror ("PTRACE_GETFPREGS");
 			free (p);
 			return NULL;
 		}
@@ -255,7 +261,7 @@ static siginfo_t *linux_get_siginfo(RDebug *dbg, int pid) {
 	}
 	int ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, pid, 0, (r_ptrace_data_t)(size_t)siginfo);
 	if (ret == -1 || !siginfo->si_signo) {
-		perror ("PTRACE_GETSIGINFO");
+		r_sys_perror ("PTRACE_GETSIGINFO");
 		free (siginfo);
 		return NULL;
 	}
@@ -562,7 +568,7 @@ static auxv_buff_t *linux_get_auxv(RDebug *dbg) {
 	int auxv_entries;
 	size_t size;
 
-	const char *file = sdb_fmt ("/proc/%d/auxv", dbg->pid);
+	r_strf_var (file, 32, "/proc/%d/auxv", dbg->pid);
 	buff = r_file_slurp (file, &size);
 	if (!buff) {
 		return NULL;
@@ -779,12 +785,13 @@ static bool dump_elf_map_content(RDebug *dbg, RBuffer *dest, linux_map_entry_t *
 	return true;
 }
 
-static proc_per_process_t *get_proc_process_content (RDebug *dbg) {
+static proc_per_process_t *get_proc_process_content(RDebug *dbg) {
 	proc_per_process_t *p;
+	r_strf_buffer (32);
 	char *temp_p_uid, *temp_p_gid, *p_uid, *p_gid;
 	ut16 filter_flags, default_filter_flags = 0x33;
 	char *buff;
-	const char *file = sdb_fmt ("/proc/%d/stat", dbg->pid);
+	const char *file = r_strf ("/proc/%d/stat", dbg->pid);
 	size_t size;
 
 	buff = r_file_slurp (file, &size);
@@ -815,10 +822,10 @@ static proc_per_process_t *get_proc_process_content (RDebug *dbg) {
 	}
 	if (!p->num_threads || p->num_threads < 1) {
 		free (p);
-		eprintf ("Warning: number of threads is < 1\n");
+		R_LOG_WARN ("number of threads is < 1");
 		return NULL;
 	}
-	file = sdb_fmt ("/proc/%d/status", dbg->pid);
+	file = r_strf ("/proc/%d/status", dbg->pid);
 	buff = r_file_slurp (file, &size);
 	if (!buff) {
 		free (p);
@@ -858,7 +865,7 @@ static proc_per_process_t *get_proc_process_content (RDebug *dbg) {
 
 	free (buff);
 	/* Check the coredump_filter value if we have*/
-	file = sdb_fmt ("/proc/%d/coredump_filter", dbg->pid);
+	file = r_strf ("/proc/%d/coredump_filter", dbg->pid);
 	buff = r_file_slurp (file, &size);
 	if (buff) {
 		sscanf (buff, "%hx", &filter_flags);
@@ -903,7 +910,7 @@ static bool dump_elf_sheader_pxnum(RBuffer *dest, elf_shdr_t *shdr) {
 }
 
 #if __i386__
-static elf_fpxregset_t *linux_get_fpx_regset (RDebug *dbg, int tid) {
+static elf_fpxregset_t *linux_get_fpx_regset(RDebug *dbg, int tid) {
 #ifdef PTRACE_GETREGSET
 	struct iovec transfer;
 	elf_fpxregset_t *fpxregset = R_NEW0 (elf_fpxregset_t);
@@ -911,7 +918,7 @@ static elf_fpxregset_t *linux_get_fpx_regset (RDebug *dbg, int tid) {
 		transfer.iov_base = fpxregset;
 		transfer.iov_len = sizeof (elf_fpxregset_t);
 		if (r_debug_ptrace (dbg, PTRACE_GETREGSET, tid, (void *)NT_PRXFPREG, &transfer) < 0) {
-			perror ("linux_get_fpx_regset");
+			r_sys_perror ("linux_get_fpx_regset");
 			R_FREE (fpxregset);
 		}
 	}
@@ -933,7 +940,7 @@ void *linux_get_xsave_data (RDebug *dbg, int tid, ut32 size) {
 	transfer.iov_base = xsave_data;
 	transfer.iov_len = size;
 	if (r_debug_ptrace (dbg, PTRACE_GETREGSET, tid, (void *)NT_X86_XSTATE, &transfer) < 0) {
-		perror ("linux_get_xsave_data");
+		r_sys_perror ("linux_get_xsave_data");
 		free (xsave_data);
 		return NULL;
 	}
@@ -953,7 +960,7 @@ void *linux_get_arm_vfp_data (RDebug *dbg, int tid) {
 	}
 
 	if (r_debug_ptrace (dbg, PTRACE_GETVFPREGS, tid, 0, vfp_data) < 0) {
-		perror ("linux_get_arm_vfp_data");
+		r_sys_perror ("linux_get_arm_vfp_data");
 		free (vfp_data);
 		return NULL;
 	}
@@ -1028,7 +1035,7 @@ void write_note_hdr (note_type_t type, ut8 **note_data) {
 	*note_data += size_note_hdr;
 }
 
-static int *get_unique_thread_id (RDebug *dbg, int n_threads) {
+static int *get_unique_thread_id(RDebug *dbg, int n_threads) {
 	RListIter *it;
 	RList *list;
 	RDebugPid *th;
@@ -1060,7 +1067,7 @@ static int *get_unique_thread_id (RDebug *dbg, int n_threads) {
 					/* The main thread is already being traced */
 					if (th->pid != dbg->pid) {
 						if (r_debug_ptrace (dbg, PTRACE_ATTACH, thread_id[i], 0, 0) < 0) {
-							perror ("Could not attach to thread");
+							r_sys_perror ("Could not attach to thread");
 						}
 					}
 					i++;
@@ -1075,10 +1082,10 @@ static int *get_unique_thread_id (RDebug *dbg, int n_threads) {
 
 void detach_threads (RDebug *dbg, int *thread_id, int n_threads) {
 	int i;
-	for(i = 0; i < n_threads ; i++) {
+	for (i = 0; i < n_threads ; i++) {
 		if (dbg->pid != thread_id[i]) {
 			if (r_debug_ptrace (dbg, PTRACE_DETACH, thread_id[i], 0, 0) < 0) {
-				perror ("PTRACE_DETACH");
+				r_sys_perror ("PTRACE_DETACH");
 			}
 		}
 	}
@@ -1139,14 +1146,14 @@ static ut8 *build_note_section(RDebug *dbg, elf_proc_note_t *elf_proc_note, proc
 		n_notes++;
 		type = NT_FPREGSET_T;
 		size += note_info[type].size_roundedup;
-                size += note_info[type].size_name;
+		size += note_info[type].size_name;
 		n_notes++;
 #if __i386__
 		type = NT_PRXFPREG_T;
 		if (note_info[type].size) {
 			fpx_flag = true;
 			size += note_info[type].size_roundedup;
-        	        size += note_info[type].size_name;
+			size += note_info[type].size_name;
 			n_notes++;
 		}
 #endif
@@ -1155,7 +1162,7 @@ static ut8 *build_note_section(RDebug *dbg, elf_proc_note_t *elf_proc_note, proc
 		if (note_info[type].size) {
 			xsave_flag = true;
 			size += note_info[type].size_roundedup;
-                	size += note_info[type].size_name;
+			size += note_info[type].size_name;
 			n_notes++;
 		}
 #endif
@@ -1343,7 +1350,7 @@ static int get_xsave_size(RDebug *dbg, int pid) {
 	local.iov_base = xstate_hdr;
 	local.iov_len = sizeof (xstate_hdr);
 	if (r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (void *)NT_X86_XSTATE, &local) < 0) {
-		perror ("NT_X86_XSTATE");
+		r_sys_perror ("NT_X86_XSTATE");
 		return 0;
 	}
 
