@@ -1,12 +1,8 @@
-/* radare - LGPL - Copyright 2007-2021 - pancake */
+/* radare - LGPL - Copyright 2007-2022 - pancake */
 
 #include <errno.h>
-#include <r_io.h>
 #include <r_lib.h>
-#include <r_util.h>
-#include <r_cons.h>
-#include <r_debug.h> /* only used for BSD PTRACE redefinitions */
-#include <string.h>
+#include <r_core.h>
 
 #if __linux__ ||  __APPLE__ || __WINDOWS__ || __NetBSD__ || __KFBSD__ || __OpenBSD__
 #define DEBUGGER_SUPPORTED 1
@@ -51,12 +47,6 @@
 #include <r_util/r_w32dw.h>
 #endif
 
-/*
- * Creates a new process and returns the result:
- * -1 : error
- *  0 : ok
- */
-
 #if __WINDOWS__
 typedef struct {
 	HANDLE hnd;
@@ -166,7 +156,7 @@ static int fork_and_ptraceme(RIO *io, int bits, const char *cmd) {
 
 	/* check if is a create process debug event */
 	if (de.dwDebugEventCode != CREATE_PROCESS_DEBUG_EVENT) {
-		eprintf ("exception code 0x%04x\n", (ut32)de.dwDebugEventCode);
+		R_LOG_ERROR ("exception code 0x%04x", (ut32)de.dwDebugEventCode);
 		goto err_fork;
 	}
 
@@ -176,11 +166,11 @@ static int fork_and_ptraceme(RIO *io, int bits, const char *cmd) {
 	wrap->pi.hThread = pi.hThread;
 	wrap->winbase = (ut64)de.u.CreateProcessInfo.lpBaseOfImage;
 
-	eprintf ("Spawned new process with pid %d, tid = %d\n", pid, tid);
+	R_LOG_INFO ("Spawned new process with pid %d, tid = %d", pid, tid);
 	return pid;
 
 err_fork:
-	eprintf ("Error: Cannot create new process.\n");
+	R_LOG_ERROR ("Cannot create new process");
 	TerminateProcess (pi.hProcess, 1);
 	r_w32dw_free (io->dbgwrap);
 	io->dbgwrap = NULL;
@@ -194,7 +184,7 @@ err_fork:
 
 #if __APPLE__ || __BSD__
 static void inferior_abort_handler(int pid) {
-	eprintf ("Inferior received signal SIGABRT. Executing BKPT.\n");
+	R_LOG_ERROR ("Inferior received signal SIGABRT. Executing BKPT");
 }
 #endif
 
@@ -223,17 +213,16 @@ static void trace_me(void) {
 #if __APPLE__ && !__POWERPC__
 static void handle_posix_error(int err) {
 	switch (err) {
-	case 0:
-		// eprintf ("Success\n");
+	case 0: // success, no error so far
 		break;
 	case 22:
-		eprintf ("posix_spawnp: Invalid argument\n");
+		R_LOG_ERROR ("posix_spawnp: Invalid argument");
 		break;
 	case 86:
-		eprintf ("Unsupported architecture. Please specify -b 32\n");
+		R_LOG_ERROR ("Unsupported architecture. Please specify -b 32");
 		break;
 	default:
-		eprintf ("posix_spawnp: unknown error %d\n", err);
+		R_LOG_ERROR ("posix_spawnp: unknown error %d", err);
 		r_sys_perror ("posix_spawnp");
 		break;
 	}
@@ -260,7 +249,7 @@ static RRunProfile* _get_run_profile(RIO *io, int bits, char **argv) {
 	rp->_dodebug = true;
 	if (io->runprofile && *io->runprofile) {
 		if (!r_run_parsefile (rp, io->runprofile)) {
-			eprintf ("Can't find profile '%s'\n", io->runprofile);
+			R_LOG_ERROR ("Can't find profile '%s'", io->runprofile);
 			r_run_free (rp);
 			return NULL;
 		}
@@ -269,19 +258,19 @@ static RRunProfile* _get_run_profile(RIO *io, int bits, char **argv) {
 		}
 	} else if (io->envprofile) {
 		if (!r_run_parse (rp, io->envprofile)) {
-			eprintf ("Can't parse default rarun2 profile\n");
+			R_LOG_ERROR ("Can't parse default rarun2 profile");
 			r_run_free (rp);
 			return NULL;
 		}
 	}
 	if (bits == 64) {
-		r_run_parseline (rp, expr=strdup ("bits=64"));
+		r_run_parseline (rp, expr = strdup ("bits=64"));
 	} else if (bits == 32) {
-		r_run_parseline (rp, expr=strdup ("bits=32"));
+		r_run_parseline (rp, expr = strdup ("bits=32"));
 	}
 	free (expr);
 	if (r_run_config_env (rp)) {
-		eprintf ("Can't config the environment.\n");
+		R_LOG_ERROR ("Cannot configure the environment");
 		r_run_free (rp);
 		return NULL;
 	}
@@ -414,7 +403,7 @@ static int fork_and_ptraceme_for_unix(RIO *io, int bits, const char *cmd) {
 		return -1;
 	}
 	if (WEXITSTATUS (status) == MAGIC_EXIT || r_cons_is_breaked ()) {
-		eprintf ("Killing child process %d due to an error\n", (int)child_pid);
+		R_LOG_INFO ("Killing child process %d due to an error", (int)child_pid);
 		kill (child_pid, SIGSTOP);
 		return -1;
 	}
@@ -438,18 +427,21 @@ static int fork_and_ptraceme(RIO *io, int bits, const char *cmd) {
 #endif
 
 static bool __plugin_open(RIO *io, const char *file, bool many) {
-	if (!strncmp (file, "waitfor://", 10)) {
+	if (r_str_startswith (file, "waitfor://")) {
 		return true;
 	}
-	if (!strncmp (file, "pidof://", 8)) {
+	if (r_str_startswith (file, "pidof://")) {
 		return true;
 	}
-	return (!strncmp (file, "dbg://", 6) && file[6]);
+	return r_str_startswith (file, "dbg://") && file[6];
 }
 
-#include <r_core.h>
 static int get_pid_of(RIO *io, const char *procname) {
-	RCore *c = io->corebind.core;
+	RCore *c = io->coreb.core;
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_EXEC)) {
+		return -1;
+	}
+	// check sandbox
 	if (c && c->dbg && c->dbg->h) {
 		RListIter *iter;
 		RDebugPid *proc;
@@ -457,12 +449,12 @@ static int get_pid_of(RIO *io, const char *procname) {
 		RList *pids = d->h->pids (d, 0);
 		r_list_foreach (pids, iter, proc) {
 			if (strstr (proc->path, procname)) {
-				eprintf ("Matching PID %d %s\n", proc->pid, proc->path);
+				R_LOG_INFO ("Matching PID %d %s", proc->pid, proc->path);
 				return proc->pid;
 			}
 		}
 	} else {
-		eprintf ("Cannot enumerate processes\n");
+		R_LOG_ERROR ("Cannot enumerate processes");
 	}
 	return -1;
 }
@@ -471,9 +463,12 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 	RIOPlugin *_plugin;
 	RIODesc *ret = NULL;
 	char uri[128];
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_EXEC)) {
+		return NULL;
+	}
 	if (!strncmp (file, "waitfor://", 10)) {
 		const char *procname = file + 10;
-		eprintf ("Waiting for %s\n", procname);
+		R_LOG_INFO ("Waiting for %s", procname);
 		while (true) {
 			int target_pid = get_pid_of (io, procname);
 			if (target_pid != -1) {
@@ -487,7 +482,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 		const char *procname = file + 8;
 		int target_pid = get_pid_of (io, procname);
 		if (target_pid == -1) {
-			eprintf ("Cannot find matching process for %s\n", file);
+			R_LOG_ERROR ("Cannot find matching process for %s", file);
 			return NULL;
 		}
 		snprintf (uri, sizeof (uri), "dbg://%d", target_pid);
@@ -512,7 +507,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 				return NULL;
 			}
 			if ((ret = _plugin->open (io, uri, rw, mode))) {
-				RCore *c = io->corebind.core;
+				RCore *c = io->coreb.core;
 				RW32Dw *wrap = (RW32Dw *)ret->data;
 				c->dbg->user = wrap;
 			}
@@ -541,7 +536,7 @@ static RIODesc *__open(RIO *io, const char *file, int rw, int mode) {
 			ret = _plugin->open (io, uri, rw, mode);
 #if __WINDOWS__
 			if (ret) {
-				RCore *c = io->corebind.core;
+				RCore *c = io->coreb.core;
 				RW32Dw *wrap = (RW32Dw *)ret->data;
 				c->dbg->user = wrap;
 			}
