@@ -1,6 +1,6 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nikolai */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nikolai */
 
-#include <r_diff.h>
+#include <r_util/r_diff.h>
 
 // the non-system-diff doesnt work well
 #define USE_SYSTEM_DIFF 1
@@ -13,7 +13,7 @@ R_API RDiff *r_diff_new_from(ut64 off_a, ut64 off_b) {
 		d->user = NULL;
 		d->off_a = off_a;
 		d->off_b = off_b;
-		d->diff_cmd = "diff -u";
+		d->diff_cmd = strdup ("diff -u");
 	}
 	return d;
 }
@@ -22,9 +22,11 @@ R_API RDiff *r_diff_new(void) {
 	return r_diff_new_from (0, 0);
 }
 
-R_API RDiff *r_diff_free(RDiff *d) {
-	free (d);
-	return NULL;
+R_API void r_diff_free(RDiff *d) {
+	if (d) {
+		free (d->diff_cmd);
+		free (d);
+	}
 }
 
 R_API int r_diff_set_callback(RDiff *d, RDiffCallback callback, void *user) {
@@ -223,7 +225,7 @@ R_API char *r_diff_buffers_to_string(RDiff *d, const ut8 *a, int la, const ut8 *
 }
 #endif
 
-#define diffHit(void) { \
+#define diffHit() { \
 	const size_t i_hit = i - hit; \
 	int ra = la - i_hit; \
 	int rb = lb - i_hit; \
@@ -261,29 +263,42 @@ R_API int r_diff_buffers_static(RDiff *d, const ut8 *a, int la, const ut8 *b, in
 	return 0;
 }
 
-// XXX: temporary files are
 R_API char *r_diff_buffers_unified(RDiff *d, const ut8 *a, int la, const ut8 *b, int lb) {
-	r_file_dump (".a", a, la, 0);
-	r_file_dump (".b", b, lb, 0);
+	char *fa = NULL;
+	char *fb = NULL;
+	int fd = r_file_mkstemp ("r_diff", &fa);
+	int fe = r_file_mkstemp ("r_diff", &fb);
+	if (fd == -1 || fe == -1) {
+		R_LOG_ERROR ("Failed to create temporary files");
+		return NULL;
+	}
+	if (!fa || !fb) {
+		R_LOG_ERROR ("fafb nul");
+		return NULL;
+	}
+	r_file_dump (fa, a, la, 0);
+	r_file_dump (fb, b, lb, 0);
 #if 0
 	if (r_mem_is_printable (a, R_MIN (5, la))) {
-		r_file_dump (".a", a, la, 0);
-		r_file_dump (".b", b, lb, 0);
+		r_file_dump (fa, a, la, 0);
+		r_file_dump (fb, b, lb, 0);
 	} else {
-		r_file_hexdump (".a", a, la, 0);
-		r_file_hexdump (".b", b, lb, 0);
+		r_file_hexdump (fa, a, la, 0);
+		r_file_hexdump (fb, b, lb, 0);
 	}
 #endif
 	char *err = NULL;
 	char *out = NULL;
 	int out_len;
-	char *diff_cmdline = r_str_newf ("%s .a .b", d->diff_cmd);
+	char *diff_cmdline = r_str_newf ("%s %s %s", d->diff_cmd, fa, fb);
 	if (diff_cmdline) {
 		(void)r_sys_cmd_str_full (diff_cmdline, NULL, 0, &out, &out_len, &err);
 		free (diff_cmdline);
 	}
-	r_file_rm (".a");
-	r_file_rm (".b");
+	close (fd);
+	close (fe);
+	r_file_rm (fa);
+	r_file_rm (fb);
 	free (err);
 	return out;
 }
@@ -330,7 +345,7 @@ R_API bool r_diff_buffers_distance_myers(RDiff *diff, const ut8 *a, ut32 la, con
 			}
 		}
 		if (verbose && di % 10000 == 0) {
-			eprintf ("\rProcessing dist %" PFMT64d " of max %" PFMT64d "\r", di, m);
+			eprintf ("\rProcessing dist %" PFMT64d " of max %" PFMT64d "\r", (st64)di, (st64)m);
 		}
 	}
 
@@ -467,9 +482,11 @@ R_API RDiffChar *r_diffchar_new(const ut8 *a, const ut8 *b) {
 			const ut8 a_ch = a[col - 1];
 			const ut8 b_ch = b[row - 1];
 			const st16 tl_score = *(align_table + (row - 1) * dim + col - 1)
-			                    + (a_ch == b_ch ?
-			                       (a_ch == '\n' ? match_nl : match) :
-			                       mismatch);
+				+ (a_ch == b_ch
+					? (a_ch == '\n'
+						? match_nl
+						: match)
+					: mismatch);
 			const st16 t_score = *(align_table + (row - 1) * dim + col) + gap;
 			const st16 l_score = *(align_table + row * dim + col - 1) + gap;
 			st16 score;
@@ -627,14 +644,16 @@ R_API void r_diffchar_print(RDiffChar *diffchar) {
 					printf ("%c", a_ch);
 				}
 			} else if (cur_align == R2R_ALIGN_BOTTOM_GAP) {
-				printf (a_ch == '\n' ?
-				        "%c"Color_HLDELETE :
-				        Color_HLDELETE"%c", a_ch);
+				printf (a_ch == '\n'
+						? "%c" Color_HLDELETE
+						: Color_HLDELETE "%c",
+					a_ch);
 				cur_mode = R2R_DIFF_DELETE;
 			} else if (cur_align == R2R_ALIGN_TOP_GAP) {
-				printf (b_ch == '\n' ?
-				        "%c"Color_HLINSERT :
-				        Color_HLINSERT"%c", b_ch);
+				printf (b_ch == '\n'
+						? "%c" Color_HLINSERT
+						: Color_HLINSERT "%c",
+					b_ch);
 				cur_mode = R2R_DIFF_INSERT;
 			}
 		} else if (cur_mode == R2R_DIFF_DELETE) {
@@ -645,13 +664,15 @@ R_API void r_diffchar_print(RDiffChar *diffchar) {
 				}
 				cur_mode = R2R_DIFF_MATCH;
 			} else if (cur_align == R2R_ALIGN_BOTTOM_GAP) {
-				printf (a_ch == '\n' ?
-				        Color_RESET"%c"Color_HLDELETE :
-				        "%c", a_ch);
+				printf (a_ch == '\n'
+						? Color_RESET "%c" Color_HLDELETE
+						: "%c",
+					a_ch);
 			} else if (cur_align == R2R_ALIGN_TOP_GAP) {
-				printf (b_ch == '\n' ?
-				        Color_RESET"%c"Color_HLINSERT :
-				        Color_HLINSERT"%c", b_ch);
+				printf (b_ch == '\n'
+						? Color_RESET "%c" Color_HLINSERT
+						: Color_HLINSERT "%c",
+					b_ch);
 				cur_mode = R2R_DIFF_INSERT;
 			}
 		} else if (cur_mode == R2R_DIFF_INSERT) {
@@ -662,19 +683,21 @@ R_API void r_diffchar_print(RDiffChar *diffchar) {
 				}
 				cur_mode = R2R_DIFF_MATCH;
 			} else if (cur_align == R2R_ALIGN_BOTTOM_GAP) {
-				printf (a_ch == '\n' ?
-				        Color_RESET"%c"Color_HLDELETE :
-				        Color_HLDELETE"%c", a_ch);
+				printf (a_ch == '\n'
+						? Color_RESET "%c" Color_HLDELETE
+						: Color_HLDELETE "%c",
+					a_ch);
 				cur_mode = R2R_DIFF_DELETE;
 			} else if (cur_align == R2R_ALIGN_TOP_GAP) {
-				printf (b_ch == '\n' ?
-				        Color_RESET"%c"Color_HLINSERT :
-				        "%c", b_ch);
+				printf (b_ch == '\n'
+						? Color_RESET "%c" Color_HLINSERT
+						: "%c",
+					b_ch);
 			}
 		}
 		idx_align++;
 	}
-	printf (Color_RESET"\n");
+	printf (Color_RESET "\n");
 }
 
 R_API void r_diffchar_free(RDiffChar *diffchar) {
@@ -711,6 +734,7 @@ static st32 r_diff_levenshtein_nopath(RLevBuf *bufa, RLevBuf *bufb, ut32 maxdst,
 
 	ut32 ldelta = blen - alen;
 	if (ldelta > maxdst) {
+		lev_matrix_free (matrix, alen + 1);
 		return ST32_MAX;
 	}
 
